Package: BioNERO
Type: Package
Title: Biological Network Reconstruction Omnibus
Version: 1.21.0
Authors@R: c(person("Fabricio", "Almeida-Silva", 
                    email="fabricio_almeidasilva@hotmail.com", 
                    role=c("cre", "aut"), 
                    comment = c(ORCID = "0000-0002-5314-2964")),
            person("Thiago", "Venancio", 
            role="aut", 
            comment = c(ORCID = "0000-0002-2215-8082")))
Description: BioNERO aims to integrate all aspects of biological network inference
        in a single package, including data preprocessing, exploratory analyses,
        network inference, and analyses for biological interpretations. BioNERO
        can be used to infer gene coexpression networks (GCNs) and gene 
        regulatory networks (GRNs) from gene expression data. Additionally,
        it can be used to explore topological properties of protein-protein 
        interaction (PPI) networks. GCN inference relies on the popular WGCNA 
        algorithm. GRN inference is based on the "wisdom of the crowds" principle,
        which consists in inferring GRNs with multiple algorithms (here, CLR,
        GENIE3 and ARACNE) and calculating the average rank for each interaction
        pair. As all steps of network analyses are included in this package,
        BioNERO makes users avoid having to learn the syntaxes of several 
        packages and how to communicate between them. Finally, users can also 
        identify consensus modules across independent expression sets and
        calculate intra and interspecies module preservation statistics 
        between different networks.
Depends: R (>= 4.1)
License: GPL-3
Encoding: UTF-8
LazyData: false
URL: https://github.com/almeidasilvaf/BioNERO
BugReports: https://github.com/almeidasilvaf/BioNERO/issues
biocViews: Software, GeneExpression, GeneRegulation, SystemsBiology,
        GraphAndNetwork, Preprocessing, Network, NetworkInference
Imports: WGCNA, dynamicTreeCut, ggdendro, matrixStats, sva,
        RColorBrewer, ComplexHeatmap, ggplot2, rlang, ggrepel,
        patchwork, reshape2, igraph, ggnetwork, intergraph, NetRep,
        stats, grDevices, utils, methods, BiocParallel, minet, GENIE3,
        SummarizedExperiment
RoxygenNote: 7.3.2
Suggests: knitr, rmarkdown, testthat (>= 3.0.0), BiocStyle, DESeq2,
        networkD3, covr
VignetteBuilder: knitr
Config/testthat/edition: 3
git_url: https://git.bioconductor.org/packages/BioNERO
git_branch: devel
git_last_commit: 1351094
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 21:35:59 UTC; biocbuild
Author: Fabricio Almeida-Silva [cre, aut] (ORCID:
    <https://orcid.org/0000-0002-5314-2964>),
  Thiago Venancio [aut] (ORCID: <https://orcid.org/0000-0002-2215-8082>)
Maintainer: Fabricio Almeida-Silva <fabricio_almeidasilva@hotmail.com>

# Generated by roxygen2: do not edit by hand

export(PC_correction)
export(SFT_fit)
export(ZKfiltering)
export(check_SFT)
export(consensus_SFT_fit)
export(consensus_modules)
export(consensus_trait_cor)
export(cor2adj)
export(cormat_to_edgelist)
export(detect_communities)
export(dfs2one)
export(enrichment_analysis)
export(exp2cor)
export(exp2gcn)
export(exp2gcn_blockwise)
export(exp2grn)
export(exp_genes2orthogroups)
export(exp_preprocess)
export(filter_by_variance)
export(gene_significance)
export(get_HK)
export(get_edge_list)
export(get_hubs_gcn)
export(get_hubs_grn)
export(get_hubs_ppi)
export(get_neighbors)
export(grn_average_rank)
export(grn_combined)
export(grn_filter)
export(grn_infer)
export(is_singleton)
export(modPres_WGCNA)
export(modPres_netrep)
export(module_enrichment)
export(module_preservation)
export(module_stability)
export(module_trait_cor)
export(net_stats)
export(parse_orthofinder)
export(plot_PCA)
export(plot_dendro_and_colors)
export(plot_eigengene_network)
export(plot_expression_profile)
export(plot_gcn)
export(plot_gene_significance)
export(plot_grn)
export(plot_heatmap)
export(plot_module_trait_cor)
export(plot_ngenes_per_module)
export(plot_ppi)
export(q_normalize)
export(remove_nonexp)
export(replace_na)
import(intergraph)
importFrom(BiocParallel,SerialParam)
importFrom(BiocParallel,bplapply)
importFrom(ComplexHeatmap,pheatmap)
importFrom(GENIE3,GENIE3)
importFrom(NetRep,modulePreservation)
importFrom(RColorBrewer,brewer.pal)
importFrom(SummarizedExperiment,SummarizedExperiment)
importFrom(SummarizedExperiment,assay)
importFrom(SummarizedExperiment,colData)
importFrom(SummarizedExperiment,rowData)
importFrom(WGCNA,TOMsimilarity)
importFrom(WGCNA,adjacency)
importFrom(WGCNA,adjacency.fromSimilarity)
importFrom(WGCNA,bicor)
importFrom(WGCNA,blockwiseModules)
importFrom(WGCNA,checkSets)
importFrom(WGCNA,consensusMEDissimilarity)
importFrom(WGCNA,corPvalueFisher)
importFrom(WGCNA,corPvalueStudent)
importFrom(WGCNA,fundamentalNetworkConcepts)
importFrom(WGCNA,intramodularConnectivity)
importFrom(WGCNA,labels2colors)
importFrom(WGCNA,matchLabels)
importFrom(WGCNA,mergeCloseModules)
importFrom(WGCNA,moduleEigengenes)
importFrom(WGCNA,multiSetMEs)
importFrom(WGCNA,pickSoftThreshold)
importFrom(WGCNA,pquantile)
importFrom(WGCNA,sampledBlockwiseModules)
importFrom(WGCNA,scaleFreeFitIndex)
importFrom(WGCNA,signedKME)
importFrom(WGCNA,standardColors)
importFrom(dynamicTreeCut,cutreeDynamic)
importFrom(dynamicTreeCut,cutreeDynamicTree)
importFrom(ggdendro,dendro_data)
importFrom(ggnetwork,geom_edges)
importFrom(ggnetwork,geom_nodelabel_repel)
importFrom(ggnetwork,geom_nodes)
importFrom(ggnetwork,geom_nodetext)
importFrom(ggnetwork,ggnetwork)
importFrom(ggnetwork,theme_blank)
importFrom(ggnetwork,unit)
importFrom(ggplot2,aes)
importFrom(ggplot2,arrow)
importFrom(ggplot2,coord_cartesian)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_rect)
importFrom(ggplot2,element_text)
importFrom(ggplot2,geom_col)
importFrom(ggplot2,geom_hline)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_segment)
importFrom(ggplot2,geom_text)
importFrom(ggplot2,geom_tile)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggtitle)
importFrom(ggplot2,guides)
importFrom(ggplot2,labs)
importFrom(ggplot2,scale_color_manual)
importFrom(ggplot2,scale_fill_identity)
importFrom(ggplot2,scale_fill_manual)
importFrom(ggplot2,scale_x_continuous)
importFrom(ggplot2,scale_y_discrete)
importFrom(ggplot2,stat_summary)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_bw)
importFrom(ggplot2,theme_classic)
importFrom(ggplot2,theme_minimal)
importFrom(ggplot2,xlim)
importFrom(ggplot2,ylim)
importFrom(ggrepel,geom_text_repel)
importFrom(grDevices,colorRampPalette)
importFrom(igraph,V)
importFrom(igraph,as_adjacency_matrix)
importFrom(igraph,betweenness)
importFrom(igraph,centr_degree)
importFrom(igraph,cliques)
importFrom(igraph,closeness)
importFrom(igraph,cluster_edge_betweenness)
importFrom(igraph,cluster_fast_greedy)
importFrom(igraph,cluster_infomap)
importFrom(igraph,cluster_label_prop)
importFrom(igraph,cluster_leading_eigen)
importFrom(igraph,cluster_louvain)
importFrom(igraph,cluster_spinglass)
importFrom(igraph,cluster_walktrap)
importFrom(igraph,degree)
importFrom(igraph,diameter)
importFrom(igraph,edge_density)
importFrom(igraph,fit_power_law)
importFrom(igraph,graph_from_adjacency_matrix)
importFrom(igraph,graph_from_data_frame)
importFrom(igraph,simplify)
importFrom(igraph,transitivity)
importFrom(igraph,vcount)
importFrom(matrixStats,rowMedians)
importFrom(methods,is)
importFrom(minet,aracne)
importFrom(minet,build.mim)
importFrom(minet,clr)
importFrom(patchwork,wrap_plots)
importFrom(reshape2,dcast)
importFrom(reshape2,melt)
importFrom(rlang,.data)
importFrom(stats,aggregate)
importFrom(stats,as.dist)
importFrom(stats,as.formula)
importFrom(stats,cor)
importFrom(stats,fisher.test)
importFrom(stats,hclust)
importFrom(stats,median)
importFrom(stats,model.matrix)
importFrom(stats,na.omit)
importFrom(stats,p.adjust)
importFrom(stats,phyper)
importFrom(stats,prcomp)
importFrom(stats,qnorm)
importFrom(stats,qqplot)
importFrom(stats,quantile)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stats,var)
importFrom(sva,num.sv)
importFrom(sva,sva_network)
importFrom(utils,head)
importFrom(utils,read.csv)
importFrom(utils,write.table)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# BioNERO <img src='man/figures/logo.png' align="right" height="139" />

<!-- badges: start -->

[![GitHub
issues](https://img.shields.io/github/issues/almeidasilvaf/BioNERO)](https://github.com/almeidasilvaf/BioNERO/issues)
[![Lifecycle:
stable](https://img.shields.io/badge/lifecycle-stable-brightgreen.svg)](https://lifecycle.r-lib.org/articles/stages.html#stable)
[![R-CMD-check-bioc](https://github.com/almeidasilvaf/BioNERO/workflows/R-CMD-check-bioc/badge.svg)](https://github.com/almeidasilvaf/BioNERO/actions)
[![Codecov test
coverage](https://codecov.io/gh/almeidasilvaf/BioNERO/branch/devel/graph/badge.svg)](https://codecov.io/gh/almeidasilvaf/BioNERO?branch=devel)
<!-- badges: end -->

`BioNERO` aims to integrate all aspects of biological network inference
in a single package, so users don’t have to learn the syntaxes of
several packages and how to communicate among them. `BioNERO` features:

- **Expression data preprocessing** using state-of-the-art techniques
  for network inference.
- **Automated exploratory data analyses**, including principal component
  analysis (PCA) and heatmaps of gene expression or sample correlations.
- **Inference of gene coexpression networks (GCNs)** using the popular
  WGCNA algorithm.
- **Inference of gene regulatory networks (GRNs)** based on the “wisdom
  of the crowds” principle. This principle consists in inferring GRNs
  with multiple algorithms (here, CLR, GENIE3 and ARACNE) and
  calculating the average rank for each interaction pair.
- **Exploration of network topology** of GCNs, GRNs, and protein-protein
  interaction networks.
- **Network visualization**.
- **Network comparison**, including identification of consensus modules
  across independent expression sets, and calculation of intra and
  interspecies module preservation statistics between different
  networks.

## Installation instructions

Get the latest stable `R` release from
[CRAN](http://cran.r-project.org/). Then install `BioNERO` from
[Bioconductor](http://bioconductor.org/) using the following code:

``` r
if (!requireNamespace("BiocManager", quietly = TRUE)) {
    install.packages("BiocManager")
}
BiocManager::install("BioNERO")
```

And the development version from
[GitHub](https://github.com/almeidasilvaf/BioNERO) with:

``` r
BiocManager::install("almeidasilvaf/BioNERO")
```

## Citation

Below is the citation output from using `citation('BioNERO')` in R.
Please run this yourself to check for any updates on how to cite
**BioNERO**.

``` r
print(citation('BioNERO'), bibtex = TRUE)
# 
# To cite BioNERO in publications use:
# 
#   Almeida-Silva, F., Venancio, T.M. BioNERO: an all-in-one
#   R/Bioconductor package for comprehensive and easy biological network
#   reconstruction. Funct Integr Genomics 22, 131-136 (2022).
#   https://doi.org/10.1007/s10142-021-00821-9
# 
# A BibTeX entry for LaTeX users is
# 
#   @Article{,
#     title = {BioNERO: an all-in-one R/Bioconductor package for comprehensive and easy biological network reconstruction},
#     author = {Fabricio Almeida-Silva and Thiago M. Venancio},
#     journal = {Functional And Integrative Genomics},
#     year = {2022},
#     volume = {22},
#     number = {1},
#     pages = {131-136},
#     url = {https://link.springer.com/article/10.1007/s10142-021-00821-9},
#     doi = {10.1007/s10142-021-00821-9},
#   }
```

Please note that the `BioNERO` was only made possible thanks to many
other R and bioinformatics software authors, which are cited either in
the vignettes and/or the paper(s) describing this package.

The data set "Orthogroups.tsv" was downloaded from OrthoFinder's example results.

```{bash eval=FALSE}
wget https://bioinformatics.plants.ox.ac.uk/davidemms/public_data/Results_model_species.tar.gz
tar -zxvf Results_model_species.tar.gz
head -n 20 Results_Nov26/Orthogroups/Orthogroups.tsv > Orthogroups.tsv
```

## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  message = TRUE,
  warning = FALSE,
  cache = FALSE,
  fig.align = 'center',
  fig.width = 5,
  fig.height = 4,
  crop = NULL
)

## ----installation, eval=FALSE-------------------------------------------------
# if(!requireNamespace('BiocManager', quietly = TRUE))
#   install.packages('BiocManager')
# 
# BiocManager::install("BioNERO")

## ----load_package-------------------------------------------------------------
# Load package after installation
library(BioNERO)
set.seed(123) # for reproducibility

## ----data_loading_soybean, message=FALSE--------------------------------------
data(zma.se)

# Take a quick look at the data
zma.se
SummarizedExperiment::colData(zma.se)

## ----remove_na----------------------------------------------------------------
exp_filt <- replace_na(zma.se)
sum(is.na(zma.se))

## ----remove_nonexp------------------------------------------------------------
exp_filt <- remove_nonexp(exp_filt, method = "median", min_exp = 10)
dim(exp_filt)

## ----filter_by_variance-------------------------------------------------------
exp_filt <- filter_by_variance(exp_filt, n = 2000)
dim(exp_filt)

## ----ZKfiltering--------------------------------------------------------------
exp_filt <- ZKfiltering(exp_filt, cor_method = "pearson")
dim(exp_filt)

## ----PC_correction------------------------------------------------------------
exp_filt <- PC_correction(exp_filt)

## ----exp_preprocess-----------------------------------------------------------
final_exp <- exp_preprocess(
    zma.se, min_exp = 10, variance_filter = TRUE, n = 2000
)
identical(dim(exp_filt), dim(final_exp))

# Take a look at the final data
final_exp

## ----include=FALSE------------------------------------------------------------
# This object is no longer necessary
rm(exp_filt)

## ----plot_heatmap, fig.width=6, fig.height=4, message=FALSE-------------------
# Heatmap of sample correlations
p <- plot_heatmap(final_exp, type = "samplecor", show_rownames = FALSE)
p

# Heatmap of gene expression (here, only the first 50 genes)
p <- plot_heatmap(
    final_exp[1:50, ], type = "expr", show_rownames = FALSE, show_colnames = FALSE
)
p

## ----pcaplot, fig.small=TRUE--------------------------------------------------
plot_PCA(final_exp)

## ----sft_fit------------------------------------------------------------------
sft <- SFT_fit(final_exp, net_type = "signed hybrid", cor_method = "pearson")
sft$power
power <- sft$power

## ----plot_sft, fig.width=8----------------------------------------------------
sft$plot

## ----exp2net------------------------------------------------------------------
net <- exp2gcn(
    final_exp, net_type = "signed hybrid", SFTpower = power, 
    cor_method = "pearson"
)
names(net)

## ----dendro-------------------------------------------------------------------
# Dendro and colors
plot_dendro_and_colors(net)

## ----eigengene_net, fig.height=4, fig.width=6---------------------------------
# Eigengene networks
plot_eigengene_network(net)

## ----genes_per_module, fig.wide=TRUE------------------------------------------
plot_ngenes_per_module(net)

## ----module_stability---------------------------------------------------------
module_stability(final_exp, net, nRuns = 5)

## ----moddtraitcor-------------------------------------------------------------
MEtrait <- module_trait_cor(exp = final_exp, MEs = net$MEs)
head(MEtrait)

## ----plot-module-trait-cor, fig.width = 5, fig.height = 5---------------------
plot_module_trait_cor(MEtrait)

## ----exp_profile, fig.width=5, fig.height=3-----------------------------------
plot_expression_profile(
    exp = final_exp, 
    net = net, 
    plot_module = TRUE, 
    modulename = "yellow"
)

## ----module_enrichment--------------------------------------------------------
# Enrichment analysis for conserved protein domains (Interpro)
data(zma.interpro)
interpro_enrichment <- module_enrichment(
    net = net, 
    background_genes = rownames(final_exp),
    annotation = zma.interpro
)

# Print results without geneIDs for better visualization
interpro_enrichment[, -6]

## -----------------------------------------------------------------------------
hubs <- get_hubs_gcn(final_exp, net)
head(hubs)

## -----------------------------------------------------------------------------
edges <- get_edge_list(net, module="midnightblue")
head(edges)

## ----filter_edges, fig.small=TRUE---------------------------------------------
# Remove edges based on optimal scale-free topology fit
edges_filtered <- get_edge_list(net, module = "midnightblue", filter = TRUE)
dim(edges_filtered)

# Remove edges based on p-value
edges_filtered <- get_edge_list(
    net, module = "midnightblue",
    filter = TRUE, method = "pvalue", 
    nSamples = ncol(final_exp)
)
dim(edges_filtered)

# Remove edges based on minimum correlation
edges_filtered <- get_edge_list(
    net, module = "midnightblue", 
    filter = TRUE, method = "min_cor", rcutoff = 0.7
)
dim(edges_filtered)

## ----plot_gcn, fig.width=5, fig.height=5--------------------------------------
plot_gcn(
    edgelist_gcn = edges_filtered, 
    net = net, 
    color_by = "module", 
    hubs = hubs
)

## ----interactive_gcn----------------------------------------------------------
plot_gcn(
    edgelist_gcn = edges_filtered, 
    net = net,
    color_by = "module",
    hubs = hubs,
    interactive = TRUE,
    dim_interactive = c(500, 500)
)

## -----------------------------------------------------------------------------
sessionInfo()

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  message = TRUE,
  warning = FALSE,
  cache = FALSE,
  fig.align = 'center',
  fig.width = 5,
  fig.height = 4,
  crop = NULL
)

## ----installation, eval=FALSE-------------------------------------------------
# if(!requireNamespace('BiocManager', quietly = TRUE))
#   install.packages('BiocManager')
# 
# BiocManager::install("BioNERO")

## ----load_package-------------------------------------------------------------
# Load package after installation
library(BioNERO)
set.seed(12) # for reproducibility

## ----inspect_data-------------------------------------------------------------
data(zma.se)
zma.se

data(osa.se)
osa.se

## ----create_list_consensus----------------------------------------------------
# Preprocess data and keep top 2000 genes with highest variances
filt_zma <- exp_preprocess(zma.se, variance_filter = TRUE, n = 2000)

# Create different subsets by resampling data
zma_set1 <- filt_zma[, sample(colnames(filt_zma), size=22, replace=FALSE)]
zma_set2 <- filt_zma[, sample(colnames(filt_zma), size=22, replace=FALSE)]
colnames(zma_set1)
colnames(zma_set2)

# Create list
zma_list <- list(set1 = zma_set1, set2 = zma_set2)
length(zma_list)

## ----consensus_sft, message=FALSE---------------------------------------------
cons_sft <- consensus_SFT_fit(zma_list, setLabels = c("Maize 1", "Maize 2"),
                              cor_method = "pearson")

## ----sft_results, fig.width=8-------------------------------------------------
powers <- cons_sft$power
powers
cons_sft$plot

## ----consensus_modules_identification-----------------------------------------
consensus <- consensus_modules(zma_list, power = powers, cor_method = "pearson")
names(consensus)
head(consensus$genes_cmodules)

## ----consensus_trait_cor, fig.width=5, fig.height=5---------------------------
consensus_trait <- consensus_trait_cor(consensus, cor_method = "pearson")
head(consensus_trait)

## ----plot-consensus-trait-cor, fig.width = 5, fig.height = 6------------------
plot_module_trait_cor(consensus_trait)

## ----load_orthogroups---------------------------------------------------------
data(og.zma.osa)
head(og.zma.osa)

## ----genes2orthogorups--------------------------------------------------------
# Store SummarizedExperiment objects in a list
zma_osa_list <- list(osa = osa.se, zma = zma.se)

# Collapse gene-level expression to orthogroup-level
ortho_exp <- exp_genes2orthogroups(zma_osa_list, og.zma.osa, summarize = "mean")

# Inspect new expression data
ortho_exp$osa[1:5, 1:5]
ortho_exp$zma[1:5, 1:5]

## ----create_list_preservation-------------------------------------------------
# Preprocess data and keep top 1000 genes with highest variances
ortho_exp <- lapply(ortho_exp, exp_preprocess, variance_filter=TRUE, n=1000)

# Check orthogroup number
sapply(ortho_exp, nrow)

## ----gcn_inference------------------------------------------------------------
# Calculate SFT power
power_ortho <- lapply(ortho_exp, SFT_fit, cor_method="pearson")

# Infer GCNs
gcns <- lapply(seq_along(power_ortho), function(n) 
  exp2gcn(ortho_exp[[n]], SFTpower = power_ortho[[n]]$power, 
          cor_method = "pearson")
  )

length(gcns)

## ----preservation-------------------------------------------------------------
# Using rice as reference and maize as test
pres <- module_preservation(ortho_exp, 
                            ref_net = gcns[[1]], 
                            test_net = gcns[[2]], 
                            algorithm = "netrep")

## ----singleton----------------------------------------------------------------
# Sample 50 random genes
genes <- sample(rownames(zma.se), size = 50)
is_singleton(genes, og.zma.osa)

## ----sessionInfo, echo=FALSE--------------------------------------------------
sessionInfo()

#' Detect communities in a network
#'
#' @param edgelist Data frame containing the network as an edge list.
#' First column must be node 1 and second column must be node 2.
#' Additional columns will be interpreted as edge attributes and will
#' be modified by this function.
#' @param method igraph function to be used for community detection.
#' Available functions are cluster_infomap, cluster_edge_betweenness,
#' cluster_fast_greedy, cluster_walktrap, cluster_spinglass,
#' cluster_leading_eigen, cluster_louvain, and cluster_label_prop.
#' Default is cluster_infomap.
#' @param directed Logical indicating whether the network is directed (GRN only)
#' or not (GCN and PPI networks). Default: TRUE.
#' @return A data frame containing node names in the first column, and
#' communities to which nodes belong in the second column.
#' @seealso
#'  \code{\link[igraph]{cluster_infomap}},
#'  \code{\link[igraph]{cluster_edge_betweenness}},
#'  \code{\link[igraph]{cluster_fast_greedy}},
#'  \code{\link[igraph]{cluster_walktrap}},
#'  \code{\link[igraph]{cluster_spinglass}},
#'  \code{\link[igraph]{cluster_leading_eigen}},
#'  \code{\link[igraph]{cluster_louvain}},
#'  \code{\link[igraph]{cluster_label_prop}}
#' @rdname detect_communities
#' @author Fabricio Almeida-Silva
#' @export
#' @importFrom igraph graph_from_data_frame simplify cluster_infomap
#' cluster_edge_betweenness cluster_fast_greedy
#' cluster_walktrap cluster_spinglass cluster_leading_eigen
#' cluster_louvain cluster_label_prop
#' @examples
#' data(filt.se)
#' tfs <- sample(rownames(filt.se), size=50, replace=FALSE)
#' grn_edges <- grn_infer(filt.se, method = "clr", regulators = tfs)
#' com <- detect_communities(grn_edges, directed=TRUE)
detect_communities <- function(edgelist,
                               method = igraph::cluster_infomap,
                               directed = TRUE) {
    graph <- igraph::graph_from_data_frame(edgelist, directed = directed)
    graph <- igraph::simplify(graph)
    com <- method(graph, modularity = FALSE)
    df_com <- as.data.frame(list(names = com$names, mem = com$membership))
    return(df_com)
}


#' Plot protein-protein interaction network from edge list
#'
#' @param edgelist_int Data frame containing the edge list for the PPI network.
#' First column is the protein 1 and second column is the protein 2.
#' All other columns are interpreted as edge attributes.
#' @param color_by How should nodes be colored? It must be either "community" or
#' a 2-column data frame containing proteins in the first column and a
#' custom annotation in the second column. If "community", a clustering
#' algorithm will be applied. Default: "community".
#' @param clustering_method igraph function to be used for community detection.
#' Available functions are cluster_infomap, cluster_edge_betweenness,
#' cluster_fast_greedy, cluster_walktrap, cluster_spinglass,
#' cluster_leading_eigen, cluster_louvain, and cluster_label_prop.
#' Default is cluster_infomap.
#' @param show_labels Character indicating which nodes will be labeled.
#' One of "all", "allhubs", "tophubs", or "none".
#' @param top_n_hubs Number of top hubs to be labeled. It is only valid
#' if \code{show_labels} equals "tophubs". Default is 5.
#' @param add_color_legend Logical indicating whether to add a color legend
#' for nodes. Default: TRUE.
#' @param curvature Numeric indicating the amount of curvature in edges.
#' Negative values produce left-hand curves, positive values produce right-hand
#' curves, and zero produces a straight line. Default: 0.
#' @param interactive Logical indicating whether the network should be
#' interactive or not. Default is FALSE.
#' @param dim_interactive Numeric vector with width and height of window
#' for interactive plotting. Default: c(600,600).
#'
#' @return A ggplot object.
#' @rdname plot_ppi
#' @author Fabricio Almeida-Silva
#' @export
#' @importFrom igraph graph_from_data_frame degree simplify vcount
#' @importFrom ggnetwork ggnetwork geom_edges geom_nodes geom_nodetext
#' theme_blank geom_nodelabel_repel unit
#' @importFrom ggplot2 ggplot aes guides
#' @import intergraph
#' @examples
#' ppi_edges <- igraph::sample_pa(n = 500)
#' ppi_edges <- igraph::as_edgelist(ppi_edges)
#' p <- plot_ppi(ppi_edges, add_color_legend = FALSE)
plot_ppi <- function(edgelist_int, color_by = "community",
                     clustering_method = igraph::cluster_infomap,
                     show_labels = "tophubs",
                     top_n_hubs = 5,
                     add_color_legend=TRUE, curvature = 0,
                     interactive = FALSE, dim_interactive = c(600, 600)) {

    requireNamespace("intergraph", quietly=TRUE)
    # How should genes be colored?
    if(is.data.frame(color_by)) {
        prot_annotation <- color_by # color by custom annotation
    } else {
        prot_annotation <- detect_communities(edgelist_int,
                                              method = clustering_method,
                                              directed = FALSE)
    }

    # Get degree of nodes and find hubs
    h <- get_hubs_ppi(edgelist_int, return_degree = TRUE)

    # Create a data frame of nodes and node attributes
    protIDs <- unique(c(as.character(edgelist_int[,1]), as.character(edgelist_int[,2])))
    nod_at <- data.frame(Protein = protIDs)
    nod_at$Class <- as.factor(prot_annotation[prot_annotation[,1] %in% nod_at$Protein, 2])
    nod_at$Degree <- h$Degree$Degree[h$Degree$Protein %in% nod_at$Protein]
    nod_at$isHub <- ifelse(nod_at$Protein %in% h$Hubs$Protein, TRUE, FALSE)
    nod_at <- nod_at[order(nod_at$Class, -nod_at$Degree), ]

    # Should the network be interactive?
    if(interactive) {

        if(!requireNamespace("networkD3", quietly = TRUE)) {
            stop("`interactive = TRUE` requires the 'networkD3' package.")
        }

        graph <- igraph::graph_from_data_frame(d = edgelist_int,
                                               vertices = nod_at, directed=FALSE)
        graph <- igraph::simplify(graph)
        graph_d3 <- networkD3::igraph_to_networkD3(graph, group = nod_at$Class)
        graph_d3$nodes <- merge(
            graph_d3$nodes, nod_at, by.x = "name", by.y = "Protein", sort = FALSE
        )
        d <- dim_interactive
        p <- networkD3::forceNetwork(
            Links = graph_d3$links, Nodes = graph_d3$nodes,
            Source = 'source', Target = 'target',
            NodeID = 'name', Group = 'group',
            Nodesize = 'Degree', height = d[2], width = d[1],
            opacity=0.8, zoom = TRUE, fontSize = 20
        )

    } else { #Static network
        # Define plotting parameters
        if(nlevels(nod_at$Class) <= 20) {
            palette <- custom_palette(1)
        } else {
            n <- nlevels(nod_at$Class)
            palette <- colorRampPalette(custom_palette(1))(n)
        }

        # Handle legend
        if(add_color_legend) {
            set_legend <- NULL
        } else {
            set_legend <- ggplot2::guides(color = "none")
        }

        # Handle node labeling based on which labels to display (top hubs, hubs or all genes)
        if(show_labels == "all") {
            nod_at$Degree2 <- nod_at$Degree * 0.4
            add_nodelabel <- geom_nodetext(
                aes(label = .data$name, size = .data$Degree2),
                show.legend = FALSE
            )
        } else if(show_labels == "allhubs") {
            add_nodelabel <- geom_nodelabel_repel(
                aes(label = .data$name), color = "azure4",
                box.padding = ggnetwork::unit(1, "lines"),
                data = function(x) { x[ x$isHub, ]},
                show.legend = FALSE, max.overlaps = Inf
            )
        } else if(show_labels == "tophubs") {
            tophubs <- nod_at[nod_at$isHub == TRUE, 1][seq_len(top_n_hubs)]
            nod_at$isTopHub <- ifelse(nod_at$Protein %in% tophubs, TRUE, FALSE)
            add_nodelabel <- geom_nodelabel_repel(
                aes(label = .data$name), color = "azure4",
                box.padding = ggnetwork::unit(1, "lines"),
                data = function(x) { x[ x$isTopHub, ]},
                show.legend = FALSE, max.overlaps = Inf
            )

        } else if(show_labels == "none") {
            add_nodelabel <- NULL
        } else {
            stop("Please, specify a valid option for 'show_labels'.")
        }

        # Create graph object
        graph <- simplify(graph_from_data_frame(
            d = edgelist_int, vertices = nod_at, directed = FALSE
        ))
        n <- ggnetwork::ggnetwork(graph, arrow.gap = 0)

        # Plot graph
        p <- ggplot(
            n, aes(x = .data$x, y = .data$y, xend = .data$xend, yend = .data$yend)
        ) +
            geom_edges(
                color = "grey75", alpha = 0.5, curvature = curvature,
                show.legend = FALSE
            ) +
            geom_nodes(aes(size = .data$Degree, color = .data$Class)) +
            set_legend +
            scale_color_manual(values = palette) +
            add_nodelabel +
            ggnetwork::theme_blank()
    }

    return(p)
}



#' Plot gene regulatory network from edge list
#'
#' @param edgelist_grn Data frame containing the edge list for the GRN network.
#' First column is the TF and second column is the target gene.
#' All other columns are interpreted as edge attributes.
#' @param show_labels Character indicating which nodes will be labeled.
#' One of "all", "allhubs", "tophubs", or "none".
#' @param top_n_hubs Number of top hubs to be labeled. It is only valid
#' if \code{show_labels} equals "tophubs". Default is 5.
#' @param interactive Logical indicating whether the network should be
#' interactive or not. Default is FALSE.
#' @param layout igraph function for the network layout. One of
#' with_dh, with_drl, with_gem, with_lgl, with_fr, with_graphopt,
#' with_kk and with_mds. Default is with_kk.
#' @param arrow.gap Numeric indicating the distance between nodes and arrows.
#' Default is 0.2.
#' @param ranked Logical indicating whether to treat third column of
#' the edge list (edge weights) as ranked values. Default: TRUE.
#' @param curvature Numeric indicating the amount of curvature in edges.
#' Negative values produce left-hand curves, positive values produce right-hand
#' curves, and zero produces a straight line. Default: 0.1.
#' @param dim_interactive Numeric vector with width and height of window
#' for interactive plotting. Default: c(600,600).
#'
#' @return A ggplot object containing the network.
#' @rdname plot_grn
#' @author Fabricio Almeida-Silva
#' @export
#' @import intergraph
#' @importFrom igraph graph_from_data_frame degree vcount
#' @importFrom ggplot2 ggplot aes arrow scale_color_manual guides
#' @importFrom ggnetwork geom_edges unit geom_nodes geom_nodetext theme_blank geom_nodelabel_repel
#' @examples
#' data(filt.se)
#' tfs <- sample(rownames(filt.se), size = 50, replace = FALSE)
#' grn_edges <- grn_infer(filt.se, method = "clr", regulators = tfs)
#' p <- plot_grn(grn_edges, ranked = FALSE)
plot_grn <- function(edgelist_grn, show_labels = "tophubs", top_n_hubs = 5,
                     layout = igraph::with_kk, arrow.gap = 0.01,
                     ranked = TRUE, curvature = 0.1,
                     interactive = FALSE, dim_interactive = c(600,600)) {

    requireNamespace("intergraph", quietly = TRUE)

    # Get degree of nodes and find hubs
    h <- get_hubs_grn(edgelist_grn, return_degree = TRUE, ranked = ranked)

    # Create a data frame of nodes and node attributes
    geneIDs <- unique(c(as.character(edgelist_grn[,1]), as.character(edgelist_grn[,2])))
    nod_at <- data.frame(Gene = geneIDs, stringsAsFactors = FALSE)
    nod_at$Class <- ifelse(nod_at$Gene %in% edgelist_grn[,1], "Regulator", "Target")
    nod_at$Degree <- h$Degree$Degree[h$Degree$Gene %in% nod_at$Gene]
    nod_at$isHub <- ifelse(nod_at$Gene %in% h$Hubs$Gene, TRUE, FALSE)
    nod_at <- nod_at[order(nod_at$Class, -nod_at$Degree), ]

    # Should the network be interactive?
    if(interactive) {

        if(!requireNamespace("networkD3", quietly = TRUE)) {
            stop("`interactive = TRUE` requires the 'networkD3' package.")
        }

        graph <- igraph::graph_from_data_frame(d = edgelist_grn,
                                               vertices = nod_at, directed=TRUE)
        graph <- igraph::simplify(graph)
        graph_d3 <- networkD3::igraph_to_networkD3(graph, group = nod_at$Class)
        graph_d3$nodes <- merge(graph_d3$nodes, nod_at, by.x="name", by.y="Gene", sort = FALSE)
        my_color <- 'd3.scaleOrdinal() .domain(["Regulator", "Target"]) .range(["forestgreen", "orange"])'
        d <- dim_interactive
        p <- networkD3::forceNetwork(Links = graph_d3$links, Nodes = graph_d3$nodes,
                                     Source = 'source', Target = 'target',
                                     NodeID = 'name', Group = 'group',
                                     colourScale = my_color,
                                     Nodesize = 'Degree', height=d[2], width=d[1],
                                     opacity=1, zoom = TRUE, fontSize = 20, legend=TRUE)

    } else { #Static network
        # Define plotting parameters
        add_edges <- geom_edges(
            color = "grey60", alpha = 0.3, arrow = ggplot2::arrow(
                length = ggnetwork::unit(0.1, "lines"), type = "closed"
            ), curvature = curvature, show.legend = FALSE
        )

        # Handle node labeling based on which labels to display (top hubs, hubs or all genes)
        if(show_labels == "all") {
            nod_at$Degree2 <- nod_at$Degree * 0.4
            add_nodelabel <- geom_nodetext(
                aes(label = .data$name, size = .data$Degree2),
            show.legend = FALSE)

        } else if(show_labels == "allhubs") {
            add_nodelabel <- geom_nodelabel_repel(
                aes(label = .data$name), color = "azure4",
                box.padding = ggnetwork::unit(1, "lines"),
                data = function(x) { x[ x$isHub, ]},
                show.legend = FALSE, max.overlaps = Inf
            )
        } else if(show_labels == "tophubs") {
            tophubs <- nod_at[nod_at$isHub == TRUE, 1][seq_len(top_n_hubs)]
            nod_at$isTopHub <- ifelse(nod_at$Gene %in% tophubs, TRUE, FALSE)
            add_nodelabel <- geom_nodelabel_repel(
                aes(label = .data$name), color = "azure4",
                box.padding = ggnetwork::unit(1, "lines"),
                data = function(x) { x[ x$isTopHub, ]},
                show.legend = FALSE, max.overlaps = Inf
            )
        } else if(show_labels == "none") {
            add_nodelabel <- NULL
        } else {
            stop("Please, specify a valid option for 'show_labels'.")
        }
        # Create graph object
        graph <- graph_from_data_frame(
            d = edgelist_grn, vertices = nod_at, directed = TRUE
        )
        graph <- igraph::simplify(graph)
        n <- ggnetwork(graph, layout = layout(), arrow.gap = arrow.gap)
        n$Class <- factor(n$Class, levels=c("Target", "Regulator"))

        # Plot graph
        p <- ggplot(
            n, aes(x = .data$x, y = .data$y, xend = .data$xend, yend = .data$yend)
        ) +
            add_edges +
            geom_nodes(
                aes(fill = .data$Class),
                shape = 21, color = "gray40"
            ) +
            scale_fill_manual(values = c("darkolivegreen3", "mediumpurple3")) +
            add_nodelabel +
            ggnetwork::theme_blank()
        }
    return(p)
}


#' Plot gene coexpression network from edge list
#'
#' @param edgelist_gcn Data frame containing the edge list for the GCN.
#' The edge list can be generated with \code{get_edge_list()}.
#' @param net List object returned by \code{exp2net}.
#' @param color_by How should nodes be colored? It must be either "module"
#' (nodes will have the colors of their modules) or a 2-column data frame
#' containing genes in the first column and a custom gene annotation
#' in the second column. Default: "module".
#' @param hubs Data frame containing hub genes in the first column,
#' their modules in the second column, and intramodular connectivity in
#' the third column.
#' @param show_labels Character indicating which nodes will be labeled.
#' One of "all", "allhubs", "tophubs", or "none". Default: tophubs.
#' @param top_n_hubs Number of top hubs to be labeled. It is only valid
#' if \code{show_labels} equals "tophubs". Default is 5.
#' @param curvature Numeric indicating the amount of curvature in edges.
#' Negative values produce left-hand curves, positive values produce right-hand
#' curves, and zero produces a straight line. Default: 0.1.
#' @param interactive Logical indicating whether the network should be
#' interactive or not. Default is FALSE.
#' @param dim_interactive Numeric vector with width and height of window
#' for interactive plotting. Default: c(600,600).
#'
#' @return A ggplot object.
#'
#' @rdname plot_gcn
#' @author Fabricio Almeida-Silva
#' @export
#' @importFrom igraph simplify graph_from_data_frame
#' @importFrom ggnetwork ggnetwork geom_edges geom_nodes geom_nodetext theme_blank geom_nodelabel_repel unit
#' @importFrom ggplot2 ggplot aes guides
#' @import intergraph
#' @examples
#' data(filt.se)
#' gcn <- exp2gcn(filt.se, SFTpower = 18, cor_method = "pearson")
#' gcn_edges <- get_edge_list(gcn, module="brown", filter=TRUE,
#'                            method="min_cor")
#' hubs <- get_hubs_gcn(filt.se, gcn)
#' p <- plot_gcn(gcn_edges, gcn, hubs = hubs)
plot_gcn <- function(edgelist_gcn, net, color_by = "module", hubs = NULL,
                     show_labels = "tophubs", top_n_hubs = 5,
                     curvature = 0,
                     interactive = FALSE, dim_interactive = c(600,600)) {
    requireNamespace("intergraph", quietly=TRUE)

    if(is.null(hubs) | is.null(edgelist_gcn)) {
        stop("Arguments 'edgelist_gcn' and 'hubs' are mandatory for this network.")
    }

    # How should genes be colored?
    if(is.data.frame(color_by)) {
        gene_annotation <- color_by # color by custom annotation
    } else {
        gene_annotation <- net$genes_and_modules # color by module color
    }
    kIN <- net$kIN

    # Create a data frame of nodes and node attributes
    geneIDs <- unique(c(as.character(edgelist_gcn[,1]), as.character(edgelist_gcn[,2])))
    nod_at <- data.frame(Gene = geneIDs, stringsAsFactors = FALSE)
    nod_at <- merge(nod_at, gene_annotation, by = 1)
    names(nod_at)[2] <- "Class"
    nod_at$Class <- as.factor(nod_at$Class)
    nod_at$Degree <- kIN$kWithin[rownames(kIN) %in% nod_at$Gene]
    nod_at$isHub <- ifelse(nod_at$Gene %in% hubs[,1], TRUE, FALSE)
    nod_at <- nod_at[order(nod_at$Class, -nod_at$Degree), ]

    # Should the network be interactive?
    if(interactive) {

        if(!requireNamespace("networkD3", quietly = TRUE)) {
            stop("`interactive = TRUE` requires the 'networkD3' package.")
        }

        graph <- igraph::simplify(igraph::graph_from_data_frame(d = edgelist_gcn, vertices = nod_at, directed=FALSE))
        graph_d3 <- networkD3::igraph_to_networkD3(graph, group = nod_at$Class)
        graph_d3$nodes <- merge(graph_d3$nodes, nod_at, by.x="name", by.y="Gene", sort = FALSE)
        d <- dim_interactive
        p <- networkD3::forceNetwork(Links = graph_d3$links, Nodes = graph_d3$nodes,
                                     Source = 'source', Target = 'target',
                                     NodeID = 'name', Group = 'group',
                                     Nodesize = 'Degree', height=d[2], width=d[1],
                                     opacity=0.8, zoom = TRUE, fontSize = 13)
    } else {
        # Handle gene coloring based on number of annotation classes
        if(nlevels(nod_at$Class) == 1) {
            nodecol <- levels(nod_at$Class)
            add_nodes <- geom_nodes(
                aes(size = .data$Degree, alpha = .data$Degree), color = nodecol
            )
        } else {
            add_nodes <- geom_nodes(
                aes(size = .data$Degree, alpha = .data$Degree, color = .data$Class)
            )
        }

        if(is.data.frame(color_by)) {
            palette <- custom_palette(1)[seq_len(nlevels(nod_at$Class))]
            scale_color <- ggplot2::scale_color_manual(values = palette)
        } else {
            scale_color <- ggplot2::scale_color_manual(values = levels(nod_at$Class))
        }

        # Handle node labeling based on which labels to display (top hubs, hubs or all genes)
        if(show_labels == "all") {
            nod_at$Degree2 <- nod_at$Degree * 0.4
            add_nodelabel <- ggnetwork::geom_nodetext(
                aes(label = .data$name, size = .data$Degree2),
                show.legend = FALSE
            )
        } else if(show_labels == "allhubs") {
            add_nodelabel <- ggnetwork::geom_nodelabel_repel(
                aes(label = .data$name), color = "azure4",
                box.padding = ggnetwork::unit(1, "lines"),
                data = function(x) { x[ x$isHub, ]},
                show.legend = FALSE, max.overlaps = Inf
            )
        } else if(show_labels == "tophubs") {
            tophubs <- nod_at[nod_at$isHub == TRUE, 1][seq_len(top_n_hubs)]
            nod_at$isTopHub <- ifelse(nod_at$Gene %in% tophubs, TRUE, FALSE)
            add_nodelabel <- geom_nodelabel_repel(
                aes(label = .data$name), color = "azure4",
                box.padding = ggnetwork::unit(1, "lines"),
                data = function(x) { x[ x$isTopHub, ]},
                show.legend = FALSE
            )

        } else if(show_labels == "none") {
            add_nodelabel <- NULL
        } else {
            stop("Please, specify a valid option for 'show_labels'.")
        }

        # Create graph object
        graph <- graph_from_data_frame(
            d = edgelist_gcn, vertices = nod_at, directed = FALSE
        )
        n <- ggnetwork::ggnetwork(graph, arrow.gap=0)

        # Plot graph
        p <- ggplot(
            n, aes(x = .data$x, y = .data$y, xend = .data$xend, yend = .data$yend)
        ) +
            geom_edges(
                color = "grey75", alpha = 0.5, curvature = curvature,
                show.legend = FALSE
            ) +
            add_nodes +
            scale_color +
            add_nodelabel +
            ggnetwork::theme_blank()
    }
    return(p)
    }

#----Load data------------------------------------------------------------------
data(filt.se)
tfs <- sample(rownames(filt.se), size=50, replace=FALSE)

## Infer networks to be used
set.seed(123)
ppi_edges <- igraph::as_edgelist(igraph::sample_pa(n=50))
gcn <- exp2gcn(filt.se, SFTpower = 18, cor_method = "pearson")
gcn_edges <- get_edge_list(
    gcn, module = "brown", filter = TRUE, method = "min_cor"
)


#----Start tests----------------------------------------------------------------
test_that("detect_communities() returns a data frame of communities", {
    com <- detect_communities(ppi_edges, directed=TRUE)
    com2 <- detect_communities(ppi_edges, directed=FALSE)
    expect_equal(class(com), "data.frame")
    expect_equal(class(com2), "data.frame")
})


test_that("plot_ppi() plots a PPI network", {

    color_by <- data.frame(
        Node = 1:50,
        Class = paste0("Class", sample(c(1:50), size = 50, replace = TRUE))
    )

    p <- plot_ppi(ppi_edges, add_color_legend = FALSE)
    p2 <- plot_ppi(ppi_edges, color_by = color_by)
    p3 <- plot_ppi(ppi_edges, add_color_legend = FALSE, show_labels = "all")
    p4 <- plot_ppi(ppi_edges, add_color_legend = FALSE, show_labels = "allhubs")
    p5 <- plot_ppi(ppi_edges, add_color_legend = FALSE, show_labels = "none")

    expect_error(
        plot_ppi(ppi_edges, add_color_legend = FALSE, show_labels = "any")
    )

    pint <- plot_ppi(ppi_edges, interactive = TRUE)

    expect_true("ggplot" %in% class(p))
    expect_true("ggplot" %in% class(p2))
    expect_true("ggplot" %in% class(p3))
    expect_true("ggplot" %in% class(p4))
    expect_true("ggplot" %in% class(p5))

    expect_true("forceNetwork" %in% class(pint))

})


test_that("plot_grn() plots a GRN", {

    p <- plot_grn(ppi_edges, ranked = FALSE)
    p3 <- plot_grn(ppi_edges, show_labels = "all")
    p4 <- plot_grn(ppi_edges, show_labels = "allhubs")
    p5 <- plot_grn(ppi_edges, show_labels = "none")

    expect_error(
        plot_ppi(ppi_edges, show_labels = "any")
    )

    pint <- plot_grn(ppi_edges, interactive = TRUE)

    expect_true("ggplot" %in% class(p))
    expect_true("ggplot" %in% class(p3))
    expect_true("ggplot" %in% class(p4))
    expect_true("ggplot" %in% class(p5))

    expect_true("forceNetwork" %in% class(pint))

})


test_that("plot_gcn() plots a GCN", {

    color_by <- data.frame(
        Node = unique(c(gcn_edges$Gene1, gcn_edges$Gene2)),
        Class = paste0("Class", sample(c(1:50), size = 32, replace = TRUE))
    )
    hubs <- data.frame(
        Gene = c("ZeamMp044", "ZeamMp092", "ZeamMp108"),
        Degree = rep(10, 3)
    )


    p <- plot_gcn(gcn_edges, gcn, hubs = hubs)
    p2 <- plot_gcn(gcn_edges, gcn, hubs = hubs, color_by = color_by)
    p3 <- plot_gcn(gcn_edges, gcn, hubs = hubs, show_labels = "all")
    p4 <- plot_gcn(gcn_edges, gcn, hubs = hubs, show_labels = "allhubs")
    p5 <- plot_gcn(gcn_edges, gcn, hubs = hubs, show_labels = "none")

    expect_error(
        plot_gcn(gcn_edges)
    )

    expect_error(
        plot_gcn(gcn_edges, gcn, hubs = hubs, show_labels = "wrong")
    )

    pint <- plot_gcn(gcn_edges, gcn, hubs = hubs, show_labels = "allhubs",
                     interactive = TRUE)

    expect_true("ggplot" %in% class(p))
    expect_true("ggplot" %in% class(p2))
    expect_true("ggplot" %in% class(p3))
    expect_true("ggplot" %in% class(p4))
    expect_true("ggplot" %in% class(p5))

    expect_true("forceNetwork" %in% class(pint))
})

#----Load data----
data(og.zma.osa)
data(filt.se)

#----Start tests
test_that("is_singleton() returns logical vector of singletons", {
    genes <- tail(rownames(filt.se), n=100)
    s <- is_singleton(genes, og.zma.osa)
    expect_true(is.logical(s))
})

#' Pick power to fit networks to scale-free topology
#'
#' @param exp_list A list of expression data frames or
#' SummarizedExperiment objects.
#' If input is a list of data frames, row names must correspond to gene IDs
#' and column names to samples.
#' The list can be created with \code{list(exp1, exp2, ..., expn)}.
#' @param setLabels Character vector containing labels for each expression set.
#' @param metadata A data frame containing sample names in row names and
#' sample annotation in the first column.
#' Ignored if `exp_list` is a list of `SummarizedExperiment` objects, since
#' the function will extract colData.
#' @param cor_method Correlation method used for network reconstruction.
#' One of "spearman" (default), "biweight", or "pearson".
#' @param net_type Network type. One of "signed hybrid" (default),
#' "signed" or "unsigned".
#' @param rsquared Minimum R squared to consider the network similar to
#' a scale-free topology. Default is 0.8.
#'
#' @return A list of 2 elements: \describe{
#'   \item{power}{Numeric vector of optimal beta powers to fit networks to SFT}
#'   \item{plot}{A ggplot object displaying main statistics of the SFT fit test}
#' }
#' @rdname consensus_SFT_fit
#' @export
#' @importFrom WGCNA pickSoftThreshold checkSets
#' @importFrom ggplot2 ggplot aes geom_point labs theme_bw scale_color_manual
#' ylim theme geom_hline
#' @importFrom ggrepel geom_text_repel
#' @examples
#' set.seed(12)
#' data(zma.se)
#' filt.zma <- filter_by_variance(zma.se, n=500)
#' zma.set1 <- filt.zma[, sample(colnames(filt.zma), size=20, replace=FALSE)]
#' zma.set2 <- filt.zma[, sample(colnames(filt.zma), size=20, replace=FALSE)]
#' list.sets <- list(zma.set1, zma.set2)
#' cons_sft <- consensus_SFT_fit(list.sets, setLabels = c("Maize1", "Maize2"),
#'                               cor_method = "pearson")
consensus_SFT_fit <- function(exp_list, setLabels = NULL, metadata = NULL,
                              cor_method = "spearman",
                              net_type = "signed hybrid", rsquared = 0.8) {
    metadata <- handle_metadata(exp_list, metadata)
    exp_list <- handleSElist(exp_list)
    if(is.null(setLabels)) {
        setLabels <- seq_along(exp_list)
    }

    # Build multi-set object
    multiExp <- lapply(exp_list, function(x) {
        element <- list(data=as.data.frame(t(x)))
        return(element)
    })

    # Check if the data has the correct format for downstream analysis
    expSize <- WGCNA::checkSets(multiExp)

    # Choose a set of soft-thresholding powers
    powers <- seq(5, 20, by=1)
    sft <- lapply(multiExp, function(x) {
        if(cor_method == "pearson") {
            sft <- WGCNA::pickSoftThreshold(
                x$data, networkType = net_type, powerVector = powers,
                RsquaredCut = rsquared
            )
        } else if(cor_method == "biweight") {
            sft <- WGCNA::pickSoftThreshold(
                x$data, networkType = net_type, powerVector = powers,
                RsquaredCut = rsquared, corFnc = bicor,
                corOptions = list(use = 'p', maxPOutliers = 0.05)
            )
        } else if (cor_method == "spearman") {
            sft <- WGCNA::pickSoftThreshold(
                x$data, networkType = net_type, powerVector = powers,
                RsquaredCut = rsquared,
                corOptions = list(use = 'p', method = "spearman")
            )
        } else {
            stop("Please, specify a correlation method (one of 'spearman', 'pearson' or 'biweight').")
        }
        wgcna_power <- sft$powerEstimate
        if(is.na(wgcna_power)){
            message("No power reached R-squared cut-off, now choosing max R-squared based power")
            wgcna_power <- sft$fitIndices$Power[which(sft$fitIndices$SFT.R.sq == max(sft$fitIndices$SFT.R.sq))]
        }
        return(list(sft = sft, wgcna_power = wgcna_power))
    })

    power <- unlist(lapply(sft, function(x) return(x$wgcna_power)))

    # Create data frame with indices to plot
    sft_df <- Reduce(rbind, lapply(seq_along(sft), function(x) {
        indices <- sft[[x]]$sft$fitIndices
        df <- data.frame(power = indices[,1],
                         fit = -sign(indices[,3]) * indices[,2],
                         meank = indices[,5],
                         Set = setLabels[x])
        return(df)
    }))

    # Plot 1
    cols <- custom_palette(1)
    p1 <- ggplot(sft_df, aes(x = .data$power, y = .data$fit)) +
        geom_point(aes(color = .data$Set)) +
        ggrepel::geom_text_repel(aes(label = .data$power, color = .data$Set)) +
        scale_color_manual(values = cols) +
        labs(
            x = "Soft threshold (power)",
            y = expression(paste("Scale-free topology fit - ", R^{2})),
            title = "Scale independence"
        ) +
        theme_bw() +
        ylim(c(0,1)) +
        geom_hline(yintercept = rsquared, color = "brown3") +
        theme(legend.position = "none")

    # Plot 2
    p2 <- ggplot(sft_df, aes(x = .data$power, y = .data$meank)) +
        geom_point(aes(color = .data$Set)) +
        ggrepel::geom_text_repel(aes(label = .data$power, color = .data$Set),
                                 show.legend = FALSE) +
        scale_color_manual(values = cols) +
        labs(
            x = "Soft threshold (power)",
            y = "Mean connectivity (k)",
            title = "Mean connectivity"
        ) +
        theme_bw()

    sft_plot <- patchwork::wrap_plots(p1, p2, ncol = 2)

    results <- list(power = power, plot = sft_plot)
    return(results)
}


#' Identify consensus modules across independent data sets
#'
#' @param exp_list A list containing the expression data frames with genes in
#' row names and samples in column names or `SummarizedExperiment` objects.
#' The list can be created by using \code{list(exp1, exp2, ..., expn)}.
#' @param metadata A data frame containing sample names in row names and
#' sample annotation in the first column.
#' Ignored if `exp_list` is a list of `SummarizedExperiment` objects, since
#' the function will extract colData.
#' @param power Numeric vector of beta power for each expression set
#' as calculated by \code{consensus_SFT_fit}.
#' @param cor_method Correlation method used for network reconstruction.
#' One of "spearman" (default), "biweight", or "pearson".
#' @param net_type Network type. One of "signed hybrid" (default),
#' "signed" or "unsigned".
#' @param module_merging_threshold Correlation threshold to merge
#' similar modules into a single one. Default: 0.8.
#' @param TOM_type Character indicating the type of Topological
#' Overlap Matrix to (TOM) create. One of
#' 'unsigned', 'signed', 'signed Nowick', 'unsigned 2', 'signed 2',
#' and 'signed Nowick 2'. By default, TOM type is automatically
#' selected based on network type.
#' @param verbose Logical indicating whether to display progress
#' messages or not. Default: FALSE.
#'
#' @return A list containing 4 elements: \describe{
#'   \item{consMEs}{Consensus module eigengenes}
#'   \item{exprSize}{Description of the multi-set object returned by the function \code{WGCNA::checkSets}}
#'   \item{sampleInfo}{Metadata for each expression set}
#'   \item{genes_cmodules}{Data frame of genes and consensus modules}
#'   \item{dendro_plot_objects}{Objects to be used in dendrogram plotting}
#' }
#'
#' @rdname consensus_modules
#' @export
#' @importFrom WGCNA checkSets adjacency TOMsimilarity pquantile labels2colors
#' multiSetMEs consensusMEDissimilarity mergeCloseModules
#' @importFrom dynamicTreeCut cutreeDynamic
#' @examples
#' set.seed(12)
#' data(zma.se)
#' filt.zma <- filter_by_variance(zma.se, n=500)
#' zma.set1 <- filt.zma[, sample(colnames(filt.zma), size=20, replace=FALSE)]
#' zma.set2 <- filt.zma[, sample(colnames(filt.zma), size=20, replace=FALSE)]
#' list.sets <- list(zma.set1, zma.set2)
#' # SFT power previously identified with consensus_SFT_fit()
#' cons_mod <- consensus_modules(list.sets, power = c(11, 13),
#'                               cor_method = "pearson")
consensus_modules <- function(
        exp_list, metadata, power,
        cor_method = "spearman", net_type = "signed hybrid",
        module_merging_threshold = 0.8,
        TOM_type = NULL,
        verbose = FALSE
) {

    nSets <- length(exp_list)

    # Extract sample metadata as a list of data frames
    if(is(exp_list[[1]], "SummarizedExperiment")) {
        metadata <- lapply(exp_list, function(x) {
            return(se2metadata(x)$coldata)
        })
    }

    # Keep only shared columns in sample metadata data frames
    shared <- Reduce(intersect, lapply(metadata, colnames))
    metadata <- lapply(metadata, function(x) return(x[, shared, drop = FALSE]))

    # Build multi-set object
    exp_list <- handleSElist(exp_list)
    multiExp <- lapply(exp_list, function(x) {
        element <- list(data = as.data.frame(t(x)))
        return(element)
    })

    # Check if the data has the correct format for downstream analysis
    expSize <- WGCNA::checkSets(multiExp)
    nGenes <- expSize$nGenes
    nSamples <- expSize$nSamples

    # Build multi-set object with metadata
    sampleinfo <- lapply(seq_along(multiExp), function(x) {
        sinfo <- metadata[[x]][rownames(multiExp[[x]]$data), , drop = FALSE]
        return(sinfo)
    })

    # Calculate adjacencies for each individual set
    if(verbose) { message("Calculating adjacency matrix...") }
    adj <- lapply(seq_len(nSets), function(x) {
        if(cor_method == "pearson") {
            adjacencies <- WGCNA::adjacency(
                multiExp[[x]]$data, power = power[x], type = net_type
            )
        } else if(cor_method == "spearman") {
            adjacencies <- WGCNA::adjacency(
                multiExp[[x]]$data, power = power[x], type = net_type,
                corOptions = list(use = "p", method = "spearman")
            )
        } else if (cor_method == "biweight") {
            adjacencies <- WGCNA::adjacency(
                multiExp[[x]]$data, power = power[x], type = net_type,
                corFnc = bicor
            )
        } else {
            stop("Please, specify a correlation method. One of 'spearman', 'pearson' or 'biweight'.")
        }
        return(adjacencies)
    })

    # Calculate TOMs in each individual data set
    if(verbose) { message("Calculating topological overlap matrix (TOM)...") }
    tom_type <- switch(
        net_type,
        "signed hybrid" = "signed",
        "signed" = "signed Nowick",
        "unsigned"
    )
    if(!is.null(TOM_type)) { tom_type <- TOM_type }

    TOM <- lapply(adj, function(x) return(TOMsimilarity(x, TOMType = tom_type)))

    # Scaling TOM to make them comparable
    scaleP <- 0.95
    nSamples <- as.integer(1 / (1-scaleP) * 100)
    scaleSample <- sample(nGenes * (nGenes-1) / 2, size = nSamples)

    scaledTOM <- lapply(seq_len(nSets), function(x) {
        tom_scaled <- TOM[[x]]
        if(x > 1) {
            TOMScalingSamples <- as.dist(TOM[[x]])[scaleSample]
            scaleQuant <- quantile(TOMScalingSamples, probs = scaleP, type = 8)
            scalePowers <- log(scaleQuant[1]) / log(scaleQuant)
            tom_scaled <- tom_scaled^scalePowers
        }
        return(tom_scaled)
    })

    # Calculation of consensus TOM
    if (nSets <= 3) {
        consensusTOM <- do.call(pmin, lapply(seq_along(TOM), function(i) TOM[[i]]))
    } else {
        consensusTOM <- do.call(function(x) WGCNA::pquantile(x, prob=0.25),
                                lapply(seq_along(TOM), function(i) TOM[[i]]))
    }

    # Clustering and module identification
    consTree <- hclust(as.dist(1-consensusTOM), method = "average")
    unmergedLabels <- dynamicTreeCut::cutreeDynamic(
        dendro = consTree,
        distM = 1 - consensusTOM,
        deepSplit = 2,
        cutHeight = 0.995,
        minClusterSize = 30,
        pamRespectsDendro = FALSE
    )

    nmod <- length(unique(unmergedLabels))
    palette <- rev(WGCNA::standardColors(nmod))
    unmergedColors <- WGCNA::labels2colors(unmergedLabels, colorSeq = palette)

    # Merge similar modules
    unmergedMEs <- WGCNA::multiSetMEs(
        multiExp, colors = NULL, universalColors = unmergedColors
    )

    merging_threshold <- 1-module_merging_threshold
    merge <- WGCNA::mergeCloseModules(
        multiExp, unmergedColors, cutHeight = merging_threshold, verbose = 0
    )

    genes_cmod <- data.frame(
        Genes = rownames(adj[[1]]),
        Cons_modules = merge$colors
    )
    consMEs <- merge$newMEs

    # Plot dendrogram with merged colors
    result_list <- list(
        consMEs = consMEs,
        exprSize = expSize,
        sampleInfo = sampleinfo,
        genes_cmodules = genes_cmod,
        dendro_plot_objects = list(
            tree = consTree,
            Unmerged = unmergedColors,
            Merged = merge$colors
        )
    )
    return(result_list)
}


#' Correlate set-specific modules and consensus modules to sample information
#'
#' @param consensus Consensus network returned by \code{consensus_modules}.
#' @param cor_method Correlation method to be used. One of 'spearman' or
#' 'pearson'. Default: 'pearson'.
#' @param metadata_cols A vector (either numeric or character) indicating
#' which columns should be extracted from column metadata if \strong{exp}
#' is a `SummarizedExperiment` object. The vector can contain column
#' indices (numeric) or column names (character). By default, all columns are
#' used.
#'
#' @return Data frame of consensus module-trait correlations and p-values,
#' with the following variables:
#' \describe{
#'   \item{trait}{Factor, trait name. Each trait corresponds to a variable
#'                of the sample metadata (if numeric) or levels of a variable
#'                (if categorical).}
#'   \item{ME}{Factor, module eigengene.}
#'   \item{cor}{Numeric, correlation.}
#'   \item{pvalue}{Numeric, correlation P-values.}
#'   \item{group}{Character, name of the metadata variable.}
#' }
#'
#' @rdname consensus_trait_cor
#' @export
#' @importFrom WGCNA corPvalueFisher labels2colors
#' @importFrom reshape2 melt
#' @examples
#' set.seed(12)
#' data(zma.se)
#' filt.zma <- filter_by_variance(zma.se, n=500)
#' zma.set1 <- filt.zma[, sample(colnames(filt.zma), size=20, replace=FALSE)]
#' zma.set2 <- filt.zma[, sample(colnames(filt.zma), size=20, replace=FALSE)]
#' list.sets <- list(zma.set1, zma.set2)
#' # SFT power previously identified with consensus_SFT_fit()
#' consensus <- consensus_modules(list.sets, power = c(11, 13),
#'                                cor_method = "pearson")
#' consensus_trait <- consensus_trait_cor(consensus, cor_method = "pearson")
consensus_trait_cor <- function(consensus, cor_method = "pearson",
                                metadata_cols = NULL) {

    consMEs <- consensus$consMEs
    metadata <- consensus$sampleInfo
    nSets <- consensus$exprSize$nSets

    if(!is.null(metadata_cols)) {
        metadata <- lapply(metadata, function(x) return(x[, metadata_cols, drop = FALSE]))
    }


    # Get a list of correlation and P-value matrices
    matrices <- lapply(seq_len(nSets), function(set) {

        coldata <- metadata[[set]]

        # Get model matrices
        mats <- Reduce(cbind, lapply(seq_along(coldata), function(x) {
            model_mat <- get_model_matrix(coldata, column_idx = x)
            return(model_mat)
        }))

        # Get correlation matrix
        cormat <- cor(consMEs[[set]]$data, mats, use = "p", method = cor_method)

        # Get P-value matrix
        pmat <- WGCNA::corPvalueFisher(cormat, consensus$exprSize$nSamples[set])

        results <- list(cor = cormat, pvals = pmat)
        return(results)
    })

    # Store correlation and P-value matrices in separate objects
    cormats <- lapply(matrices, function(x) return(x$cor))
    pmats <- lapply(matrices, function(x) return(x$pvals))

    # Create matrices of consensus correlation and P-values
    cons_cor <- matrix(
        NA, nrow(cormats[[1]]), ncol(cormats[[1]]),
        dimnames = list(rownames(cormats[[1]]), colnames(cormats[[1]]))
    )
    cons_pval <- cons_cor

    ## If there is a difference in sign for element m[i,j], add FALSE to it
    neg <- Reduce(`&`, lapply(cormats, function(x) return(x < 0)))
    pos <- Reduce(`&`, lapply(cormats, function(x) return(x > 0)))

    ## For every element, keep minimum cor and maximum P-value of all sets
    cons_cor[neg] <- do.call(pmax, lapply(cormats, function(x) return(x[neg])))
    cons_cor[pos] <- do.call(pmin, lapply(cormats, function(x) return(x[pos])))
    cons_pval[neg] <- do.call(pmax, lapply(pmats, function(x) return(x[neg])))
    cons_pval[pos] <- do.call(pmax, lapply(pmats, function(x) return(x[pos])))

    # Reshape to long format and merge data frames into one
    v <- c("ME", "trait")
    cor_long <- reshape2::melt(cons_cor, value.name = "cor", varnames = v)
    p_long <- reshape2::melt(cons_pval, value.name = "pvalue", varnames = v)
    final_df <- merge(cor_long, p_long)

    # Add a column `group` with metadata variable name
    var_levels <- Reduce(rbind, lapply(seq_along(metadata[[1]]), function(x) {
        return(data.frame(
            trait = unique(metadata[[1]][, x]),
            group = names(metadata[[1]])[x]
        ))
    }))
    final_df <- merge(final_df, var_levels)

    return(final_df)
}

#----Create data for testing----------------------------------------------------
set.seed(12)
data("zma.se")
filt.zma <- filter_by_variance(zma.se, n=500)
zma.set1 <- filt.zma[, sample(colnames(filt.zma), size=20, replace=FALSE)]
zma.set2 <- filt.zma[, sample(colnames(filt.zma), size=20, replace=FALSE)]
list.sets <- list(zma.set1, zma.set2)

rlist.sets <- lapply(list.sets, function(x) return(x[1:50, ]))
rlist.sets2 <- lapply(list.sets, function(x) return(x[1:100, ]))


#----Start tests----------------------------------------------------------------
test_that("consensus_SFT_fit() calculates best SFT fit for all networks", {
    cons_sft <- consensus_SFT_fit(rlist.sets, cor_method = "pearson")
    cons_sft2 <- consensus_SFT_fit(rlist.sets, cor_method = "spearman")
    cons_sft3 <- consensus_SFT_fit(rlist.sets, cor_method = "biweight")

    expect_error(
        consensus_SFT_fit(rlist.sets, cor_method = "error")
    )

    expect_equal(class(cons_sft[[1]]), "numeric")
    expect_true("patchwork" %in% class(cons_sft[[2]]))
    expect_true("ggplot" %in% class(cons_sft[[2]]))
    expect_equal(length(cons_sft[[1]]), length(list.sets))

    expect_equal(class(cons_sft2), "list")
    expect_equal(class(cons_sft3), "list")
})

test_that("consensus_modules() and related functions work", {

    # consensus_modules()
    cm1 <- consensus_modules(
        rlist.sets2, power = c(20, 20), cor_method = "pearson", verbose = TRUE
    )
    cm2 <- consensus_modules(
        rlist.sets2, power = c(20, 20), cor_method = "spearman", verbose = TRUE
    )
    cm3 <- consensus_modules(
        rlist.sets2, power = c(15, 15), cor_method = "biweight", verbose = TRUE
    )

    expect_error(
        consensus_modules(
            rlist.sets2, power = c(20, 20), cor_method = "error"
        )
    )

    expect_equal(class(cm1), "list")
    expect_equal(length(cm1), 5)

    expect_equal(class(cm2), "list")
    expect_equal(class(cm3), "list")

    # plot_dendro_and_cons_colors()
    p <- plot_dendro_and_colors(cm1)
    expect_true("ggplot" %in% class(p))

    # consensus_trait_cor()
    ct <- consensus_trait_cor(cm1, cor_method = "pearson")

    expect_equal(class(ct), "data.frame")
    expect_equal(names(ct), c("trait", "ME", "cor", "pvalue", "group"))

})

#' Helper to handle SummarizedExperiment or expression data frame as input
#'
#' @param exp Expression data as a data frame or a SummarizedExperiment object
#'
#' @return If exp is a SummarizedExperiment object, it will return `assay(exp)`.
#' Otherwhise, it will simply return exp as it is.
#' @noRd
#' @importFrom SummarizedExperiment assay
handleSE <- function(exp) {
    if(is(exp, "SummarizedExperiment")) {
        fexp <- SummarizedExperiment::assay(exp)
    } else {
        fexp <- exp
    }
    return(fexp)
}

#' Replace content of a SummarizedExperiment object based on filtered expression data frame
#'
#' @param exp Expression data frame with genes IDs in row names and samples
#' in column names.
#' @param SE Original SummarizedExperiment object.
#' @return A SummarizedExperiment object
#' @noRd
#' @importFrom SummarizedExperiment SummarizedExperiment colData assay
exp2SE <- function(exp, SE) {

    # Get overlaps between samples in both sets
    SE_cols <- rownames(SummarizedExperiment::colData(SE))
    filt_exp_cols <- colnames(exp)
    overlap <- which(SE_cols %in% filt_exp_cols)

    # Modify original SE based on filtered expression data frame
    SE_final <- SummarizedExperiment::SummarizedExperiment(
        assays = exp,
        colData = SummarizedExperiment::colData(SE)[overlap, , drop = FALSE]
    )

    return(SE_final)
}


#' Generate custom color palette
#'
#' @param pal Numeric specifying palette number, from 1 to 3.
#'
#' @return Character vector of custom color palette with 20 colors
#' @noRd
custom_palette <- function(pal = 1) {
    pal1 <- c("#1F77B4FF", "#FF7F0EFF", "#2CA02CFF", "#D62728FF",
              "#9467BDFF", "#8C564BFF", "#E377C2FF", "#7F7F7FFF",
              "#BCBD22FF", "#17BECFFF", "#AEC7E8FF", "#FFBB78FF",
              "#98DF8AFF", "#FF9896FF", "#C5B0D5FF", "#C49C94FF",
              "#F7B6D2FF", "#C7C7C7FF", "#DBDB8DFF", "#9EDAE5FF")

    pal2 <- c("#3182BDFF", "#E6550DFF", "#31A354FF", "#756BB1FF",
              "#636363FF", "#6BAED6FF", "#FD8D3CFF", "#74C476FF",
              "#9E9AC8FF", "#969696FF", "#9ECAE1FF", "#FDAE6BFF",
              "#A1D99BFF", "#BCBDDCFF", "#BDBDBDFF", "#C6DBEFFF",
              "#FDD0A2FF", "#C7E9C0FF", "#DADAEBFF", "#D9D9D9FF")

    pal3 <- c("#393B79FF", "#637939FF", "#8C6D31FF", "#843C39FF",
              "#7B4173FF", "#5254A3FF", "#8CA252FF", "#BD9E39FF",
              "#AD494AFF", "#A55194FF", "#6B6ECFFF", "#B5CF6BFF",
              "#E7BA52FF", "#D6616BFF", "#CE6DBDFF", "#9C9EDEFF",
              "#CEDB9CFF", "#E7CB94FF", "#E7969CFF", "#DE9ED6FF")

    l <- list(pal1, pal2, pal3)
    l_final <- l[[pal]]
    return(l_final)
}

#' Map levels of metadata variables to colors for plotting
#'
#' @param col_metadata A data frame with column or row metadata. If column
#' metadata is passed, row names must contain sample names. If row metadata is
#' passed, row names must contain gene names.
#'
#' @return A list containing the following elements:
#' \describe{
#'   \item{metadata}{A metadata data frame as in \strong{col_metadata}, but
#'                   with rows sorted by levels of every column.}
#'   \item{colors}{A list of named character vectors containing the mapping
#'                 between levels of metadata variables and colors.}
#' }
#'
#' @noRd
#' @importFrom stats setNames
metadata2colors <- function(col_metadata) {

    coldata <- NA
    colors <- NA

    if(is.data.frame(col_metadata)) {
        # Get variable names in metadata
        col_names <- names(col_metadata)
        if(length(col_names) > 3) {
            stop("Maximum number of columns for row and sample metadata is 3.")
        }

        # Sort elements in all columns
        coldata <- col_metadata[do.call(order, col_metadata), , drop = FALSE]

        # Create a list of named vectors with variable levels and colors
        colors <- lapply(seq_along(col_names), function(x) {
            levels <- unique(coldata[, x])
            cols <- setNames(custom_palette(x)[seq_along(levels)], levels)
            return(cols)
        })
        names(colors) <- col_names
    }

    # Return results as a list
    results <- list(
        metadata = coldata,
        colors = colors
    )

    return(results)
}

#' Construct parameters to plot heatmap
#'
#' @param exp A gene expression matrix with gene IDs in row names and
#' samples in column names.
#' @param palette Character indicating an RColorBrewer palette to use.
#' @param heatmap_type Character indicating which heatmap type to plot.
#' One of "samplecor" (for pairwise sample correlations) or "expr" (for
#' gene expression).
#' @param cor_method Character indicating which correlation method to use
#' in the \code{cor()} function. Only meaningful if parameter
#' \strong{heatmap_type} equals "samplecor".
#'
#' @return A list with the following elements:
#' \describe{
#'   \item{pal}{Character, name of the RColorBrewer palette to use.}
#'   \item{mat}{Matrix to use to construct the heatmap.}
#'   \item{title}{Character, title of the heatmap.}
#'   \item{name}{Character, matrix name. This is what goes to the title of
#'               matrix's main legend.}
#' }
#'
#' @noRd
#' @importFrom RColorBrewer brewer.pal
#' @importFrom stats cor
#'
heatmap_attributes <- function(exp, palette = NULL, heatmap_type = "samplecor",
                               cor_method = "pearson") {

    # Create objects with attributes `pal`, `mat`, `title`, and `name`
    if(heatmap_type == "samplecor") {
        pal <- "Blues"
        mat <- cor(exp, method = cor_method)
        title <- "Pairwise correlations between samples"
        name <- "Correlation"
    } else if(heatmap_type == "expr") {
        pal <- "YlOrRd"
        mat <- exp
        title <- "Gene expression"
        name <- "Expression"
    } else {
        stop("Invalid argument to `type`. Pick one of 'samplecor' or 'expr'.")
    }
    if(!is.null(palette)) { pal <- palette }
    pal <- colorRampPalette(RColorBrewer::brewer.pal(9, pal))(100)

    # Return results as a list
    results <- list(
        pal = pal,
        mat = mat,
        title = title,
        name = name
    )
    return(results)
}

#' Extract row and column metadata from `SummarizedExperiment` objects
#'
#' @param se A `SummarizedExperiment` object.
#' @param rowdata_cols Columns to use from the rowData element of the
#' `SummarizedExperiment` object. It can be either a character vector
#' of column names or a numeric vector of column indices.
#' By default, all columns are used.
#' @param coldata_cols Columns to use from the colData element of the
#' `SummarizedExperiment` object. It can be either a character vector
#' of column names or a numeric vector of column indices.
#' By default, all columns are used.
#'
#' @return A list with the following elements:
#' \describe{
#'   \item{rowdata}{A data frame of row metadata containing only the selected
#'                  columns.}
#'   \item{coldata}{A data frame of column metadata containing only the
#'                  selected columns.}
#' }
#'
#' @noRd
#' @importFrom SummarizedExperiment rowData
#' @importFrom SummarizedExperiment colData
se2metadata <- function(se, rowdata_cols = NULL, coldata_cols = NULL) {

    final_rowdata <- NA
    final_coldata <- NA

    # Extract row and column metadata
    rowdata <- SummarizedExperiment::rowData(se)
    coldata <- SummarizedExperiment::colData(se)

    # Extract user-defined columns from metadata
    ## Row metadata
    if(ncol(rowdata) > 0) {
        r_cols <- rowdata_cols
        if(is.null(rowdata_cols)) { r_cols <- seq_along(rowdata) }

        final_rowdata <- as.data.frame(rowdata[, r_cols, drop = FALSE])
    }

    if(ncol(coldata) > 0) {
        c_cols <- coldata_cols
        if(is.null(coldata_cols)) { c_cols <- seq_along(coldata) }
        final_coldata <- as.data.frame(coldata[, c_cols, drop = FALSE])
    }

    # Return resuls as a list
    metadata_list <- list(
        rowdata = final_rowdata,
        coldata = final_coldata
    )

    return(metadata_list)
}


#' Get model matrix for metadata variables
#'
#' @param metadata A data frame of column metadata with sample names
#' in row names.
#' @param column_idx Column to use to create the model matrix.
#'
#' @details If the variable is numeric (continuous or discrete),
#' the model matrix is created by simply subsetting the column
#' indicated in \strong{column_idx}. If the variable if categorical,
#' a dummy model matrix is created (without an intercept).
#'
#' @return A data frame with the model matrix to use
#' in \code{module_trait_cor()}.
#'
#' @noRd
#' @importFrom stats model.matrix as.formula
get_model_matrix <- function(metadata, column_idx = 1) {

    # By default, assuming variable is numeric (continuous or discrete)
    mat <- metadata[, column_idx, drop = FALSE]

    if(!is.numeric(metadata[, column_idx])) {

        # Create formula (format: `~var + 0`)
        var_name <- names(metadata)[column_idx]
        formula <- as.formula(paste("~", var_name, "+ 0"))

        # Get model matrix
        mat <- as.data.frame(model.matrix(formula, metadata))
        names(mat) <- gsub(var_name, "", names(mat))
    }

    return(mat)
}

#' Calculate pairwise correlations between genes in a matrix
#'
#' @param exp A numeric matrix containing a gene expression matrix,
#' with genes in rows and samples in columns.
#' @param cor_method Character indicating the correlation method to use.
#' One of "pearson", "spearman", or "biweight". Default: "pearson".
#'
#' @return A numeric, symmetric matrix with pairwise correlations between
#' genes.
#'
#' @importFrom WGCNA bicor
#' @importFrom stats cor
#' @rdname exp2cor
#' @export
#' @examples
#' # Simulate an expression matrix with 100 genes and 50 samples
#' exp <- matrix(
#'     rnorm(100 * 50, mean = 10, sd = 2),
#'     nrow = 100,
#'     dimnames = list(
#'         paste0("gene", seq_len(100)),
#'         paste0("sample", seq_len(50))
#'     )
#' )
#'
#' # Calculate correlation matrix
#' cor_mat <- exp2cor(exp)
exp2cor <- function(exp, cor_method = "pearson") {

    if(cor_method == "pearson") {
        cor_matrix <- cor(t(exp), method = "pearson")

    } else if(cor_method == "spearman") {
        cor_matrix <- cor(t(exp), use = "p", method = "spearman")

    } else if(cor_method == "biweight") {
        cor_matrix <- WGCNA::bicor(t(exp), maxPOutliers = 0.1)

    } else {
        stop(
            "Please, specify a valid correlation method. ",
            "One of 'spearman', 'pearson' or 'biweight'."
        )
    }

    return(cor_matrix)
}

#' Calculate an adjacency matrix from a correlation matrix
#'
#' @param cor_matrix A numeric, symmetric matrix with pairwise correlations
#' between genes (i.e., a 'correlation matrix').
#' @param beta Numeric scalar indicating the value of the \eqn{\beta}
#' power to which correlation coefficients will be raised to ensure
#' scale-free topology fit.
#' @param net_type Character indicating the type of network to infer.
#' Default: "signed hybrid".
#'
#' @return A numeric, symmetric matrix with network adjacency values between
#' genes.
#'
#' @importFrom WGCNA adjacency.fromSimilarity
#' @rdname cor2adj
#' @export
#' @examples
#' # Simulate an expression matrix with 100 genes and 50 samples
#' exp <- matrix(
#'     rnorm(100 * 50, mean = 10, sd = 2),
#'     nrow = 100,
#'     dimnames = list(
#'         paste0("gene", seq_len(100)),
#'         paste0("sample", seq_len(50))
#'     )
#' )
#'
#' # Calculate correlation matrix
#' cor_mat <- exp2cor(exp)
#'
#' # Calculate adjacency matrix (random value for beta)
#' adj <- cor2adj(cor_mat, beta = 9)
cor2adj <- function(cor_matrix, beta, net_type = "signed hybrid") {

    if(!net_type %in% c("signed", "signed hybrid", "unsigned")) {
        stop(
            "Please, specify a valid argument to parameter `net_type`. ",
            "Possible network types: 'signed', 'signed hybrid', or 'unsigned'."
        )
    }

    # Get adjacency matrix from a correlation matrix
    adj_matrix <- WGCNA::adjacency.fromSimilarity(
        cor_matrix,
        type = net_type,
        power = beta
    )

    # Convert to matrix
    adj_matrix <- matrix(
        adj_matrix, nrow = nrow(adj_matrix),
        dimnames = list(rownames(adj_matrix), rownames(adj_matrix))
    )

    return(adj_matrix)
}

#' Wrapper to automatically guess TOM type based on network type
#'
#' @param net_type Character indicating the type of network to infer.
#' One of 'signed', 'signed hybrid' or 'unsigned'.
#'
#' @return Character scalar with TOM type.
#' @noRd
guess_tom <- function(net_type) {

    if(net_type == "signed hybrid") {
        TOMType <- "signed"

    } else if(net_type == "signed") {
        TOMType <- "signed Nowick"

    } else {
        TOMType <- "unsigned"
    }

    return(TOMType)
}


#' Wrapper to merge very similar modules
#'
#' @param exp A gene expression matrix with genes in rows and samples in
#' columns.
#' @param module_colors A character vector with module names for each gene
#' in the adjacency matrix.
#' @param ME A data frame of module eigengenes with samples in rows
#' and module eigengene names in columns.
#' @param dissimilarity Numeric indicating the dissimilarity threshold to merge
#' very similar modules. Default: 0.2.
#' @param cor_method Character indicating the correlation method to use.
#' Default: "pearson".
#'
#' @return A list as returned by \code{WGCNA::mergeCloseModules}
#' @importFrom WGCNA mergeCloseModules
#' @noRd
merge_modules <- function(
        exp, module_colors, ME, palette, dissimilarity = 0.2,
        cor_method = "pearson"
) {

    if(cor_method == "pearson") {
        merged <- WGCNA::mergeCloseModules(
            t(exp), module_colors, MEs = ME,
            cutHeight = dissimilarity,
            verbose = 0,
            colorSeq = palette
        )

    } else if(cor_method == "spearman") {
        merged <- WGCNA::mergeCloseModules(
            t(exp), module_colors, MEs = ME,
            cutHeight = dissimilarity,
            verbose = 0,
            corOptions = list(use = "p", method = "spearman"),
            colorSeq = palette
        )

    } else if(cor_method == "biweight") {
        merged <- WGCNA::mergeCloseModules(
            t(exp), module_colors, MEs = ME,
            cutHeight = dissimilarity,
            verbose = 0, corFnc = bicor,
            colorSeq = palette
        )
    }

    return(merged)
}


#' Transform a correlation matrix to an edge list
#'
#' @param matrix Symmetrical correlation matrix.
#'
#' @return A 2-column data frame containing node 1, node 2 and edge weight.
#' @export
#' @rdname cormat_to_edgelist
#' @examples
#' data(filt.se)
#' cor_mat <- cor(t(SummarizedExperiment::assay(filt.se)))
#' edgelist <- cormat_to_edgelist(cor_mat)
cormat_to_edgelist <- function(matrix) {

    edgelist <- matrix
    edgelist[lower.tri(edgelist, diag=TRUE)] <- NA
    edgelist <- na.omit(data.frame(as.table(edgelist), stringsAsFactors=FALSE))
    colnames(edgelist) <- c("Node1", "Node2", "Weight")
    edgelist$Node1 <- as.character(edgelist$Node1)
    edgelist$Node2 <- as.character(edgelist$Node2)
    edgelist$Weight <- as.numeric(edgelist$Weight)
    return(edgelist)
}


#' Check scale-free topology fit for a given network
#'
#' @param edgelist Edge list as a data frame containing node 1,
#' node 2 and edge weight.
#' @param net_type Type of biological network. One of "gcn", "grn", or "ppi".
#' Default: gcn.
#'
#' @return A list with SFT fit statistics and a message indicating if
#' the network is scale-free.
#' @rdname check_SFT
#' @export
#' @importFrom igraph graph_from_data_frame as_adjacency_matrix fit_power_law
#' @examples
#' set.seed(1)
#' exp <- t(matrix(rnorm(10000), ncol=1000, nrow=200))
#' rownames(exp) <- paste0("Gene", 1:nrow(exp))
#' colnames(exp) <- paste0("Sample", 1:ncol(exp))
#' cormat <- cor(t(exp))
#' edges <- cormat_to_edgelist(cormat)
#' edges <- edges[abs(edges$Weight) > 0.10, ]
#' check_SFT(edges)
check_SFT <- function(edgelist, net_type = "gcn") {

    # Calculate degree of the resulting graph
    if(net_type == "gcn") {
        graph <- igraph::graph_from_data_frame(edgelist, directed = FALSE)
        adj <- igraph::as_adjacency_matrix(graph, sparse = FALSE)
        diag(adj) <- 0
        degree <- apply(adj, 1, sum, na.rm = TRUE)
    } else if(net_type == "grn") {
        graph <- igraph::graph_from_data_frame(edgelist, directed = TRUE)
        degree <- igraph::degree(graph, mode = "out")
    } else if(net_type == "ppi") {
        graph <- igraph::graph_from_data_frame(edgelist, directed = FALSE)
        degree <- igraph::degree(graph)
    } else {
        stop("Invalid network type. Please, input one of 'gcn', 'grn', or 'ppi'.")
    }

    # Test for scale-free topology fit
    test <- igraph::fit_power_law(degree)
    if("KS.p" %in% names(test)) {
        if(test$KS.p < 0.05) {
            message("At the 95% confidence level for the Kolmogorov-Smirnov statistic, your graph does not fit the scale-free topology. P-value:", test$KS.p)
        } else {
            message("Your graph fits the scale-free topology. P-value:", test$KS.p)
        }
    } else {
        message("Could not obtain P-value for the Kolmogorov-Smirnov statistic.")
    }

    return(test)
}


#' Helper to handle list of SummarizedExperiment objects as input
#'
#' @param exp List of data frames or SummarizedExperiment objects.
#'
#' @return If exp is a list of SummarizedExperiment objects,
#' it will return a list of data frames. Otherwise, it will simply return exp as it is.
#' @noRd
#' @importFrom SummarizedExperiment assay
handleSElist <- function(exp) {

    if(is(exp[[1]], "SummarizedExperiment")) {
        list <- lapply(exp, handleSE)
    } else {
        list <- exp
    }
    return(list)
}


#' Helper function to handle metadata input for consensus modules identification
#'
#' @param exp List of data frames or SummarizedExperiment objects.
#' @param metadata A data frame containing sample names in row names and sample annotation in the first column.
#' @return Data frame of metadata for all expression sets.
#' @noRd
#' @importFrom SummarizedExperiment colData
handle_metadata <- function(exp, metadata) {

    if(is(exp[[1]], "SummarizedExperiment")) {
        metadata <- Reduce(rbind, lapply(exp, function(x) {
            meta <- as.data.frame(SummarizedExperiment::colData(x))
            return(meta)
        }))
    } else {
        metadata <- metadata
    }
    return(metadata)
}



#' Convert p-values in matrix to symbols
#'
#' @param matrix Matrix of p-values.
#'
#' @return Matrix of symbols.
#' @noRd
pval2symbol <- function(matrix) {
    modtraitsymbol <- matrix
    modtraitsymbol[modtraitsymbol < 0.001] <- "***"
    modtraitsymbol[modtraitsymbol >= 0.001 & modtraitsymbol < 0.01] <- "**"
    modtraitsymbol[modtraitsymbol >= 0.01 & modtraitsymbol < 0.05] <- "*"
    modtraitsymbol[modtraitsymbol >= 0.05] <- ""
    return(modtraitsymbol)
}

#----Load data------------------------------------------------------------------
set.seed(12)

# Simulate data
## Expression data
exp <- matrix(
    rnorm(1000, mean = 10, sd = 2), nrow = 100, ncol = 100, byrow = TRUE
)
rownames(exp) <- paste0("Gene", seq_len(nrow(exp)))
colnames(exp) <- paste0("Sample", seq_len(ncol(exp)))

## Sample metadata
col_metadata <- data.frame(
    row.names = colnames(exp),
    Class = paste0("Class", sample(1:5, size = ncol(exp), replace = TRUE)),
    Weight = stats::rnorm(ncol(exp), 50, 15)
)

## Gene metadata
row_metadata <- data.frame(
    row.names = rownames(exp),
    Pathway = paste0("Pathway", sample(1:5, size = nrow(exp), replace = TRUE))
)

## Simulate a SummarizedExperiment object
se <- SummarizedExperiment::SummarizedExperiment(
    assays = exp, colData = col_metadata, rowData = row_metadata
)

## Edge list
edges <- igraph::as_edgelist(igraph::sample_pa(n = 50))

#----Start tests----------------------------------------------------------------
test_that("metadata2colors() returns a list of metadata and named vectors", {

    cols <- metadata2colors(col_metadata)

    expect_equal(names(cols), c("metadata", "colors"))
    expect_equal(length(cols), 2)

    metadata_4columns <- cbind(
        col_metadata, col_metadata, col_metadata, col_metadata
    )

    expect_error(metadata2colors(metadata_4columns))
})


test_that("heatmap_attributes() returns heatmap parameters as a list", {

    p <- heatmap_attributes(exp)

    expect_equal(length(p), 4)
    expect_equal(names(p), c("pal", "mat", "title", "name"))

    expect_error(heatmap_attributes(exp, heatmap_type = "wrong"))
})


test_that("se2metadata() returns a list of row and coldata", {

    m <- se2metadata(se)

    expect_equal(length(m), 2)
    expect_equal(names(m), c("rowdata", "coldata"))
})

test_that("get_model_matrix() returns a model matrix for module-trait cor", {

    mat_char <- get_model_matrix(col_metadata, 1)
    mat_num <- get_model_matrix(col_metadata, 2)

    expect_equal(ncol(mat_char), 5)
    expect_equal(ncol(mat_num), 1)
})


test_that("exp2cor() and cor2adj() return symmetric matrices", {

    exp <- matrix(
        rnorm(100 * 50, mean = 10, sd = 2),
        nrow = 100,
        dimnames = list(
            paste0("gene", seq_len(100)),
            paste0("sample", seq_len(50))
        )
    )
    cor_mat <- exp2cor(exp)
    cor_mat2 <- exp2cor(exp, cor_method = "spearman")
    cor_mat3 <- exp2cor(exp, cor_method = "biweight")
    adj <- cor2adj(cor_mat, beta = 9)


    expect_error(cor2adj(cor_mat, beta = 9, net_type = "error"))
    expect_error(exp2cor(exp, cor_method = "error"))

    expect_true(is(cor_mat, "matrix"))
    expect_true(is(cor_mat2, "matrix"))
    expect_true(is(cor_mat3, "matrix"))
    expect_true(is(adj, "matrix"))

})


test_that("guess_tom() returns a character with TOM type", {

    t <- guess_tom("unsigned")

    expect_equal(t, "unsigned")
})


test_that("check_SFT() returns a list with SFT stats", {

    c1 <- check_SFT(edges, "gcn")
    c2 <- check_SFT(edges, "grn")
    c3 <- check_SFT(edges, "ppi")

    expect_error(check_SFT(edges, "error"))

    expect_equal(class(c1), "list")
    expect_equal(class(c2), "list")
    expect_equal(class(c3), "list")

})


test_that("handle_metadata() handles metadata", {

    h <- handle_metadata(exp, col_metadata)

    expect_equal(class(h), "data.frame")

})

#' Infer gene regulatory network with one of three algorithms
#'
#' The available algorithms are Context Likelihood of Relatedness (CLR),
#' ARACNE, or GENIE3.
#'
#' @param exp A gene expression data frame with genes in row names and
#' samples in column names or a `SummarizedExperiment` object.
#' @param regulators A character vector of regulators
#' (e.g., transcription factors or miRNAs). All regulators must be
#' included in `exp`.
#' @param method GRN inference algorithm to be used. One of "clr", "aracne",
#' or "genie3".
#' @param estimator_clr Entropy estimator to be used. One of "mi.empirical",
#' "mi.mm", "mi.shrink", "mi.sg", "pearson", "spearman", or "kendall".
#' Default: "pearson".
#' @param estimator_aracne Entropy estimator to be used. One of "mi.empirical",
#' "mi.mm", "mi.shrink", "mi.sg", "pearson", "spearman", or "kendall".
#' Default: "spearman".
#' @param eps Numeric value indicating the threshold used when removing
#' an edge: for each triplet of nodes (i,j,k), the weakest edge, say (ij),
#' is removed if its weight is below \code{min\{(ik),(jk)\}} - eps. Default: 0.1.
#' @param remove_zero Logical indicating whether to remove edges whose weight
#' is exactly zero. Default: TRUE
#' @param ... Additional arguments passed to `GENIE3::GENIE3()`.
#' @return A gene regulatory network represented as an edge list.
#' @rdname grn_infer
#' @export
#' @importFrom minet build.mim clr aracne
#' @importFrom GENIE3 GENIE3
#' @examples
#' data(filt.se)
#' tfs <- sample(rownames(filt.se), size=20, replace=FALSE)
#' clr <- grn_infer(filt.se, method = "clr", regulators=tfs)
#' aracne <- grn_infer(filt.se, method = "aracne", regulators=tfs)
#' # only 2 trees for demonstration purposes
#' genie3 <- grn_infer(filt.se, method = "genie3", regulators=tfs, nTrees=2)
grn_infer <- function(exp, regulators = NULL,
                      method = c("clr", "aracne", "genie3"),
                      estimator_clr = "pearson",
                      estimator_aracne = "spearman",
                      eps = 0.1,
                      remove_zero = TRUE, ...) {

    exp <- handleSE(exp)
    if(is.null(regulators)) {
        stop("Please, input a character vector of IDs of regulators.")
    }
    regulators <- regulators[regulators %in% rownames(exp)]

    if(method == "clr") {
        mi_mat <- minet::build.mim(t(exp), estimator = estimator_clr)
        grn <- minet::clr(mi_mat)
    } else if(method == "aracne") {
        # Build Mutual Information matrix and infer GRN
        mi_mat <- minet::build.mim(t(exp), estimator = estimator_aracne)
        grn <- minet::aracne(mi_mat, eps = eps)
    } else if(method == "genie3") {
        grn <- GENIE3::GENIE3(as.matrix(exp), regulators = regulators, ...)
    } else {
        stop("Method must be one of 'clr', 'aracne', or 'genie3'.")
    }

    # Keep only interactions between regulators and targets
    grn <- grn[regulators, !(colnames(grn) %in% regulators)]
    grn_edges <- cormat_to_edgelist(grn)

    # Should we remove edges that were removed by ARACNE?
    if(remove_zero) {
        grn_edges <- grn_edges[grn_edges$Weight != 0, ]
    }

    # Sort weights in decreasing order
    grn_edges <- grn_edges[order(-grn_edges$Weight), ]
    return(grn_edges)
}


#' Infer gene regulatory network with multiple algorithms and combine results in a list
#'
#' @param exp A gene expression data frame with genes in row names and
#' samples in column names or a `SummarizedExperiment` object.
#' @param regulators A character vector of regulators
#' (e.g., transcription factors or miRNAs). All regulators must be
#' included in `exp`.
#' @param eps Numeric value indicating the threshold used when removing
#' an edge: for each triplet of nodes (i,j,k), the weakest edge, say (ij),
#' is removed if its weight is below \code{min\{(ik),(jk)\}} - eps. Default: 0.1.
#' @param estimator_clr Entropy estimator to be used in CLR inference.
#' One of "mi.empirical", "mi.mm", "mi.shrink", "mi.sg", "pearson", "spearman",
#' or "kendall". Default: "pearson".
#' @param estimator_aracne Entropy estimator to be used in ARACNE inference.
#' One of "mi.empirical", "mi.mm", "mi.shrink", "mi.sg", "pearson", "spearman",
#' or "kendall". Default: "spearman".
#' @param remove_zero Logical indicating whether to remove edges
#' whose weight is exactly zero. Zero values indicate edges that were
#' removed by ARACNE. Default: TRUE.
#' @param ... Additional arguments passed to `GENIE3::GENIE3()`.
#' @return A list of data frames representing edge lists. Each list element
#' is an edge list for a specific method.
#' @rdname grn_combined
#' @export
#' @examples
#' data(filt.se)
#' tfs <- sample(rownames(filt.se), size=50, replace=FALSE)
#' grn_list <- grn_combined(filt.se, regulators=tfs, nTrees=2)
grn_combined <- function(exp, regulators = NULL,
                         eps = 0.1,
                         estimator_aracne = "spearman",
                         estimator_clr = "pearson",
                         remove_zero = TRUE, ...) {
    regulators <- regulators[regulators %in% rownames(exp)]
    methods <- c("genie3", "aracne", "clr")
    res_list <- lapply(methods, function(x) {
        grns <- grn_infer(
            exp, regulators = regulators, method = x,
            estimator_aracne = estimator_aracne,
            estimator_clr = estimator_clr,
            remove_zero = remove_zero, ...
        )
        return(grns)
    })
    names(res_list) <- methods
    return(res_list)
}

#' Rank edge weights for GRNs and calculate average across different methods
#'
#' @param list_edges List containing edge lists as returned by
#' the function \code{grn_combined}.
#'
#' @return Edge list containing regulator, target and mean rank from
#' all algorithms.
#' @rdname grn_average_rank
#' @export
#' @examples
#' data(filt.se)
#' tfs <- sample(rownames(filt.se), size=50, replace=FALSE)
#' grn_list <- grn_combined(filt.se, regulators=tfs, nTrees=2)
#' ranked_grn <- grn_average_rank(grn_list)
grn_average_rank <- function(list_edges) {

    # Add ranks to each edge list
    ranked_list <- lapply(list_edges, function(x) {
        x$Rank <- seq_len(nrow(x))
        x$Weight <- NULL
        return(x)
    })

    # Reduce list to a single data frame
    df_ranks <- Reduce(function(x, y) merge(x, y, by = c("Node1", "Node2")), ranked_list)
    colnames(df_ranks) <- c("Regulator", "Target", "Rank1", "Rank2", "Rank3")

    # Calculate mean and order from the highest mean rank to the lowest
    df_ranks$mean_rank <- apply(df_ranks[, 3:ncol(df_ranks)], 1, mean)
    df_ranks <- df_ranks[, c("Regulator", "Target", "mean_rank")]
    df_ranks <- df_ranks[order(df_ranks$mean_rank), ]
    return(df_ranks)
}



#' Filter a gene regulatory network based on optimal scale-free topology fit
#'
#' @param edgelist A gene regulatory network represented as an edge list.
#' @param nsplit Number of groups in which the edge list will be split.
#' Default: 10.
#' @param bp_param BiocParallel back-end to be used.
#' Default: BiocParallel::SerialParam()
#'
#' @details The edge list will be split in n groups and the scale-free
#' topology fit will be tested for each subset of the edge list.
#' For instance, if an edge list of 10000 rows is used as input,
#' the function will test SFT fit for the top 1000 edges, then top 2000 edges,
#' and so on up to the whole edge list.
#' @return The edge list that best fits the scale-free topology.
#' @export
#' @rdname grn_filter
#' @importFrom igraph graph_from_data_frame degree
#' @importFrom BiocParallel bplapply SerialParam
#' @importFrom WGCNA scaleFreeFitIndex
#' @importFrom ggplot2 theme element_text geom_point geom_line theme_bw
#' @examples
#' data(filt.se)
#' tfs <- sample(rownames(filt.se), size=50, replace=FALSE)
#' grn_list <- grn_combined(filt.se, regulators=tfs, nTrees=2)
#' ranked_grn <- grn_average_rank(grn_list)
#' # split in only 2 groups for demonstration purposes
#' filtered_edges <- grn_filter(ranked_grn, nsplit=2)
grn_filter <- function(edgelist, nsplit = 10,
                       bp_param = BiocParallel::SerialParam()) {

    # Split edge list into n data frames and calculate degree
    n_edges <- seq_len(nrow(edgelist))
    cutpoints <- round(
        unname(quantile(n_edges, probs = seq(0, 1, 1 / nsplit)))
    )[-1]

    list_degree <- BiocParallel::bplapply(cutpoints, function(x) {
        filt_edges <- edgelist[seq_len(x), c(1,2)]
        graph <- igraph::graph_from_data_frame(filt_edges, directed=TRUE)
        degree <- igraph::degree(graph, mode = "out")
    }, BPPARAM = bp_param)

    # Calculate scale-free topology fit for the degree list
    sft.rsquared <- unlist(BiocParallel::bplapply(list_degree, function(x) {
        return(WGCNA::scaleFreeFitIndex(x)$Rsquared.SFT)
    }, BPPARAM = bp_param))
    max.index <- which.max(sft.rsquared)

    # Plot scale-free topology fit for r values
    plot.data <- data.frame(x = cutpoints, y = sft.rsquared)

    plot <- ggplot(plot.data, aes(x = .data$x, y = .data$y, group = 1)) +
        geom_point(color = "firebrick", size = 4) +
        geom_line(color = "firebrick", linewidth = 2) +
        labs(
            x = "Number of top edges considered",
            y = expression(paste("Scale-free topology fit - ", R^{2})),
            title = "Scale-free topology fit for given r values"
        ) +
        theme_bw()

    plot

    optimal_cutoff <- cutpoints[max.index]
    message("The top number of edges that best fits the scale-free topology is ", optimal_cutoff)

    edgelist <- edgelist[seq_len(optimal_cutoff), c(1,2)]
    return(edgelist)
}


#' Infer gene regulatory network from expression data
#'
#' @param exp A gene expression data frame with genes in row names and
#' samples in column names or a `SummarizedExperiment` object.
#' @param estimator_clr Entropy estimator to be used in CLR inference.
#' One of "mi.empirical", "mi.mm", "mi.shrink", "mi.sg", "pearson",
#' "spearman", or "kendall". Default: "pearson".
#' @param estimator_aracne Entropy estimator to be used in ARACNE inference.
#' One of "mi.empirical", "mi.mm", "mi.shrink", "mi.sg", "pearson", "spearman",
#' or "kendall". Default: "spearman".
#' @param regulators A character vector of regulators
#' (e.g., transcription factors or miRNAs). All regulators must be
#' included in `exp`.
#' @param eps Numeric value indicating the threshold used when
#' removing an edge: for each triplet of nodes (i,j,k), the weakest edge,
#' say (ij), is removed if its weight is below \code{min\{(ik),(jk)\}} - eps. Default: 0.
#' @param remove_zero Logical indicating whether to remove edges whose
#' weight is exactly zero. Zero values indicate edges that were
#' removed by ARACNE. Default: TRUE.
#' @param nsplit Number of groups in which the edge list will be split.
#' Default: 10.
#' @param ... Additional arguments passed to `GENIE3::GENIE3()`.
#' @details
#' This function infers GRNs with ARACNE, GENIE3 and CLR, ranks correlation
#' weights for each GRN and calculates the average rank for each edge.
#' Then, the resulting GRN is filtered to keep the top n edges that lead
#' to the optimal scale-free topology fit.
#'
#' @rdname exp2grn
#' @export
#' @return A filtered edge list with regulators in the first column and
#' targets in the second column.
#' @examples
#' data(filt.se)
#' tfs <- sample(rownames(filt.se), size=50, replace=FALSE)
#' # Test with small number of trees for demonstration purpose
#' grn <- exp2grn(filt.se, regulators = tfs, nTrees=2, nsplit=2)
exp2grn <- function(exp, regulators = NULL,
                    eps = 0,
                    estimator_aracne = "spearman",
                    estimator_clr = "pearson",
                    remove_zero = TRUE,
                    nsplit = 10,
                    ...) {
    grn_list <- grn_combined(
        exp, regulators = regulators,
        eps = eps,
        estimator_aracne = estimator_aracne,
        estimator_clr = estimator_clr,
        remove_zero = remove_zero, ...
    )

    grn_ranks <- grn_average_rank(grn_list)
    final_grn <- grn_filter(grn_ranks, nsplit = nsplit)
    return(final_grn)
}

#' Get hubs for gene regulatory network
#'
#' @param edgelist A gene regulatory network represented as an edge list.
#' @param top_percentile Numeric from 0 to 1 indicating the percentage of genes
#' with the highest degree to consider hubs. Default: 0.1.
#' @param top_n Numeric indicating the number of genes with the highest degree
#' to consider hubs.
#' @param return_degree Logical indicating whether to return a data frame of
#' degree for all genes. If TRUE, the function will return a list instead of
#' a data frame. Default: FALSE.
#' @param ranked Logical indicating whether to treat third column of the
#' edge list (edge weights) as ranked values. Ignored if the edge list only
#' contains 2 columns. Default: TRUE.
#' @return A data frame with gene ID in the first column and out degree in
#' the second column or a list of two data frames with hubs and degree for
#' all genes, respectively.
#' @export
#' @rdname get_hubs_grn
#' @importFrom igraph graph_from_data_frame degree
#' @examples
#' data(filt.se)
#' tfs <- sample(rownames(filt.se), size=50, replace=FALSE)
#' grn_list <- grn_combined(filt.se, regulators=tfs, nTrees=2)
#' ranked_grn <- grn_average_rank(grn_list)
#' # split in only 2 groups for demonstration purposes
#' filtered_edges <- grn_filter(ranked_grn, nsplit=2)
#' hubs <- get_hubs_grn(filtered_edges)
get_hubs_grn <- function(edgelist, top_percentile = 0.1, top_n = NULL,
                         return_degree=FALSE, ranked=TRUE) {

    # Calculate degree
    if(ncol(edgelist) != 2) {
        if(ranked) {
            edgelist[,3] <- 10 / edgelist[,3]
        }
    }
    graph <- igraph::graph_from_data_frame(edgelist, directed = TRUE)
    degree <- sort(igraph::degree(graph, mode = "out"), decreasing = TRUE)

    # Find hubs
    degree_df <- data.frame(
        row.names = seq_along(degree),
        Gene = names(degree),
        Degree = degree
    )

    if(is.null(top_n)) {
        nrows <- nrow(degree_df) * top_percentile
        hubs <- degree_df[seq_len(nrows), ]
    } else {
        hubs <- degree_df[seq_len(top_n), ]
    }
    results <- hubs
    if(return_degree) {
        results <- list(Hubs=hubs,
                        Degree=degree_df)
    }
    return(results)
}


#' Get hubs for protein-protein interaction network
#'
#' @param edgelist A protein-protein interaction network represented
#' as an edge list.
#' @param top_percentile Numeric from 0 to 1 indicating the percentage of
#' proteins with the highest degree to consider hubs. Default: 0.1.
#' @param top_n Numeric indicating the number of proteins with the highest
#' degree to consider hubs.
#' @param return_degree Logical indicating whether to return a data frame
#' of degree for all proteins. If TRUE, the function will return a list
#' instead of a data frame. Default: FALSE.
#' @return A data frame with protein ID in the first column and degree
#' in the second column or a list of two data frames with hubs and degree
#' for all genes, respectively.
#' @export
#' @rdname get_hubs_grn
#' @importFrom igraph graph_from_data_frame degree
#' @examples
#' ppi_edges <- igraph::sample_pa(n = 500)
#' ppi_edges <- igraph::as_edgelist(ppi_edges)
#' hubs <- get_hubs_ppi(ppi_edges, return_degree = TRUE)
get_hubs_ppi <- function(edgelist, top_percentile = 0.1, top_n = NULL,
                         return_degree = FALSE) {

    # Calculate degree
    graph <- igraph::graph_from_data_frame(edgelist, directed = FALSE)
    degree <- sort(igraph::degree(graph), decreasing = TRUE)

    # Find hubs
    degree_df <- data.frame(
        row.names = seq_along(degree),
        Protein = names(degree),
        Degree=degree
    )

    if(is.null(top_n)) {
        nrows <- nrow(degree_df) * top_percentile
        hubs <- degree_df[seq_len(nrows), ]
    } else {
        hubs <- degree_df[seq_len(top_n), ]
    }
    results <- hubs
    if(return_degree) {
        results <- list(Hubs = hubs, Degree = degree_df)
    }
    return(results)
}

#---Load data----
set.seed(1)
data(filt.se)
tfs <- sample(rownames(filt.se), size=50, replace=FALSE)

#----Infer GRNs----
clr <- grn_infer(filt.se, method = "clr", regulators = tfs)
aracne <- grn_infer(filt.se, method = "aracne", regulators = tfs)
genie3 <- grn_infer(filt.se, method = "genie3", regulators = tfs, nTrees = 1)
grn_list <- grn_combined(filt.se, regulators = tfs, nTrees = 1)
ranked_grn <- grn_average_rank(grn_list)

#----Start tests----
test_that("cormat_to_edgelist() converts correlation matrix to edge list", {
    cor_mat <- cor(t(SummarizedExperiment::assay(filt.se)))
    edgelist <- cormat_to_edgelist(cor_mat)
    expect_equal(ncol(edgelist), 3)
    expect_equal(colnames(edgelist), c("Node1", "Node2", "Weight"))
})

test_that("grn_infer() produces an edge list", {

    expect_equal(colnames(clr), c("Node1", "Node2", "Weight"))
    expect_equal(is.character(clr[,1]), TRUE)
    expect_equal(is.character(clr[,2]), TRUE)
    expect_equal(is.numeric(clr[,3]), TRUE)
    expect_equal(ncol(aracne), 3)
    expect_equal(colnames(aracne), c("Node1", "Node2", "Weight"))
    expect_equal(is.character(aracne[,1]), TRUE)
    expect_equal(is.character(aracne[,2]), TRUE)
    expect_equal(is.numeric(aracne[,3]), TRUE)
    expect_equal(ncol(genie3), 3)
    expect_equal(is.character(genie3[,1]), TRUE)
    expect_equal(is.character(genie3[,2]), TRUE)
    expect_equal(is.numeric(genie3[,3]), TRUE)

    expect_error(grn_infer(filt.se, method = "random", regulators = tfs))
    expect_error(grn_infer(filt.se, method = "clr"))
})

test_that("grn_combined() produces a list of edge lists", {
    nrow1 <- nrow(grn_list[[1]])
    nrow2 <- nrow(grn_list[[2]])
    nrow3 <- nrow(grn_list[[3]])
    expect_equal(class(grn_list), "list")
    expect_equal(length(grn_list), 3)
})

test_that("all inference algorithms return regulators in the first column
          and target in the second column", {
    edge_structure <- function(grn, tfs) {
        x <- nrow(grn[grn[,2] %in% tfs, ])
        y <- nrow(grn[grn[,1] %in% tfs, ])
        result <- c(x,y)
        return(result)
    }
    expect_equal(edge_structure(genie3, tfs), c(0, nrow(genie3)))
    expect_equal(edge_structure(clr, tfs), c(0, nrow(clr)))
    expect_equal(edge_structure(aracne, tfs), c(0, nrow(aracne)))
})


test_that("grn_average_rank() ranks GRN weights and calculate the average across methods", {
    row_test <- sample(seq_len(nrow(ranked_grn)), size=1)
    expect_equal(ncol(ranked_grn), 3)
    expect_equal(ranked_grn[row_test, 3] <= ranked_grn[row_test+1, 3], TRUE)
})


test_that("check_SFT() checks SFT fit for different network types", {
    check <- check_SFT(clr, net_type = "grn")
    expect_equal(class(check), "list")
})


test_that("grn_filter() calculates the best SFT fit for n number of top edges", {
    filtered_edges <- grn_filter(ranked_grn, nsplit = 2)

    expect_equal(ncol(filtered_edges), 2)
    expect_lte(nrow(filtered_edges), nrow(ranked_grn))
})


test_that("get_hubs_grn() returns a data frame with hub genes and their out degree", {
    filtered_edges <- grn_filter(ranked_grn, nsplit = 2)
    n_genes <- length(unique(
        c(as.character(filtered_edges[,1]), as.character(filtered_edges[,2]))
    ))

    fedges <- filtered_edges
    fedges$rank <- seq_len(nrow(fedges))

    hubs <- get_hubs_grn(filtered_edges)
    hubs2 <- get_hubs_grn(filtered_edges, top_n = 1)
    hubs3 <- get_hubs_grn(fedges, top_n = 1)

    expect_equal(nrow(hubs), floor(n_genes * 0.1))
    expect_equal(nrow(hubs2), 1)
    expect_equal(nrow(hubs3), 1)
})

test_that("get_hubs_ppi() returns a list or data frame of hubs and degree", {
    ppi_edges <- igraph::as_edgelist(igraph::sample_pa(n=500))
    hubs <- get_hubs_ppi(ppi_edges, return_degree = TRUE)
    hubs2 <- get_hubs_ppi(ppi_edges, top_n = 1)

    expect_equal(class(hubs[[1]]), "data.frame")
    expect_equal(class(hubs), "list")
    expect_equal(nrow(hubs2), 1)
})


test_that("exp2grn() infers a GRN from expression", {
    grn <- exp2grn(filt.se, regulators = tfs, nTrees=2, nsplit=2)

    expect_equal(ncol(grn), 2)
    expect_equal(class(grn), "data.frame")
})

#' Pick power to fit network to a scale-free topology
#'
#' @param exp A gene expression data frame with genes in row names and
#' samples in column names or a `SummarizedExperiment` object.
#' @param net_type Network type. One of 'signed', 'signed hybrid' or
#' 'unsigned'. Default is signed.
#' @param rsquared R squared cutoff. Default is 0.8.
#' @param cor_method Correlation method. One of "pearson", "biweight"
#' or "spearman". Default is "spearman".
#'
#' @return A list containing:
#' \itemize{
#'   \item{\strong{power} Numeric, optimal power based on scale-free topology fit}
#'   \item{\strong{plot} A ggplot object displaying main statistics of the SFT fit test}
#' }
#'
#' @author Fabricio Almeida-Silva
#' @seealso
#'  \code{\link[WGCNA]{pickSoftThreshold}}
#' @rdname SFT_fit
#' @export
#' @importFrom WGCNA pickSoftThreshold
#' @importFrom ggplot2 theme element_text geom_hline
#' @examples
#' data(filt.se)
#' sft <- SFT_fit(filt.se, cor_method = "pearson")
SFT_fit <- function(exp, net_type = "signed", rsquared = 0.8,
                    cor_method = "spearman") {
    exp <- handleSE(exp)
    texp <- t(exp)

    if(cor_method == "pearson") {
        sft <- WGCNA::pickSoftThreshold(
            texp, networkType = net_type, powerVector = 3:20,
            RsquaredCut = rsquared
        )
    } else if(cor_method == "biweight") {
        sft <- WGCNA::pickSoftThreshold(
            texp, networkType = net_type, powerVector = 3:20,
            RsquaredCut = rsquared, corFnc = bicor,
            corOptions = list(use = 'p', maxPOutliers = 0.05)
        )
    } else if (cor_method == "spearman"){
        sft <- WGCNA::pickSoftThreshold(
            texp, networkType = net_type, powerVector=3:20,
            RsquaredCut = rsquared,
            corOptions = list(use = 'p', method = "spearman")
        )
    } else {
        stop("Please, specify a correlation method (one of 'spearman', 'pearson' or 'biweight').")
    }
    wgcna_power <- sft$powerEstimate
    if(is.na(wgcna_power)){
        message("No power reached R-squared cut-off, now choosing max R-squared based power")
        wgcna_power <- sft$fitIndices$Power[which(sft$fitIndices$SFT.R.sq == max(sft$fitIndices$SFT.R.sq))]
    }

    # Create data frame with indices to plot
    sft_df <- data.frame(
        power = sft$fitIndices$Power,
        fit = -sign(sft$fitIndices$slope) * sft$fitIndices$SFT.R.sq,
        meank = sft$fitIndices$mean.k.
    )
    sft_df <- sft_df[sft_df$fit > 0, ]

    # Plot 1
    p1 <- ggplot(sft_df, aes(x = .data$power, y = .data$fit)) +
        geom_point(color = "gray10") +
        ggrepel::geom_text_repel(aes(label = .data$power)) +
        labs(
            x = "Soft threshold (power)",
            y = expression(paste("Scale-free topology fit - ", R^{2})),
            title = "Scale independence"
        ) +
        theme_bw() +
        ylim(c(0,1)) +
        geom_hline(yintercept = rsquared, color = "brown3") +
        theme(legend.position = "none")

    # Plot 2
    p2 <- ggplot(sft_df, aes(x = .data$power, y = .data$meank)) +
        geom_point(color = "gray10") +
        ggrepel::geom_text_repel(aes(label = .data$power)) +
        labs(
            x = "Soft threshold (power)",
            y = "Mean connectivity (k)",
            title = "Mean connectivity"
        ) +
        theme_bw()

    # Combined plot
    sft_plot <- patchwork::wrap_plots(p1, p2, ncol = 2)
    result <- list(power = as.numeric(wgcna_power), plot = sft_plot)
    return(result)
}

#' Infer gene coexpression network from gene expression
#'
#' @param exp Either a `SummarizedExperiment` object, or a gene expression
#' matrix/data frame with genes in row names and samples in column names.
#' @param net_type Character indicating the type of network to infer.
#' One of 'signed', 'signed hybrid' or 'unsigned'. Default: 'signed'.
#' @param module_merging_threshold Numeric indicating the minimum correlation
#' threshold to merge similar modules into a single one. Default: 0.8.
#' @param SFTpower Numeric scalar indicating the value of the \eqn{\beta}
#' power to which correlation coefficients will be raised to ensure
#' scale-free topology fit. This value can be obtained with
#' the function \code{SFT_fit()}.
#' @param cor_method Character with correlation method to use.
#' One of "pearson", "biweight" or "spearman". Default: "spearman".
#' @param TOM_type Character specifying the method to use to calculate a
#' topological overlap matrix (TOM). If NULL, TOM type will be automatically
#' inferred from network type specified in \strong{net_type}. Default: NULL.
#' @param min_module_size Numeric indicating the minimum module size.
#' Default: 30.
#' @param return_cormat Logical indicating whether the correlation matrix
#' should be returned. If TRUE (default), an element named `correlation_matrix`
#' containing the correlation matrix will be included in the result list.
#' @param verbose Logical indicating whether to display progress
#' messages or not. Default: FALSE.
#'
#' @return A list containing the following elements: \itemize{
#'   \item \emph{adjacency_matrix} Numeric matrix with network adjacencies.
#'   \item \emph{MEs} Data frame of module eigengenes, with samples
#'   in rows, and module eigengenes in columns.
#'   \item \emph{genes_and_modules} Data frame with columns 'Genes' (character)
#'   and 'Modules' (character) indicating the genes and the modules to
#'   which they belong.
#'   \item \emph{kIN} Data frame of degree centrality for each gene,
#'   with columns 'kTotal' (total degree), 'kWithin' (intramodular degree),
#'   'kOut' (extra-modular degree), and 'kDiff' (difference between
#'   the intra- and extra-modular degree).
#'   \item \emph{correlation_matrix} Numeric matrix with pairwise
#'   correlation coefficients between genes.
#'   If parameter \strong{return_cormat} is FALSE, this will be NULL.
#'   \item \emph{params} List with network inference parameters passed
#'   as input.
#'   \item \emph{dendro_plot_objects} List with objects to plot the dendrogram
#'   in \code{plot_dendro_and_colors}. Elements are named 'tree' (an hclust
#'   object with gene dendrogram), 'Unmerged' (character with per-gene module
#'   assignments before merging similar modules), and 'Merged' (character
#'   with per-gene module assignments after merging similar modules).
#' }
#' @author Fabricio Almeida-Silva
#' @rdname exp2gcn
#' @export
#' @importFrom WGCNA TOMsimilarity standardColors labels2colors
#' moduleEigengenes mergeCloseModules intramodularConnectivity
#' @importFrom dynamicTreeCut cutreeDynamicTree
#' @importFrom stats as.dist median cor fisher.test hclust na.omit prcomp
#' qnorm qqplot quantile sd var
#' @importFrom grDevices colorRampPalette
#' @examples
#' data(filt.se)
#' # The SFT fit was previously calculated and the optimal power was 16
#' gcn <- exp2gcn(exp = filt.se, SFTpower = 18, cor_method = "pearson")
exp2gcn <- function(
        exp, net_type = "signed", module_merging_threshold = 0.8,
        SFTpower = NULL, cor_method = "spearman", TOM_type = NULL,
        min_module_size = 30,
        return_cormat = TRUE, verbose = FALSE
) {

    params <- list(
        net_type = net_type,
        module_merging_threshold = module_merging_threshold,
        SFTpower = SFTpower,
        cor_method = cor_method
    )
    exp <- handleSE(exp)

    if(is.null(SFTpower)) { stop("Please, specify the SFT power.") }

    # Calculate adjacency matrix
    if(verbose) { message("Calculating adjacency matrix...") }
    cor_matrix <- NULL
    if(return_cormat) {
        cor_matrix <- exp2cor(exp, cor_method = cor_method)
        adj_matrix <- cor2adj(cor_matrix, beta = SFTpower, net_type = net_type)
    } else {
        adj_matrix <- exp2cor(exp, cor_method = cor_method) |>
            cor2adj(beta = SFTpower, net_type = net_type)
    }

    # Calculate TOM from adjacency matrix
    if(verbose) { message("Calculating topological overlap matrix (TOM)...") }
    if(is.null(TOM_type)) { TOM_type <- guess_tom(net_type) }
    TOM <- WGCNA::TOMsimilarity(adj_matrix, TOMType = TOM_type)

    # Hierarchically cluster genes
    geneTree <- hclust(as.dist(1 - TOM), method = "average")
    geneTree$height <- round(geneTree$height, 7)

    # Detect coexpression modules
    if(verbose) { message("Detecting coexpression modules...") }
    original_mods <- dynamicTreeCut::cutreeDynamicTree(
        dendro = geneTree, minModuleSize = min_module_size, deepSplit = TRUE
    )

    nmod <- length(unique(original_mods))
    palette <- rev(WGCNA::standardColors(nmod))
    original_colors <- WGCNA::labels2colors(original_mods, colorSeq = palette)

    # Calculate module eigengenes and pairwise distances between them
    if(verbose) { message("Calculating module eigengenes (MEs)...") }
    me_list <- WGCNA::moduleEigengenes(
        t(exp), colors = original_colors, softPower = SFTpower
    )
    me <- me_list$eigengenes
    original_metree <- hclust(
        as.dist(1 - cor(me, method = "spearman")), method = "average"
    )

    # Merge very similar modules
    if(verbose) { message("Merging similar modules...") }
    merged <- merge_modules(
        exp, original_colors, me, palette,
        dissimilarity = 1 - module_merging_threshold, cor_method = cor_method
    )
    new_colors <- merged$colors
    new_mes <- merged$newMEs

    # Get data frame with genes and modules they belong to
    genes_modules <- data.frame(Genes = rownames(exp), Modules = new_colors)

    if(verbose) { message("Calculating intramodular connectivity...") }
    kwithin <- WGCNA::intramodularConnectivity(adj_matrix, new_colors)

    result_list <- list(
        adjacency_matrix = adj_matrix,
        MEs = new_mes,
        genes_and_modules = genes_modules,
        kIN = kwithin,
        correlation_matrix = cor_matrix,
        params = params,
        dendro_plot_objects = list(
            tree = geneTree, Unmerged = original_colors, Merged = new_colors
        )
    )

    return(result_list)
}

#' Infer gene coexpression network from gene expression in a blockwise manner
#'
#' @param exp Either a `SummarizedExperiment` object, or a gene expression
#' matrix/data frame with genes in row names and samples in column names.
#' @param net_type Character indicating the type of network to infer.
#' One of 'signed', 'signed hybrid' or 'unsigned'. Default: 'signed'.
#' @param module_merging_threshold Numeric indicating the minimum correlation
#' threshold to merge similar modules into a single one. Default: 0.8.
#' @param SFTpower Numeric scalar indicating the value of the \eqn{\beta}
#' power to which correlation coefficients will be raised to ensure
#' scale-free topology fit. This value can be obtained with
#' the function \code{SFT_fit()}.
#' @param cor_method Character with correlation method to use.
#' One of "pearson" or "biweight". Default: "pearson".
#' @param TOM_type Character specifying the method to use to calculate a
#' topological overlap matrix (TOM). If NULL, TOM type will be automatically
#' inferred from network type specified in \strong{net_type}. Default: NULL.
#' @param max_block_size Numeric indicating the maximum block size for module
#' detection.
#' @param min_module_size Numeric indicating the minimum module size.
#' Default: 30.
#' @param ... Additional arguments to \code{WGCNA::blockwiseModules()}.
#'
#' @return A list containing the following elements: \itemize{
#'   \item \emph{MEs} Data frame of module eigengenes, with samples
#'   in rows, and module eigengenes in columns.
#'   \item \emph{genes_and_modules} Data frame with columns 'Genes' (character)
#'   and 'Modules' (character) indicating the genes and the modules to
#'   which they belong.
#'   \item \emph{params} List with network inference parameters passed
#'   as input.
#'   \item \emph{dendro_plot_objects} List with objects to plot the dendrogram
#'   in \code{plot_dendro_and_colors}. Elements are named 'tree' (an hclust
#'   object with gene dendrogram), 'Unmerged' (character with per-gene module
#'   assignments before merging similar modules), and 'Merged' (character
#'   with per-gene module assignments after merging similar modules).
#' }
#'
#' @author Fabricio Almeida-Silva
#' @rdname exp2gcn_blockwise
#' @export
#' @importFrom WGCNA blockwiseModules
#' @examples
#' data(filt.se)
#' # The SFT fit was previously calculated and the optimal power was 16
#' cor <- WGCNA::cor
#' gcn <- exp2gcn_blockwise(
#'     exp = filt.se, SFTpower = 18, cor_method = "pearson"
#' )
exp2gcn_blockwise <- function(
        exp, net_type = "signed", module_merging_threshold = 0.8,
        SFTpower = NULL, cor_method = "pearson", TOM_type = NULL,
        max_block_size = 5000, min_module_size = 30, ...
) {

    params <- list(
        net_type = net_type,
        module_merging_threshold = module_merging_threshold,
        SFTpower = SFTpower,
        cor_method = cor_method
    )
    exp <- handleSE(exp)

    if(is.null(SFTpower)) { stop("Please, specify the SFT power.") }

    cortype <- "pearson"
    if(cor_method == "biweight") {
        cortype <- "bicor"
    } else if(!cor_method %in% c("pearson", "biweight")) {
        stop("Argument to `cor_method` must be one of 'pearson' or 'biweight'.")
    }

    if(is.null(TOM_type)) { TOM_type <- guess_tom(net_type) }

    # Detect modules
    cor <- WGCNA::cor
    bmod <- WGCNA::blockwiseModules(
        datExpr = as.matrix(t(exp)),
        checkMissingData = FALSE,
        maxBlockSize = max_block_size,
        corType = cortype,
        maxPOutliers = 0.1,
        power = SFTpower,
        networkType = net_type,
        TOMType = TOM_type,
        minModuleSize = min_module_size,
        mergeCutHeight = 1 - module_merging_threshold,
        ...
    )

    # Create result list
    genes_modules <- data.frame(
        Genes = names(bmod$colors), Modules = bmod$colors, row.names = NULL
    )

    result_list <- list(
        MEs = bmod$MEs,
        genes_and_modules = genes_modules,
        params = params,
        dendro_plot_objects = list(
            tree = bmod$dendrograms[[1]],
            Unmerged = bmod$unmergedColors,
            Merged = bmod$colors
        )
    )

    return(result_list)
}



#' Plot eigengene network
#'
#' @param gcn List object returned by \code{exp2gcn}.
#' @param palette Character indicating the name of the RColorBrewer palette
#' to use. Default: "PRGn".
#'
#' @return A base plot with the eigengene network
#' @importFrom stats hclust
#' @importFrom ComplexHeatmap pheatmap
#' @export
#' @rdname plot_eigengene_network
#' @examples
#' data(filt.se)
#' gcn <- exp2gcn(filt.se, SFTpower = 18, cor_method = "pearson")
#' plot_eigengene_network(gcn)
plot_eigengene_network <- function(gcn, palette = "PRGn") {

    # Calculate pairwise correlations between eigengenes and get hclust object
    mes <- gcn$MEs[, !names(gcn$MEs) %in% "MEgrey"]
    me_correlation <- cor(mes)
    clusters <- hclust(as.dist(1 - me_correlation), method = "average")

    # Create column metadata and row metadata
    coldata <- data.frame(row.names = names(mes), Module = names(mes))
    rowdata <- data.frame(row.names = names(mes), Module = names(mes))

    colors <- list(
        Module = setNames(gsub("ME", "", names(mes)), names(mes))
    )

    # Plot heatmap with distances
    pal <- colorRampPalette(brewer.pal(n = 7, name = palette))(100)
    hm <- ComplexHeatmap::pheatmap(
        me_correlation, name = "Correlation",
        color = pal,
        cluster_cols = clusters,
        cluster_rows = clusters,
        main = "Pairwise correlations between module eigengenes",
        legend_breaks = seq(-1, 1, 0.5),
        breaks = seq(-1, 1, 0.02),
        annotation_row = rowdata,
        annotation_col = coldata,
        legend = TRUE,
        annotation_colors = colors
    )

    # Remove legend for row and column annotation
    hm@top_annotation@anno_list$Module@show_legend <- FALSE
    hm@left_annotation@anno_list$Module@show_legend <- FALSE

    return(hm)
}

#' Plot dendrogram of genes and modules
#'
#' @param gcn List object returned by \code{exp2gcn}.
#'
#' @return A base plot with the gene dendrogram and modules.
#' @importFrom ggdendro dendro_data
#' @importFrom ggplot2 geom_segment aes theme_minimal labs scale_x_continuous
#' labs theme coord_cartesian element_blank geom_tile scale_fill_identity
#' scale_y_discrete geom_line element_rect
#' @importFrom patchwork wrap_plots
#' @export
#' @rdname plot_dendro_and_colors
#' @examples
#' data(filt.se)
#' gcn <- exp2gcn(filt.se, SFTpower = 18, cor_method = "pearson")
#' plot_dendro_and_colors(gcn)
plot_dendro_and_colors <- function(gcn) {

    # Create a dendrogram with ggplot2
    ## Get coordinates as a data frame
    ddata <- ggdendro::dendro_data(
        gcn$dendro_plot_objects$tree, type = "rectangle"
    )$segments

    ## To make sure lines in the y-axis do not extend to 0 and get too long
    ddata$yend[ddata$yend < 0.05] <- ddata$y[ddata$yend < 0.1] - 0.02

    ## Plot
    p_dendro <- ggplot(ddata) +
        geom_segment(aes(
            x = .data$x, y = .data$y, xend = .data$xend, yend = .data$yend
        ), linewidth = 0.3) +
        coord_cartesian(ylim = c(0, 1)) +
        theme_minimal() +
        labs(x = "", y = "Height", title = "Dendrogram of genes and modules") +
        scale_x_continuous(expand = c(0, 0)) +
        theme(
            panel.grid = element_blank(),
            axis.text.x = element_blank()
        )

    # Create a heatmap
    ## Heatmap data frame: 3 columns named `col`, `class`, and `value`
    o <- gcn$dendro_plot_objects$tree$order

    heatmap_df <- lapply(seq_along(gcn$dendro_plot_objects)[-1], function(x) {
        modules <- gcn$dendro_plot_objects[[x]][o]
        setname <- names(gcn$dendro_plot_objects)[x]
        df <- data.frame(
            col = factor(modules, levels = unique(modules)),
            class = setname,
            value = seq_along(modules)
        )
        return(df)
    })
    heatmap_df <- Reduce(rbind, heatmap_df)

    ## Create a data with coordinates for lines between rows of the heatmap
    nrows <- length(unique(heatmap_df$class))
    ngenes <- length(unique(heatmap_df$value))
    line_df <- data.frame(
        x = c(0, ngenes) + 0.5,
        y = rep(2:nrows, each = 2) - 0.5
    )

    ## Plot
    p_hm <- ggplot(heatmap_df) +
        geom_tile(aes(x = .data$value, y = .data$class, fill = .data$col)) +
        scale_fill_identity() +
        theme_minimal() +
        labs(x = "", y = "") +
        theme(
            panel.background = element_rect(fill = "black"),
            axis.text.x = element_blank(),
            panel.grid = element_blank()
        ) +
        scale_x_continuous(expand = c(0, 0)) +
        scale_y_discrete(expand = c(0.2, 0.2)) +
        geom_line(data = line_df, aes(x = .data$x, y = .data$y, group = .data$y))

    ## Combine plots
    heights <- c(3, 1)
    if(length(gcn$dendro_plot_objects) > 3) { heights <- c(2, 1) }
    p_combined <- wrap_plots(p_dendro, p_hm, nrow = 2, heights = heights)

    return(p_combined)
}

#' Perform module stability analysis
#'
#' @param exp A gene expression data frame with genes in row names and
#' samples in column names or a `SummarizedExperiment` object.
#' @param net List object returned by \code{exp2gcn}.
#' @param nRuns Number of times to resample. Default is 20.
#'
#' @return A base plot with the module stability results.
#' @seealso
#'  \code{\link[WGCNA]{sampledBlockwiseModules}}
#' @rdname module_stability
#' @export
#' @importFrom WGCNA sampledBlockwiseModules matchLabels
#' @examples
#' data(filt.se)
#' filt <- filt.se[1:100, ] # reducing even further for testing purposes
#' # The SFT fit was previously calculated and the optimal power was 16
#' gcn <- exp2gcn(filt, SFTpower = 16, cor_method = "pearson")
#' # For simplicity, only 2 runs
#' module_stability(exp = filt, net = gcn, nRuns = 2)
module_stability <- function(exp, net, nRuns = 20) {

    norm.exp <- handleSE(exp)
    expr <- as.matrix(t(norm.exp))
    cor_method <- net$params$cor_method
    if(cor_method == "biweight") { cor_method <- "bicor" }

    # Infer modules for original and resampled data n times (n = nRuns)
    mods0 <- WGCNA::sampledBlockwiseModules(
        nRuns = nRuns,
        replace = FALSE,
        datExpr = expr,
        maxBlockSize = 5000,
        checkSoftPower = FALSE,
        corType = cor_method,
        networkType = net$params$net_type,
        TOMType = guess_tom(net$params$net_type),
        TOMDenom = "mean",
        mergeCutHeight = 1 - net$params$module_merging_threshold,
        reassignThreshold = 0,
        numericLabels = FALSE,
        checkMissingData = FALSE,
        quickCor = 0, verbose = 2
    )
    nGenes <- ncol(expr)

    # Create a matrix of labels for the original and all resampling runs
    labels <- matrix(0, nGenes, nRuns + 1)
    labels[, 1] <- mods0[[1]]$mods$colors

    ## Relabel modules in each of the resampling runs
    labs <- Reduce(cbind, lapply(2:(nRuns+1), function(x) {
        labels[, x] <- WGCNA::matchLabels(mods0[[x-1]]$mods$colors, labels[, 1])
    }))
    labels <- cbind(labels[,1], labs)
    colnames(labels) <- c("Original", paste0("Resampling ", seq_len(nRuns)))

    ## Convert matrix to list of vectors
    label_list <- as.list(as.data.frame(labels))

    # Simulate the output of `exp2gcn()` for plotting
    sim_gcn <- list(
        dendro_plot_objects = c(
            list(tree = mods0[[1]]$mods$dendrograms[[1]]), label_list
        )
    )

    # Plot dendrogram of genes and modules in each run
    p <- plot_dendro_and_colors(sim_gcn)

    return(p)
}

#' Correlate module eigengenes to trait
#'
#' @param exp A gene expression data frame with genes in row names and
#' samples in column names or a `SummarizedExperiment` object.
#' @param metadata A data frame containing sample names in row names and
#' sample annotation in the first column. Ignored if `exp` is
#' a `SummarizedExperiment` object, since the function will extract colData.
#' @param MEs Module eigengenes. It is the 2nd element of the result list
#' generated by the function \code{exp2gcn}.
#' @param metadata_cols A vector (either numeric or character) indicating
#' which columns should be extracted from column metadata if \strong{exp}
#' is a `SummarizedExperiment` object. The vector can contain column
#' indices (numeric) or column names (character). By default, all columns are
#' used.
#' @param cor_method Method to calculate correlation. One of 'pearson',
#' 'spearman' or 'kendall'. Default is 'spearman'.
#'
#' @return A data frame with correlation and correlation p-values for each pair
#' of ME and trait, with the following variables:
#' \describe{
#'   \item{ME}{Factor, module eigengene.}
#'   \item{trait}{Factor, trait name. Each trait corresponds to a variable
#'                of the sample metadata (if numeric) or levels of a variable
#'                (if categorical).}
#'   \item{cor}{Numeric, correlation.}
#'   \item{pvalue}{Numeric, correlation P-values.}
#'   \item{group}{Character, name of the metadata variable.}
#' }
#'
#' @author Fabricio Almeida-Silva
#' @rdname module_trait_cor
#' @export
#' @importFrom WGCNA corPvalueStudent
#' @importFrom reshape2 melt
#' @examples
#' data(filt.se)
#' gcn <- exp2gcn(filt.se, SFTpower = 18, cor_method = "pearson")
#' module_trait_cor(filt.se, MEs = gcn$MEs)
module_trait_cor <- function(exp, metadata, MEs, metadata_cols = NULL,
                             cor_method = "pearson") {

    if(is(exp, "SummarizedExperiment")) {
        metadata <- se2metadata(exp, coldata_cols = metadata_cols)$coldata
    }

    exp <- handleSE(exp)
    metadata <- metadata[colnames(exp), , drop = FALSE]

    # Get data frame of module-trait correlations and P-values for each variable
    cor_df <- Reduce(rbind, lapply(seq_along(metadata), function(x) {

        ## Get matrices of correlations, P-values, and P-value symbols
        trait <- get_model_matrix(metadata, column_idx = x)
        cor_matrix <- cor(as.matrix(MEs), trait, use = "p", method = cor_method)
        pvalues <- WGCNA::corPvalueStudent(cor_matrix, nSamples = ncol(exp))

        ## Reshape to long format and merge data frames into one
        v <- c("ME", "trait")
        cor_long <- reshape2::melt(cor_matrix, value.name = "cor", varnames = v)
        p_long <- reshape2::melt(pvalues, value.name = "pvalue", varnames = v)

        final_df <- merge(cor_long, p_long)
        final_df$group <- names(metadata)[x]

        return(final_df)
    }))

    return(cor_df)
}

#' Plot a heatmap of module-trait correlations
#'
#' @param corandp A data frame of module-trait correlations as returned
#' by \code{module_trait_cor()}.
#' @param palette Character indicating which RColorBrewer palette to use.
#' Default: 'RdYlBu'.
#' @param transpose Logical indicating whether to transpose the heatmap
#' or not.
#'
#' @details Significance levels:
#' 1 asterisk: significant at alpha = 0.05.
#' 2 asterisks: significant at alpha = 0.01.
#' 3 asterisks: significant at alpha = 0.001.
#' no asterisk: not significant.
#'
#' @return A `Heatmap` object created by \code{ComplexHeatmap::pheatmap()}.
#'
#' @export
#' @rdname plot_module_trait_cor
#' @importFrom reshape2 dcast
#' @importFrom RColorBrewer brewer.pal
#' @importFrom ComplexHeatmap pheatmap
#' @examples
#' data(filt.se)
#' gcn <- exp2gcn(filt.se, SFTpower = 18, cor_method = "pearson")
#' corandp <- module_trait_cor(filt.se, MEs = gcn$MEs)
#' plot_module_trait_cor(corandp)
plot_module_trait_cor <- function(corandp, palette = "RdYlBu",
                                  transpose = FALSE) {

    # Get a wide matrix of correlations
    cormat <- reshape2::dcast(corandp, ME ~ trait, value.var = "cor")
    rownames(cormat) <- cormat$ME
    cormat$ME <- NULL
    cormat <- as.matrix(cormat)
    cormat[is.na(cormat)] <- 0

    # Create a matrix of correlations and P-value symbols to display
    ## Get a matrix of P-values and convert values to symbols
    pmat <- reshape2::dcast(corandp, ME ~ trait, value.var = "pvalue")
    rownames(pmat) <- pmat$ME
    pmat$ME <- NULL
    pmat <- pval2symbol(as.matrix(pmat))
    pmat[is.na(pmat)] <- ""

    ## Get a matrices of correlations and P-values
    textmat <- paste(signif(cormat, 2), pmat, sep = "")
    dim(textmat) <- dim(cormat)

    # Get row and column annotation to decorate the heatmap
    rowdata <- data.frame(row.names = rownames(cormat), Module = rownames(cormat))
    coldata <- corandp[!duplicated(corandp$trait), ]
    coldata <- data.frame(row.names = coldata$trait, Trait = coldata$group)

    # Create a list of named vectors with colors to use in the heatmap
    cols <- list(
        Module = setNames(gsub("ME", "", rownames(cormat)), rownames(cormat)),
        Trait = setNames(
            custom_palette(1)[seq_along(unique(corandp$group))],
            unique(corandp$group)
        )
    )

    # Plot heatmap
    pal <- colorRampPalette(rev(RColorBrewer::brewer.pal(10, palette)))(100)

    hm <- ComplexHeatmap::pheatmap(
        mat = if(transpose) t(cormat) else cormat,
        name = "Correlation",
        color = pal,
        display_numbers = if(transpose) t(textmat) else textmat,
        main = "Module-trait correlations",
        legend_breaks = seq(-1, 1, 0.5),
        breaks = seq(-1, 1, 0.02),
        annotation_row = if(transpose) coldata else rowdata,
        annotation_col = if(transpose) rowdata else coldata,
        annotation_colors = cols,
        row_split = if(transpose) coldata$Trait else NULL,
        column_split = if(transpose) NULL else coldata$Trait
    )

    # Remove legend for module colors
    if(transpose) {
        hm@top_annotation@anno_list$Module@show_legend <- FALSE
    } else {
        hm@left_annotation@anno_list$Module@show_legend <- FALSE
    }

    return(hm)
}


#' Calculate gene significance for a given group of genes
#'
#' @param exp A gene expression data frame with genes in row names and
#' samples in column names or a `SummarizedExperiment` object.
#' @param metadata A data frame containing sample names in row names and
#' sample annotation in the first column. Ignored if `exp` is
#' a `SummarizedExperiment` object, since the function will extract colData.
#' @param metadata_cols A vector (either numeric or character) indicating
#' which columns should be extracted from column metadata if \strong{exp}
#' is a `SummarizedExperiment` object. The vector can contain column
#' indices (numeric) or column names (character). By default, all columns are
#' used.
#' @param genes Character vector of genes to be correlated with traits.
#' If not given, all genes in `exp` will be considered.
#' @param alpha Significance level. Default is 0.05.
#' @param cor_method Method to calculate correlation. One of 'pearson',
#' 'spearman' or 'kendall'. Default is 'spearman'.
#' @param min_cor Minimum correlation coefficient. Default is 0.2.
#' @param use_abs Logical indicating whether to filter by correlation using
#' absolute value or not. If TRUE, a \code{min_cor} of say 0.2 would keep all
#' correlations above 0.2 and below -0.2. Default is TRUE.
#'
#' @return A data frame with correlation and correlation p-values for each pair
#' of gene and trait, with the following variables:
#' \describe{
#'   \item{gene}{Factor, gene ID.}
#'   \item{trait}{Factor, trait name. Each trait corresponds to a variable
#'                of the sample metadata (if numeric) or levels of a variable
#'                (if categorical).}
#'   \item{cor}{Numeric, correlation.}
#'   \item{pvalue}{Numeric, correlation P-values.}
#'   \item{group}{Character, name of the metadata variable.}
#' }
#'
#' @author Fabricio Almeida-Silva
#' @rdname gene_significance
#' @export
#' @importFrom WGCNA corPvalueStudent
#' @importFrom reshape2 melt
#' @examples
#' data(filt.se)
#' gs <- gene_significance(filt.se)
gene_significance <- function(
        exp, metadata, metadata_cols = NULL, genes = NULL, alpha = 0.05,
        cor_method = "pearson", min_cor = 0.2, use_abs = TRUE
) {

    if(is(exp, "SummarizedExperiment")) {
        metadata <- se2metadata(exp, coldata_cols = metadata_cols)$coldata
    }

    # Make sure samples in expression matrix and sample metadata match
    exp <- handleSE(exp)
    metadata <- metadata[colnames(exp), , drop = FALSE]
    final_exp <- exp[, rownames(metadata)]

    # Use only a subset of genes?
    if(!is.null(genes)) {
        final_exp <- final_exp[genes, , drop = FALSE]
    }

    # Get data frame of gene-trait correlations and P-values for each variable
    cor_df <- Reduce(rbind, lapply(seq_along(metadata), function(x) {

        ## Get matrices of correlations, P-values, and P-value symbols
        trait <- get_model_matrix(metadata, column_idx = x)
        cor_matrix <- cor(
            as.matrix(t(exp)), trait, use = "p", method = cor_method
        )
        pvalues <- WGCNA::corPvalueStudent(cor_matrix, nSamples = ncol(exp))

        ## Reshape to long format and merge data frames into one
        v <- c("gene", "trait")
        cor_long <- reshape2::melt(cor_matrix, value.name = "cor", varnames = v)
        p_long <- reshape2::melt(pvalues, value.name = "pvalue", varnames = v)

        final_df <- merge(cor_long, p_long)
        final_df$group <- names(metadata)[x]

        return(final_df)
    }))

    # Filter by correlation and P-value
    if(use_abs) {
        cor_df <- cor_df[cor_df$pvalue < alpha & abs(cor_df$cor) >= min_cor, ]
    } else {
        cor_df <- cor_df[cor_df$pvalue < alpha & cor_df$cor >= min_cor, ]
    }

    return(cor_df)
}


#' Plot a heatmap of gene significance
#'
#' @param corandp A data frame of gene-trait correlations as returned
#' by \code{gene_significance()}.
#' @param palette Character indicating which RColorBrewer palette to use.
#' Default: 'RdYlBu'.
#' @param transpose Logical indicating whether to transpose the heatmap
#' or not.
#' @param ... Additional arguments to \code{ComplexHeatmap::pheatmap()}.
#'
#' @details Significance levels:
#' 1 asterisk: significant at alpha = 0.05.
#' 2 asterisks: significant at alpha = 0.01.
#' 3 asterisks: significant at alpha = 0.001.
#' no asterisk: not significant.
#'
#' @return A `Heatmap` object created by \code{ComplexHeatmap::pheatmap()}.
#'
#' @export
#' @rdname plot_gene_significance
#' @importFrom reshape2 dcast
#' @importFrom RColorBrewer brewer.pal
#' @importFrom ComplexHeatmap pheatmap
#' @examples
#' data(filt.se)
#' gcn <- exp2gcn(filt.se, SFTpower = 18, cor_method = "pearson")
#' corandp <- gene_significance(filt.se)
#' plot_gene_significance(corandp, show_rownames = FALSE)
plot_gene_significance <- function(
        corandp, palette = "RdYlBu", transpose = FALSE, ...
) {

    # Get a wide matrix of correlations
    cormat <- reshape2::dcast(corandp, gene ~ trait, value.var = "cor")
    rownames(cormat) <- cormat$gene
    cormat$gene <- NULL
    cormat <- as.matrix(cormat)
    cormat[is.na(cormat)] <- 0

    # Create a matrix of correlations and P-value symbols to display
    ## Get a matrix of P-values and convert values to symbols
    pmat <- reshape2::dcast(corandp, gene ~ trait, value.var = "pvalue")
    rownames(pmat) <- pmat$gene
    pmat$gene <- NULL
    pmat <- pval2symbol(as.matrix(pmat))
    pmat[is.na(pmat)] <- ""

    ## Get a matrices of correlations and P-values
    textmat <- paste(signif(cormat, 2), pmat, sep = "")
    dim(textmat) <- dim(cormat)

    # Get column annotation to decorate the heatmap
    coldata <- corandp[!duplicated(corandp$trait), ]
    coldata <- data.frame(row.names = coldata$trait, Trait = coldata$group)
    coldata <- coldata[colnames(cormat), , drop = FALSE]

    # Create a list of named vectors with colors to use in the heatmap
    cols <- list(
        Trait = setNames(
            custom_palette(1)[seq_along(unique(corandp$group))],
            unique(corandp$group)
        )
    )

    # Plot heatmap
    pal <- colorRampPalette(rev(RColorBrewer::brewer.pal(10, palette)))(100)

    hm <- ComplexHeatmap::pheatmap(
        mat = if(transpose) t(cormat) else cormat,
        name = "Correlation",
        color = pal,
        display_numbers = FALSE,
        main = "Gene-trait correlations",
        legend_breaks = seq(-1, 1, 0.5),
        breaks = seq(-1, 1, 0.02),
        annotation_row = if(transpose) coldata else NA,
        annotation_col = if(transpose) NA else coldata,
        annotation_colors = cols,
        row_split = if(transpose) coldata$Trait else NULL,
        column_split = if(transpose) NULL else coldata$Trait,
        ...
    )

    return(hm)
}


#' Get GCN hubs
#'
#' @param exp A gene expression data frame with genes in row names and
#' samples in column names or a `SummarizedExperiment` object.
#' @param net List object returned by \code{exp2gcn}.
#'
#' @return Data frame containing gene IDs, modules and
#' intramodular connectivity of all hubs.
#' @author Fabricio Almeida-Silva
#' @seealso
#'  \code{\link[WGCNA]{signedKME}}
#' @rdname get_hubs_gcn
#' @export
#' @importFrom WGCNA signedKME
#' @examples
#' data(filt.se)
#' gcn <- exp2gcn(filt.se, SFTpower = 18, cor_method = "pearson")
#' hubs <- get_hubs_gcn(filt.se, gcn)
get_hubs_gcn <- function(exp, net) {

    exp <- handleSE(exp)
    cor_method <- net$params$cor_method
    genes_modules <- net$genes_and_modules
    MEs <- net$MEs
    kIN <- net$kIN[, 2, drop=FALSE]

    # Add kIN info
    genes_modulesk <- merge(
        genes_modules, kIN, by.x = "Genes", by.y = "row.names"
    )

    # Calculate kME
    if(cor_method == "spearman") {
        MM <- WGCNA::signedKME(
            t(exp), MEs, outputColumnName = "",
            corOptions = "use = 'p', method = 'spearman'"
        )
    } else if(cor_method == "pearson") {
        MM <- WGCNA::signedKME(t(exp), MEs, outputColumnName = "")
    } else if(cor_method == "biweight") {
        MM <- WGCNA::signedKME(
            t(exp), MEs, corFnc = "bicor", outputColumnName = "",
            corOptions = "maxPOutliers = 0.1"
        )
    }

    # Add kME info
    kme_info <- merge(genes_modulesk, MM, by.x="Genes", by.y="row.names")

    # Keep top 10% genes with the highest degree for each module
    genes_mod_list <- split(kme_info, kme_info[,2])
    genes_mod_list$grey <- NULL
    top_10 <- lapply(genes_mod_list, function(x) {
        indices <- round(nrow(x) * 0.1)
        return(x[order(x$kWithin, decreasing=TRUE), ][seq_len(indices), ])
    })

    # Pick genes from the top 10% degree with kME above 0.8
    hubs <- Reduce(rbind, lapply(top_10, function(x) {
        cols <- c("Genes", "Modules", "kWithin", unique(x$Modules))
        h <- x[, cols]
        h <- h[h[,4] > 0.8, c(1,2,3)]
    }))
    rownames(hubs) <- seq_len(nrow(hubs))
    colnames(hubs) <- c("Gene", "Module", "kWithin")
    return(hubs)
}


#' Perform overrepresentation analysis for a set of genes
#'
#' @param genes Character vector containing genes for which
#' overrepresentation will be tested.
#' @param genesets Named list of character vectors, where list names
#' represent functional categories (e.g., GO, pathway, etc.), and vectors
#' represent their associated genes.
#' @param universe Character vector of genes to be used as universe.
#' @param adj Character indicating the multiple testing correction method
#' as in \code{stats::p.adjust()}.
#'
#' @return A data frame of overrepresentation results with the following
#' variables:
#' \describe{
#'   \item{term}{character, functional term ID/name.}
#'   \item{genes}{numeric, intersection length between input genes and genes
#'                in a particular functional term.}
#'   \item{all}{numeric, number of all genes in a particular functional term.}
#'   \item{pval}{numeric, P-value for the hypergeometric test.}
#'   \item{padj}{numeric, P-value adjusted for multiple comparisons using
#'               the method specified in parameter \strong{adj}.}
#'   \item{gene_id}{character, IDs of the genes associated with terms.}
#' }
#'
#' @importFrom stats p.adjust phyper
#' @noRd
ora <- function(
        genes, genesets, universe, adj = "BH"
) {

    # Remove genes that are not in `universe`
    genes <- intersect(genes, universe)
    gene_sets <- lapply(genesets, function(x) intersect(x, universe))

    # Define universe and input genes
    n_universe <- length(universe)
    n_genes <- length(genes)

    # Define variables of the hypergeometric test
    int_sets <- lapply(gene_sets, function(x) intersect(x, genes))
    x <- lengths(int_sets)
    m <- vapply(gene_sets, length, numeric(1))
    n <- n_universe - m
    k <- n_genes

    # Get P-values from a hypergeometric test
    pvals <- phyper(x - 1, m, n, k, lower.tail = FALSE)

    # Store results in a data frame
    results <- data.frame(
        term = names(pvals),
        genes = x,
        all = m,
        pval = pvals,
        row.names = NULL
    )

    results$padj <- p.adjust(results$pval, method = adj)
    gene_id <- lapply(int_sets, function(g) {
        gids <- paste0(g, collapse = "/")
        gids <- ifelse(gids == "", NA, gids)
        return(gids)
    }) |> unlist() |> as.character()

    results$gene_id <- gene_id

    return(results)
}


#' Perform overrepresentation analysis for a set of genes
#'
#' @param genes Character vector containing genes for overrepresentation
#' analysis.
#' @param background_genes Character vector of genes to be used as background
#' for the overrepresentation analysis.
#' @param annotation Annotation data frame with genes in the first column and
#' functional annotation in the other columns. This data frame can be exported
#' from Biomart or similar databases.
#' @param column Column or columns of \strong{annotation} to be used for
#' enrichment.
#' Both character or numeric values with column indices can be used.
#' If users want to supply more than one column, input a character or
#' numeric vector. Default: all columns from \strong{annotation}.
#' @param correction Multiple testing correction method. One of "holm",
#' "hochberg", "hommel", "bonferroni", "BH", "BY", "fdr" or "none".
#' Default is "BH".
#' @param p P-value threshold. P-values below this threshold will be
#' considered significant. Default: 0.05.
#' @param min_setsize Numeric indicating the minimum gene set size to be
#' considered. Gene sets correspond to levels of each variable
#' in \strong{annotation}). Default: 10.
#' @param max_setsize Numeric indicating the maximum gene set size to be
#' considered. Gene sets correspond to levels of each variable
#' in \strong{annotation}). Default: 500.
#' @param bp_param BiocParallel back-end to be used.
#' Default: BiocParallel::SerialParam()
#'
#' @return A data frame of overrepresentation results with the following
#' variables:
#' \describe{
#'   \item{term}{character, functional term ID/name.}
#'   \item{genes}{numeric, intersection length between input genes and genes
#'                in a particular functional term.}
#'   \item{all}{numeric, number of all genes in a particular functional term.}
#'   \item{pval}{numeric, P-value for the hypergeometric test.}
#'   \item{padj}{numeric, P-value adjusted for multiple comparisons using
#'               the method specified in parameter \strong{adj}.}
#'   \item{category}{character, name of the grouping variable (i.e., column
#'                   name of \strong{annotation}).}
#' }
#'
#' @author Fabricio Almeida-Silva
#' @rdname enrichment_analysis
#' @export
#' @importFrom BiocParallel bplapply
#' @examples
#' \donttest{
#' data(filt.se)
#' data(zma.interpro)
#' genes <- rownames(filt.se)[1:50]
#' background_genes <- rownames(filt.se)
#' annotation <- zma.interpro
#' # Using p = 1 to show all results
#' enrich <- enrichment_analysis(genes, background_genes, annotation, p = 1)
#' }
enrichment_analysis <- function(
        genes, background_genes, annotation, column = NULL,
        correction = "BH", p = 0.05, min_setsize = 10, max_setsize = 500,
        bp_param = BiocParallel::SerialParam()
) {

    annotation <- as.data.frame(annotation)
    names(annotation)[1] <- "Gene"

    # Filtered `annotation` data frame to keep only specified columns
    col <- names(annotation)
    if(!is.null(column)) {
        col <- if(is.numeric(column)) c(1, column) else c("Gene", column)
    }
    fannot <- annotation[, col]

    # Get a data frame of enrichment results for each column
    enrichment <- bplapply(seq_along(fannot)[-1], function(x) {

        ## Remove missing values (unannotated genes for a particular category)
        fannot[, x][is.na(fannot[, x])] <- "unannotated"

        ## Split variable into a named list
        gene_sets <- split(fannot[, 1], fannot[, x])
        gene_sets <- gene_sets[!duplicated(gene_sets)]
        gene_sets <- gene_sets[names(gene_sets) != "unannotated"]

        ## Keep only element with `min_setsize` <= n <= `max_setsize` elements
        set_sizes <- as.numeric(lengths(gene_sets))
        keep <- which(set_sizes >= min_setsize & set_sizes <= max_setsize)
        gene_sets <- gene_sets[keep]

        ## Get ORA results and add `category` column
        ora_df <- ora(genes, gene_sets, background_genes, correction)

        if(nrow(ora_df) > 0) {
            ora_df$category <- names(fannot)[x]

            ## Filter non-significant observations out
            ora_df <- ora_df[ora_df$padj <= p, ]
        }

    }, BPPARAM = bp_param)

    enrichment <- Reduce(rbind, enrichment)

    return(enrichment)
}

#' Perform enrichment analysis for coexpression network modules
#'
#' @param net List object returned by \code{exp2gcn}.
#' @param background_genes Character vector of genes to be used as background
#' for the Fisher's Exact Test.
#' @param annotation Annotation data frame with genes in the first column
#' and functional annotation in the other columns. This data frame can
#' be exported from Biomart or similar databases.
#' @param column Column or columns of \code{annotation} to be used for enrichment.
#' Both character or numeric values with column indices can be used.
#' If users want to supply more than one column, input a character or
#' numeric vector. Default: all columns from \code{annotation}.
#' @param correction Multiple testing correction method. One of "holm",
#' "hochberg", "hommel", "bonferroni", "BH", "BY", "fdr" or "none".
#' Default is "BH".
#' @param p P-value threshold. P-values below this threshold will be considered
#' significant. Default is 0.05.
#' @param min_setsize Numeric indicating the minimum gene set size to be
#' considered. Gene sets correspond to levels of each variable
#' in \strong{annotation}). Default: 10.
#' @param max_setsize Numeric indicating the maximum gene set size to be
#' considered. Gene sets correspond to levels of each variable
#' in \strong{annotation}). Default: 500.
#' @param bp_param BiocParallel back-end to be used.
#' Default: BiocParallel::SerialParam()
#'
#' @return A data frame of overrepresentation results with the following
#' variables:
#' \describe{
#'   \item{term}{character, functional term ID/name.}
#'   \item{genes}{numeric, intersection length between input genes and genes
#'                in a particular functional term.}
#'   \item{all}{numeric, number of all genes in a particular functional term.}
#'   \item{pval}{numeric, P-value for the hypergeometric test.}
#'   \item{padj}{numeric, P-value adjusted for multiple comparisons using
#'               the method specified in parameter \strong{adj}.}
#'   \item{category}{character, name of the grouping variable (i.e., column
#'                   name of \strong{annotation}).}
#'   \item{module}{character, module name.}
#' }
#'
#' @author Fabricio Almeida-Silva
#'
#' @rdname module_enrichment
#' @importFrom BiocParallel bplapply SerialParam
#' @export
#' @examples
#' \donttest{
#' data(filt.se)
#' data(zma.interpro)
#' background <- rownames(filt.se)
#' gcn <- exp2gcn(filt.se, SFTpower = 18, cor_method = "pearson")
#' mod_enrich <- module_enrichment(gcn, background, zma.interpro, p=1)
#' }
module_enrichment <- function(
        net = NULL, background_genes, annotation, column = NULL,
        correction = "BH", p = 0.05, min_setsize = 10, max_setsize = 500,
        bp_param = BiocParallel::SerialParam()
) {

    annotation <- as.data.frame(annotation)

    # Create a list of data frames with columns `Genes` and `Modules`
    genes.modules <- net$genes_and_modules
    list.gmodules <- split(genes.modules, genes.modules$Modules)
    list.gmodules <- list.gmodules[names(list.gmodules) != "grey"]

    enrichment_all <- bplapply(seq_along(list.gmodules), function(x) {

        module <- names(list.gmodules)[x]
        message("Enrichment analysis for module ", module, "...")

        # Perform enrichment analysis for {module}
        l <- enrichment_analysis(
            genes = as.character(list.gmodules[[x]][, 1]),
            background_genes = background_genes,
            annotation = annotation,
            column = column,
            correction = correction, p = p,
            min_setsize = min_setsize, max_setsize = max_setsize
        )

        # Either add a column `module` or set `l` to NULL
        if(!is.null(l)) {
            if(nrow(l) > 0) {
                l$module <- module
            } else {
                l <- NULL
            }
        }

        return(l)
    }, BPPARAM = bp_param)

    enrichment_all <- Reduce(rbind, enrichment_all)

    return(enrichment_all)
}


#' Get 1st-order neighbors of a given gene or group of genes
#'
#' @param genes Character vector containing genes from which
#' direct neighbors will be extracted.
#' @param net List object returned by \code{exp2gcn}.
#' @param cor_threshold Correlation threshold to filter connections.
#' As a weighted network is a fully connected graph, a cutoff must be selected.
#' Default is 0.7.
#'
#' @return List containing 1st-order neighbors for each input gene.
#' @seealso \code{exp2gcn} \code{SFT_fit}
#' @author Fabricio Almeida-Silva
#' @export
#' @rdname get_neighbors
#' @examples
#' data(filt.se)
#' genes <- rownames(filt.se)[1:10]
#' gcn <- exp2gcn(filt.se, SFTpower = 18, cor_method = "pearson")
#' neighbors <- get_neighbors(genes, gcn)
get_neighbors <- function(genes, net, cor_threshold = 0.7) {

    net_type <- net$params$net_type
    power <- net$params$SFTpower
    edges <- net$correlation_matrix

    edges <- cormat_to_edgelist(edges)
    colnames(edges) <- c("Gene1", "Gene2", "Weight")

    filt_edges <- edges[edges$Gene1 %in% genes | edges$Gene2 %in% genes, ]
    if(net_type == "signed") {
        filt_edges <- filt_edges[abs(filt_edges$Weight) >= cor_threshold, ]
    } else {
        filt_edges <- filt_edges[filt_edges$Weight >= cor_threshold, ]
    }

    result_list <- lapply(genes, function(x) {
        y <- filt_edges[rowSums(filt_edges == x) > 0, ]
        y <- c(as.character(y$Gene1), as.character(y$Gene2))
        y <- unique(y[y != x])
        return(y)
    })
    names(result_list) <- genes
    return(result_list)
}


#' Get edge list from an adjacency matrix for a group of genes
#'
#' @param net List object returned by \code{exp2gcn}.
#' @param genes Character vector containing a subset of genes from which
#' edges will be extracted. It can be ignored if the user wants to extract
#' an edge list for a given module instead of individual genes.
#' @param module Character with module name from which edges will be extracted.
#' To include 2 or more modules, input the names in a character vector.
#' @param filter Logical indicating whether to filter the edge list or not.
#' @param method Method to filter spurious correlations. One of "Zscore",
#' "optimalSFT", "pvalue" or "min_cor". See details for more information
#' on the methods. Default: 'optimalSFT'
#' @param r_optimal_test Numeric vector with the correlation thresholds
#' to be tested for optimal scale-free topology fit. Only valid
#' if \code{method} equals "optimalSFT". Default: seq(0.4, 0.9, by = 0.1)
#' @param Zcutoff Minimum Z-score threshold. Only valid if
#'  \code{method} equals "Zscore". Default: 1.96
#' @param pvalue_cutoff Maximum P-value threshold. Only valid
#' if \code{method} equals "pvalue". Default: 0.05
#' @param rcutoff Minimum correlation threshold. Only valid
#' if \code{method} equals "min_cor". Default: 0.7
#' @param nSamples Number of samples in the data set from which
#' the correlation matrix was calculated. Only required
#' if \code{method} equals "pvalue".
#' @param check_SFT Logical indicating whether to test if the resulting network
#' is close to a scale-free topology or not. Default: FALSE.
#' @param bp_param BiocParallel back-end to be used.
#' Default: BiocParallel::SerialParam()
#'
#' @return Data frame with edge list for the input genes.
#' @details The default method ("optimalSFT") will create several different
#' edge lists by filtering the original correlation matrix by the thresholds
#' specified in \code{r_optimal_test}. Then, it will calculate a scale-free
#' topology fit index for each of the possible networks and return the network
#' that best fits the scale-free topology.
#' The method "Zscore" will apply a Fisher Z-transformation for the correlation
#' coefficients and remove the Z-scores below the threshold specified
#' in \code{Zcutoff}.
#' The method "pvalue" will calculate Student asymptotic p-value for the
#' correlations and remove correlations whose p-values are above the threshold
#' specified in \code{pvalue_cutoff}.
#' The method "min_cor" will remove correlations below the minimum correlation
#' threshold specified in \code{rcutoff}.
#' @seealso
#'  \code{\link[WGCNA]{scaleFreeFitIndex}}
#' @seealso \code{SFT_fit}
#' @seealso \code{exp2gcn}.
#' @author Fabricio Almeida-Silva
#' @rdname get_edge_list
#' @export
#' @importFrom WGCNA scaleFreeFitIndex corPvalueStudent
#' @importFrom ggplot2 theme geom_point geom_line theme_bw
#' @importFrom BiocParallel bplapply SerialParam
#' @examples
#' data(filt.se)
#' gcn <- exp2gcn(filt.se, SFTpower = 18, cor_method = "pearson")
#' genes <- rownames(filt.se)[1:50]
#' edges <- get_edge_list(gcn, genes=genes, filter = FALSE)
get_edge_list <- function(
        net, genes = NULL, module = NULL, filter = FALSE,
        method = "optimalSFT", r_optimal_test = seq(0.4, 0.9, by = 0.1),
        Zcutoff = 1.96, pvalue_cutoff = 0.05, rcutoff = 0.7, nSamples = NULL,
        check_SFT = FALSE, bp_param = BiocParallel::SerialParam()
) {

    # Define objects containing correlation matrix and data frame of genes and modules
    cor_matrix <- net$correlation_matrix

    # Should we extract genes in a module?
    if(!is.null(module)) {
        genes_modules <- net$genes_and_modules
        keep <- genes_modules[genes_modules$Modules %in% module, 1]
        cor_matrix <- cor_matrix[keep, keep]
    }

    # Should we extract a user-defined gene set?
    if(!is.null(genes)) {
        cor_matrix <- cor_matrix[genes, genes]
    }

    # Should we filter the matrix?
    if(filter) {
        # Create edge list from correlation matrix
        edges <- cormat_to_edgelist(cor_matrix)
        colnames(edges) <- c("Gene1", "Gene2", "Weight")

        if(method == "Zscore") {
            # Apply Fisher-Z transformation to correlation values
            edgesZ <- edges
            edgesZ$Weight <- 0.5 * log((1+edges$Weight) / (1-edges$Weight))

            edgelist <- edgesZ[edgesZ$Weight >= Zcutoff, ]
        } else if(method == "optimalSFT") {
            cutoff <- r_optimal_test

            # Create list of 2 elements: edge lists, each with a different correlation threshold, and degree
            list_mat <- replicate(length(cutoff), cor_matrix, simplify = FALSE)

            list_cormat_filtered <- BiocParallel::bplapply(seq_along(list_mat), function(x) {
                # Convert r values below threshold to NA and set diagonals to 0
                matrix <- list_mat[[x]]
                matrix[matrix < cutoff[x] ] <- NA
                diag(matrix) <- 0

                # Calculate degree
                degree <- rowSums(matrix, na.rm=TRUE)

                # Convert lower triangle and diagonals to NA
                matrix[lower.tri(matrix, diag=TRUE)] <- NA

                # Convert symmetrical matrix to edge list (Gene1, Gene2, Weight)
                matrix <- na.omit(data.frame(as.table(matrix)))
                result <- list(matrix = matrix, degree = degree)
                return(result)
            }, BPPARAM = bp_param)

            degree_list <- lapply(list_cormat_filtered, function(x) return(x[[2]]))

            # Calculate scale-free topology
            sft.rsquared <- unlist(lapply(degree_list, function(x) WGCNA::scaleFreeFitIndex(x)$Rsquared.SFT))
            max.index <- which.max(sft.rsquared)

            # Plot scale-free topology fit for r values
            plot.data <- data.frame(x = cutoff, y = sft.rsquared)

            plot <- ggplot(plot.data, aes(x = .data$x, y = .data$y, group = 1)) +
                geom_point(color = "firebrick", size = 4) +
                geom_line(color = "firebrick", linewidth = 2) +
                labs(
                    x = "Correlation (r) values",
                    y = expression(paste("Scale-free topology fit - ", R^{2})),
                    title = "Scale-free topology fit for given r values"
                ) +
                theme_bw()

            plot
            optimalr <- cutoff[max.index]
            message("The correlation threshold that best fits the scale-free topology is ", optimalr)
            edgelist <- list_cormat_filtered[[max.index]][[1]]
        } else if(method == "pvalue") {
            if(is.null(nSamples)) {
                stop("Please, specify the number of samples used to calculate the correlation matrix")
            }

            # Calculate Student asymptotic p-value for given correlations
            cor.pvalue <- WGCNA::corPvalueStudent(edges$Weight, nSamples)

            # Create a data frame of correlations and p-values
            corandp <- edges
            corandp$pvalue <- cor.pvalue

            # Create a final edge list containing only significant correlations
            edgelist <- corandp[corandp$pvalue < pvalue_cutoff, c(1,2,3)]

        } else if(method == "min_cor") {
            edgelist <- edges[edges$Weight >= rcutoff, ]

        } else{
            stop("Please, specify a valid filtering method.")
        }

    } else {
        # Create edge list from correlation matrix without filtering
        edgelist <- cormat_to_edgelist(cor_matrix)
        colnames(edgelist) <- c("Gene1", "Gene2", "Weight")
    }

    # Check scale-free topology fit
    if(check_SFT) {
        test <- check_SFT(edgelist, net_type="gcn")
    }

    return(edgelist)
}

#----Load data------------------------------------------------------------------
set.seed(12)

data(zma.se)
data(zma.interpro)
data(filt.se)

# Simulate data
## Expression data
exp <- matrix(
    rnorm(1000, mean = 10, sd = 2), nrow = 100, ncol = 100, byrow = TRUE
)

rownames(exp) <- paste0("Gene", seq_len(nrow(exp)))
colnames(exp) <- paste0("Sample", seq_len(ncol(exp)))

## Correlation matrix
cormat <- abs(cor(t(exp)))
rownames(cormat) <- paste0("Gene", 1:100)
colnames(cormat) <- paste0("Gene", 1:100)

## Sample metadata
col_metadata <- data.frame(
    row.names = colnames(exp),
    Class = paste0("Class", sample(1:5, size = ncol(exp), replace = TRUE))
)

## Gene metadata
row_metadata <- data.frame(
    row.names = rownames(exp),
    Pathway = paste0("Pathway", sample(1:5, size = nrow(exp), replace = TRUE))
)

## SummarizedExperiment object
se <- SummarizedExperiment::SummarizedExperiment(
    exp, colData = col_metadata
)

# Infer GCN to avoid repetition many test chunks
gcn <- exp2gcn(filt.se, SFTpower = 18, cor_method = "pearson")

#----Start tests----------------------------------------------------------------
test_that("SFT_fit() performs SFT fit test and returns a list with", {
    sft <- suppressWarnings(SFT_fit(se, cor_method = "pearson"))
    sft2 <- suppressWarnings(SFT_fit(se, cor_method = "biweight"))
    sft3 <- suppressWarnings(SFT_fit(se, cor_method = "spearman"))

    expect_error(
        suppressWarnings(SFT_fit(se, cor_method = "error"))
    )

    expect_equal(class(sft), "list")
    expect_equal(length(sft), 2)
    expect_true("patchwork" %in% class(sft$plot))
    expect_true("ggplot" %in% class(sft$plot))
    expect_equal(class(sft$power), "numeric")

    expect_equal(class(sft2), "list")
    expect_equal(class(sft3), "list")

    expect_equal(length(sft2), 2)
    expect_equal(length(sft3), 2)
})


test_that("exp2gcn() returns a list and get_hubs_gcn() finds hubs", {

    # exp2gcn()
    gcn <- exp2gcn(
        filt.se[1:100, ], SFTpower = 18, cor_method = "pearson", verbose = TRUE,
        return_cormat = FALSE
    )
    gcn2 <- exp2gcn(filt.se[1:100, ], SFTpower = 3, cor_method = "spearman")
    gcn3 <- exp2gcn(filt.se[1:100, ], SFTpower = 18, cor_method = "biweight")

    b1 <- exp2gcn_blockwise(filt.se, SFTpower = 18, cor_method = "pearson")
    b2 <- exp2gcn_blockwise(filt.se, SFTpower = 18, cor_method = "biweight")

    expect_error(
        exp2gcn_blockwise(filt.se, SFTpower = 18, cor_method = "error")
    )

    expect_error(
        exp2gcn_blockwise(filt.se, cor_method = "error")
    )

    expect_error(
        exp2gcn(filt.se, cor_method = "pearson")
    )
    expect_error(
        exp2gcn(filt.se[1:100, ], SFTpower = 3, cor_method = "error")
    )

    expect_equal(class(gcn), "list")
    expect_equal(length(gcn), 7)

    expect_equal(class(gcn2), "list")
    expect_equal(class(gcn3), "list")

    expect_equal(class(b1), "list")

    # get_hubs_gcn()
    hubs <- get_hubs_gcn(filt.se[1:100, ], gcn)
    hubs2 <- get_hubs_gcn(filt.se[1:100, ], gcn2)
    hubs3 <- get_hubs_gcn(filt.se[1:100, ], gcn3)


    expect_equal(ncol(hubs), 3)
    expect_equal(class(hubs), "data.frame")
    expect_equal(colnames(hubs), c("Gene", "Module", "kWithin"))

    expect_equal(class(hubs2), "data.frame")
    expect_equal(class(hubs3), "data.frame")

    expect_equal(ncol(hubs2), 3)
    expect_equal(ncol(hubs3), 3)
})

test_that("plot_eigengene_network() plots eigengene networks", {

    p <- plot_eigengene_network(gcn)
    expect_equal(attr(class(p), "package"), "ComplexHeatmap")

})

test_that("plot_dendro_and_colors() plots dendro and colors", {

    p <- plot_dendro_and_colors(gcn)
    expect_true("ggplot" %in% class(p))

})

test_that("module_stability() recomputes network with n resamplings", {

    p <- module_stability(exp = filt.se[1:50, ], net = gcn, nRuns = 1)

    expect_true("ggplot" %in% class(p))
})

test_that("module_trait_cor() and plot_module_trait_cor() work", {

    # module_trait_cor() ----
    mod_trait <- module_trait_cor(filt.se, MEs = gcn$MEs)

    # plot_module_trait_cor() -----
    p <- plot_module_trait_cor(mod_trait)
    p2 <- plot_module_trait_cor(mod_trait, transpose = TRUE)

    expect_equal(class(mod_trait), "data.frame")
    expect_equal(ncol(mod_trait), 5)
    expect_equal(names(mod_trait), c("ME", "trait", "cor", "pvalue", "group"))

    expect_equal(attr(class(p), "package"), "ComplexHeatmap")
    expect_equal(attr(class(p2), "package"), "ComplexHeatmap")
})


test_that("gene_significance() returns a data frame of gene-trait cors", {
    gs <- gene_significance(filt.se)
    gs2 <- gene_significance(
        filt.se, genes = rownames(filt.se)[1:5], use_abs = FALSE
    )

    p <- plot_gene_significance(gs)

    expect_equal(class(gs), "data.frame")
    expect_equal(ncol(gs), 5)

    expect_equal(class(gs2), "data.frame")
    expect_equal(ncol(gs2), 5)

    expect_equal(attr(class(p), "package"), "ComplexHeatmap")
})


test_that("enrichment_analysis() performs overrepresentation analysis", {

    # Simulated annotation with 1 class
    annotation <- data.frame(
        Gene = rownames(row_metadata),
        Class = row_metadata$Pathway
    )
    # Simulated annotation with 2 classes
    annotation2 <- data.frame(
        Gene = rownames(row_metadata),
        Class1 = row_metadata$Pathway,
        Class2 = row_metadata$Pathway
    )

    genes <- annotation$Gene[1:10]
    background_genes <- annotation$Gene

    # ora()
    genesets <- split(annotation, annotation$Class)
    pe <- ora(genes, genesets, background_genes)


    # enrichment_analysis()
    e1 <- enrichment_analysis(genes, background_genes, annotation, p = 1)
    e2 <- enrichment_analysis(
        genes, background_genes, annotation, column = 2, p = 0.001
    )
    e3 <- enrichment_analysis(
        genes, background_genes, annotation2,
        column = c("Class1", "Class2"), p = 1
    )

    e4 <- enrichment_analysis(
        genes, background_genes, annotation2,
        column = c("Class1", "Class2"), p = 0.0001
    )

    expect_equal(class(pe), "data.frame")

    expect_equal(class(e1), "data.frame")
    expect_equal(ncol(e1), 7)

    expect_true(nrow(e2) < 1)

    expect_equal(class(e3), "data.frame")
    expect_equal(ncol(e3), 7)

    expect_true(nrow(e4) < 1)
})


test_that("module_enrichment() performs ORA for all modules", {

    # Filter GCN to increase speed
    gcn2 <- gcn
    gcn2$genes_and_modules <- gcn2$genes_and_modules[
        gcn2$genes_and_modules$Modules == "black",
    ]

    background <- rownames(filt.se)

    me1 <- module_enrichment(
        net = gcn2, background_genes = background,
        annotation = zma.interpro, p = 1
    )
    me2 <- module_enrichment(
        net = gcn2, background_genes = background,
        annotation = zma.interpro, p = 0.000000001
    )

    expect_equal(class(me1), "data.frame")
    expect_true(is.null(me2))
})


test_that("get_neighbors() returns a list of neighbors for each gene", {

    genes <- rownames(filt.se)[1:10]
    gcn2 <- gcn
    gcn2$params$net_type <- "unsigned"

    neighbors <- get_neighbors(genes, gcn)
    neighbors2 <- get_neighbors(genes, gcn2)

    expect_equal(class(neighbors), "list")
    expect_equal(length(neighbors), 10)

    expect_equal(class(neighbors2), "list")
})


test_that("get_edge_list() generates an edge list as a 3-column data frame", {

    genes <- paste0("Gene", 1:100)
    net <- list(correlation_matrix = cormat)
    n <- ncol(exp)

    edges_nofilter <- get_edge_list(net, genes = genes, filter = FALSE)
    edges_filter_optimalsft <- get_edge_list(
        net, genes = genes, filter = TRUE, r_optimal_test = c(0,1, by = 0.02)
    )
    edges_filter_pval <- get_edge_list(
        net, genes = genes, filter = TRUE, method = "pvalue", nSamples = n
    )
    edges_filter_Z <- get_edge_list(
        net, genes = genes, filter = TRUE, method = "Zscore"
    )
    edges_filter_mincor <- get_edge_list(
        net, genes = genes, filter = TRUE, method = "min_cor", rcutoff = 0.1
    )

    expect_error(
        get_edge_list(
            net, genes = genes, filter = TRUE, method = "pvalue"
        )
    )

    expect_error(
        get_edge_list(
            net, genes = genes, filter = TRUE, method = "error", nSamples = n
        )
    )

    expect_equal(ncol(edges_nofilter), 3)
    expect_equal(ncol(edges_filter_optimalsft), 3)
    expect_equal(ncol(edges_filter_pval), 3)
    expect_equal(ncol(edges_filter_Z), 3)
    expect_equal(ncol(edges_filter_mincor), 3)
})

#' Calculate network statistics
#'
#' @param adj_matrix Adjacency matrix that represents the network.
#' @param net_type One of "gcn" (gene coexpression network),
#' "ppi" (protein-protein interaction), or "grn" (gene regulatory network).
#' @param calculate_additional Logical indicating whether to calculate
#' additional network statistics (betweenness and closeness).
#' Default is FALSE.
#'
#' @return A list containing the following elements: \itemize{
#'   \item Connectivity
#'   \item ScaledConnectivity
#'   \item ClusterCoef
#'   \item MAR (for gcn only)
#'   \item Density
#'   \item Centralization
#'   \item Heterogeneity (gcn only)
#'   \item Diameter
#'   \item Betweenness
#'   \item Closeness
#' }
#'
#' @seealso
#'  \code{\link[igraph]{graph_from_adjacency_matrix}},
#'  \code{\link[igraph]{cliques}},\code{\link[igraph]{diameter}},
#'  \code{\link[igraph]{estimate_betweenness}},\code{\link[igraph]{V}},
#'  \code{\link[igraph]{closeness}},\code{\link[igraph]{degree}},
#'  \code{\link[igraph]{transitivity}},\code{\link[igraph]{edge_density}},
#'  \code{\link[igraph]{centr_degree}}
#'  \code{\link[WGCNA]{fundamentalNetworkConcepts}}
#' @rdname net_stats
#' @export
#' @importFrom igraph graph_from_adjacency_matrix cliques diameter
#' betweenness V closeness degree transitivity edge_density
#' centr_degree
#' @importFrom WGCNA fundamentalNetworkConcepts
#' @examples
#' \donttest{
#' data(filt.se)
#' set.seed(12)
#' filt.se <- exp_preprocess(
#'     filt.se, Zk_filtering = FALSE, variance_filter = TRUE, n = 200
#' )
#' gcn <- exp2gcn(
#'     filt.se, SFTpower = 7, cor_method = "pearson", net_type = "signed hybrid"
#' )
#' stats <- net_stats(gcn$adjacency_matrix, net_type = "gcn")
#' }
net_stats <- function(adj_matrix = NULL, net_type = c("gcn", "ppi", "grn"),
                      calculate_additional = FALSE) {

    weighted <- NULL
    mode <- "undirected"
    directed <- FALSE
    if(net_type == "gcn") {
        weighted <- TRUE
    } else if(net_type == "grn") {
        mode <- "directed"
        directed <- TRUE
    } else if(net_type == "ppi") {
        a <- NULL
    } else {
        stop("Please, specify a valid network type. One of 'gcn', 'ppi' or 'grn'.")
    }
    graph <- igraph::graph_from_adjacency_matrix(
        adj_matrix, mode = mode, diag = FALSE, weighted = weighted
    )
    diam <- igraph::diameter(graph, directed = directed)
    if(net_type != "grn") {
        stats <- WGCNA::fundamentalNetworkConcepts(adj_matrix)
        stats$Diameter <- diam
        if(net_type == "ppi") { stats$MAR <- NULL }
    } else {
        degree <- igraph::degree(graph, mode = "all")
        clustercoef <- igraph::transitivity(graph, type = "global")
        density <- igraph::edge_density(graph)
        centralization <- igraph::centr_degree(graph)$centralization
        stats <- list(
            Connectivity = degree,
            ScaledConnectiviy = degree / max(degree),
            ClusterCoef = clustercoef,
            Density = density,
            Centralization = centralization,
            Diameter = diam
        )
    }
    if(calculate_additional) {
        betweenness <- igraph::betweenness(graph, directed = directed)
        names(betweenness) <- igraph::V(graph)
        closeness <- igraph::closeness(graph, mode="all")
        names(closeness) <- igraph::V(graph)
        stats$betweenness <- as.data.frame(betweenness)
        stats$closeness <- as.data.frame(closeness)
    }
    return(stats)
}

#----Load data------------------------------------------------------------------
set.seed(12)

# Simulate adjacency matrices
## GCN
exp <- matrix(
    rnorm(1000, mean = 10, sd = 2), nrow = 100, ncol = 100, byrow = TRUE
)
adj <- abs(cor(t(exp)))
rownames(adj) <- paste0("Gene", 1:100)
colnames(adj) <- paste0("Gene", 1:100)

## GRN and PPI
adj_un <- adj
adj_un[adj_un > 0.4] <- 1
adj_un[adj_un <= 0.4] <- 0
diag(adj_un) <- 0

#----Start tests----------------------------------------------------------------
test_that("net_stats() returns a list of network statistics", {

    stats_gcn <- net_stats(adj, net_type = "gcn")
    stats_ppi <- net_stats(adj_un, net_type = "ppi")
    stats_grn <- net_stats(adj_un, net_type = "grn")
    stats_gcn_additional <- net_stats(adj, net_type = "gcn", TRUE)

    expect_error(net_stats(adj, net_type = "wrong"))

    expect_equal(class(stats_gcn), "list")
    expect_equal(class(stats_ppi), "list")
    expect_equal(class(stats_grn), "list")
    expect_equal(class(stats_gcn_additional), "list")

    expect_equal(length(stats_gcn), 8)
    expect_equal(length(stats_ppi), 7)
    expect_equal(length(stats_grn), 6)
    expect_equal(length(stats_gcn_additional), 10)
})

#' Logical expression to check if gene or gene set is singleton or not
#'
#' @param genes Character containing gene or group of genes to be evaluated.
#' @param og Data frame of 3 columns corresponding to orthogroup, species ID,
#' and gene ID, respectively.
#' @return Vector of logical values indicating if gene or group of genes
#' is singleton or not.
#'
#' @seealso \code{is_duplicated}
#' @author Fabricio Almeida-Silva
#' @rdname is_singleton
#' @export
#' @examples
#' data(og.zma.osa)
#' data(filt.se)
#' genes <- tail(rownames(filt.se), n = 100)
#' is_singleton(genes, og.zma.osa)
is_singleton <- function(genes, og) {
    s <- lengths(split(og[og[,3] %in% genes, 3], genes)) == 1
    return(s)
}

#' Maize gene expression data from Shin et al., 2021.
#'
#' Filtered expression data in transcripts per million (TPM) from
#' Shin et al., 2021. Genes with TPM values <5 in more than 60% of the samples
#' were removed to reduce package size. The expression data and
#' associated sample metadata are stored in a SummarizedExperiment object.
#'
#' @name zma.se
#' @format An object of class \code{SummarizedExperiment}
#' @references Shin, J., Marx, H., Richards, A., Vaneechoutte, D., Jayaraman,
#' D., Maeda, J., ... & Roy, S. (2021). A network-based comparative framework
#' to study conservation and divergence of proteomes in plant
#' phylogenies. Nucleic Acids Research, 49(1), e3-e3.
#' @examples
#' data(zma.se)
#' @usage data(zma.se)
"zma.se"


#' Rice gene expression data from Shin et al., 2021.
#'
#' Filtered expression data in transcripts per million (TPM) from
#' Shin et al., 2021. Genes with TPM values <5 in more than 60% of the samples
#' were removed to reduce package size. The expression data and
#' associated sample metadata are stored in a SummarizedExperiment object.
#'
#' @name osa.se
#' @format An object of class \code{SummarizedExperiment}
#' @references Shin, J., Marx, H., Richards, A., Vaneechoutte, D., Jayaraman,
#' D., Maeda, J., ... & Roy, S. (2021). A network-based comparative framework
#' to study conservation and divergence of proteomes in plant
#' phylogenies. Nucleic Acids Research, 49(1), e3-e3.
#' @examples
#' data(osa.se)
#' @usage data(osa.se)
"osa.se"


#' Filtered maize gene expression data from Shin et al., 2021.
#'
#' Filtered expression data in transcripts per million (TPM) from
#' Shin et al., 2021. This is the same data set described in \code{zma.se},
#' but it only contains the top 500 genes with the highest variances.
#' This data set was created to be used in unit tests and examples.
#'
#' @name filt.se
#' @format An object of class \code{SummarizedExperiment}
#' @references
#' Shin, J., Marx, H., Richards, A., Vaneechoutte, D., Jayaraman, D., Maeda,
#' J., ... & Roy, S. (2021). A network-based comparative framework to study
#' conservation and divergence of proteomes in plant
#' phylogenies. Nucleic Acids Research, 49(1), e3-e3.
#' @examples
#' data(filt.se)
#' @usage data(filt.se)
"filt.se"


#' Maize Interpro annotation
#'
#' Interpro protein domain annotation retrieved from the
#' PLAZA Monocots 4.0 database.
#' Only genes included in \code{zma.se} are present in this subset.
#'
#' @name zma.interpro
#' @format A 2-column data frame containing gene IDs and their
#' associated Interpro annotations.
#' @references
#' Van Bel, M., Diels, T., Vancaester, E., Kreft, L., Botzki, A.,
#' Van de Peer, Y., ... & Vandepoele, K. (2018). PLAZA 4.0: an integrative
#' resource for functional, evolutionary and comparative plant
#' genomics. Nucleic acids research, 46(D1), D1190-D1196.
#' @examples
#' data(zma.interpro)
#' @usage data(zma.interpro)
"zma.interpro"


#' Maize transcription factors
#'
#' Transcription factors and their families were downloaded from PlantTFDB 4.0.
#'
#' @name zma.tfs
#' @format A data frame with gene IDs of TFs and their associated families.
#' @references
#' Jin, J., Tian, F., Yang, D. C., Meng, Y. Q., Kong, L., Luo, J., &
#' Gao, G. (2016). PlantTFDB 4.0: toward a central hub for transcription
#' factors and regulatory interactions in plants. Nucleic acids research, gkw982.
#' @examples
#' data(zma.tfs)
#' @usage data(zma.tfs)
"zma.tfs"


#' Orthogroups between maize and rice
#'
#' The orthogroups were downloaded from the PLAZA 4.0 Monocots database.
#'
#' @name og.zma.osa
#' @format A 3-column data frame with orthogroups, species IDs and gene IDs.
#' @references
#' Van Bel, M., Diels, T., Vancaester, E., Kreft, L., Botzki, A.,
#' Van de Peer, Y., ... & Vandepoele, K. (2018). PLAZA 4.0: an integrative
#' resource for functional, evolutionary and comparative plant
#' genomics. Nucleic acids research, 46(D1), D1190-D1196.
#'
#' @examples
#' data(og.zma.osa)
#' @usage data(og.zma.osa)
"og.zma.osa"

#' Parse orthogroups identified by OrthoFinder
#'
#' This function converts the orthogroups file named \strong{Orthogroups.tsv} to
#' a 3-column data frame that can be interpreted by BioNERO.
#'
#' @param file_path Path to Orthogroups/Orthogroups.tsv file generated
#' by OrthoFinder.
#'
#' @author Fabricio Almeida-Silva
#' @return A 3-column data frame with orthogroups, species IDs and
#' gene IDs, respectively.
#' @importFrom reshape2 melt
#' @export
#' @rdname parse_orthofinder
#' @examples
#' path <- system.file("extdata", "Orthogroups.tsv", package = "BioNERO")
#' og <- parse_orthofinder(path)
parse_orthofinder <- function(file_path = NULL) {
  of <- read.csv(file_path, sep="\t")
  melt_of <- reshape2::melt(of, id.vars="Orthogroup")
  s <- strsplit(melt_of$value, split = ", ")
  final_of <- data.frame(
      Orthogroup = rep(melt_of$Orthogroup, vapply(s, length, numeric(1))),
      Species = rep(melt_of$variable, vapply(s, length, numeric(1))),
      Gene = unlist(s)
  )
  return(final_of)
}


#' Collapse gene-level expression data to orthogroup level
#'
#' For a given list of expression data, this function replaces genes with
#' their corresponding orthogroups to allow inter-species comparisons.
#'
#' @param explist List of expression data frames or SummarizedExperiment objects.
#' @param og Data frame of 3 columns corresponding to orthogroup,
#' species ID, and gene ID, respectively.
#' Species IDs must be the same as the names of the expression list.
#' @param summarize Centrality measure to summarize multiple paralogous genes
#' in the same orthogroup.
#' One of "median" or "mean". Default: "median".
#' @return List of expression data frames for each species with expression
#' summarized at the orthogroup level.
#'
#' @export
#' @importFrom stats aggregate
#' @rdname exp_genes2orthogroups
#' @examples
#' \donttest{
#' data(og.zma.osa)
#' data(zma.se)
#' data(osa.se)
#' explist <- list(zma = zma.se,
#'                 osa = osa.se)
#' og <- og.zma.osa
#' exp_ortho <- exp_genes2orthogroups(explist, og, summarize = "mean")
#' }
exp_genes2orthogroups <- function(explist = NULL, og = NULL,
                                  summarize = "median") {

    colnames(og) <- c("Family", "Species", "Gene")
    exp <- handleSElist(explist)
    exp <- exp[order(names(exp))]
    og_list <- split(og, og$Species)
    og_list <- og_list[order(names(og_list))]
    og_exp <- lapply(seq_along(exp), function(x) {
        set <- merge(exp[[x]], og_list[[x]], by.x = "row.names", by.y = 3)
        set <- set[, -c(1, ncol(set))]
        set <- suppressWarnings(
            aggregate(set, by = list(set$Family), FUN = summarize)
        )
        rownames(set) <- set[,1]
        set <- set[, -c(1, ncol(set))]
        return(set)
    })
    names(og_exp) <- names(exp)

    return(og_exp)
}

#' Calculate module preservation between two expression data sets using WGCNA's algorithm
#'
#' @param explist List of expression data frames or SummarizedExperiment objects.
#' @param ref_net Reference network object returned by the function \code{exp2net}.
#' @param nPerm Number of permutations for the module preservation statistics.
#' It must be greater than 1. Default: 200.
#'
#' @return A ggplot object with module preservation statistics.
#' @rdname modPres_WGCNA
#' @export
#' @importFrom WGCNA standardColors
#' @importFrom ggplot2 theme geom_hline geom_point
#' @importFrom ggrepel geom_text_repel
#' @importFrom patchwork wrap_plots
#' @examples
#' \donttest{
#' set.seed(1)
#' data(og.zma.osa)
#' data(zma.se)
#' data(osa.se)
#' explist <- list(Zma = zma.se, Osa = osa.se)
#' og <- og.zma.osa
#' exp_ortho <- exp_genes2orthogroups(explist, og, summarize = "mean")
#' exp_ortho <- lapply(exp_ortho, function(x) filter_by_variance(x, n=1500))
#' # Previously calculated power
#' powers <- c(13, 15)
#' gcn_osa <- exp2gcn(exp_ortho$Osa, net_type = "signed hybrid",
#'                    SFTpower = powers[1], cor_method = "pearson")
#' explist <- exp_ortho
#' ref_net <- gcn_osa
#' # 5 permutations for demonstration purposes
#' pres_wgcna <- modPres_WGCNA(explist, ref_net, nPerm=5)
#' }
modPres_WGCNA <- function(explist, ref_net, nPerm = 200) {

    explist <- handleSElist(explist)
    explist <- lapply(explist, function(x) return(t(x)))

    # Set parameters for network reconstruction
    net_type <- ref_net$params$net_type
    cor_method <- ref_net$params$cor_method

    # Create multiExpr object
    multiExpr <- list(
        ref = list(data = explist[[1]]), test = list(data = explist[[2]])
    )

    # Create vector of module assignments
    moduleColors <- ref_net$genes_and_modules$Modules
    names(moduleColors) <- ref_net$genes_and_modules$Genes

    multiColor <- list(ref = moduleColors)

    # Calculate module preservation
    if(cor_method == "pearson") {
        corOptions <- "use = 'p'"
        corFnc <- "cor"
    } else if(cor_method == "spearman") {
        corOptions <- "use = 'p', method = 'spearman'"
        corFnc <- "cor"
    } else if(cor_method == "biweight") {
        corOptions <- "use = 'p', maxPOutliers = 0.05"
        corFnc <- "bicor"
    }

    pres <- WGCNA::modulePreservation(
        multiExpr, multiColor, referenceNetworks = 1, nPermutations = nPerm,
        randomSeed = 1, quickCor = 0, corFnc = corFnc, corOptions = corOptions,
        verbose = 0, networkType = net_type, savePermutedStatistics = FALSE,
        plotInterpolation = FALSE
    )

    # Isolate the observed statistics and their Z-scores
    ref <- 1
    test <- 2
    statsObs <- cbind(pres$quality$observed[[ref]][[test]][, -1],
                      pres$preservation$observed[[ref]][[test]][, -1])
    statsZ <- cbind(pres$quality$Z[[ref]][[test]][, -1],
                    pres$preservation$Z[[ref]][[test]][, -1])

    # Module labels and module sizes are also contained in the results
    modColors <- rownames(pres$preservation$observed[[ref]][[test]])
    moduleSizes <- pres$preservation$Z[[ref]][[test]][, 1]

    # Do not consider grey and gold modules
    plotMods <- !(modColors %in% c("grey", "gold"))

    # Text labels for points
    text <- modColors[plotMods]

    # Auxiliary convenience variable
    plotData <- cbind(pres$preservation$observed[[ref]][[test]][, 2],
                      pres$preservation$Z[[ref]][[test]][, 2])


    # Plot preservation median rank
    dplot1 <- data.frame(
        x = moduleSizes[plotMods],
        y = plotData[plotMods, 1],
        label = as.factor(text)
    )

    p1 <- ggplot(dplot1, aes(x = .data$x, y = .data$y)) +
        geom_point(fill = text, color = "black", shape = 21) +
        geom_text_repel(aes(label = .data$label)) +
        labs(
            x = "Module size", y = "Median rank",
            title = "Preservation median rank"
        ) +
        theme_bw()

    # Plot preservation Z-summary
    dplot2 <- data.frame(
        x = moduleSizes[plotMods],
        y = plotData[plotMods, 2],
        label = text
    )

    p2 <- ggplot(dplot2, aes(x = .data$x, y = .data$y)) +
        geom_point(fill = text, color = "black", shape = 21) +
        geom_text_repel(aes(label = .data$label)) +
        labs(
            x = "Module size", y = expression("Z"[summary]),
            title = expression("Preservation Z"[summary])
        ) +
        theme_bw() +
        geom_hline(yintercept = 0, linetype = 2) +
        geom_hline(yintercept = 2, colour = "blue", linetype = 2) +
        geom_hline(yintercept = 10, colour = "forestgreen", linetype = 2)

    fig1 <- patchwork::wrap_plots(p1, p2, ncol = 2)
    return(fig1)
}


#' Calculate module preservation between two expression data sets using NetRep's algorithm
#'
#' @param explist List of expression data frames or SummarizedExperiment objects.
#' @param ref_net Reference network object returned by the function \code{exp2net}.
#' @param test_net Test network object returned by the function \code{exp2net}.
#' @param nPerm Number of permutations. Default: 1000
#' @param nThreads Number of threads to be used for parallel computing.
#' Default: 1
#' @return Output list from \code{NetRep::modulePreservation} and a message in
#' user's standard output stating which modules are preserved.
#' @seealso
#'  \code{\link[NetRep]{modulePreservation}}
#' @rdname modPres_netrep
#' @export
#' @importFrom NetRep modulePreservation
#' @examples
#' \donttest{
#' set.seed(1)
#' data(og.zma.osa)
#' data(zma.se)
#' data(osa.se)
#' og <- og.zma.osa
#' exp_ortho <- exp_genes2orthogroups(explist, og, summarize = "mean")
#' exp_ortho <- lapply(exp_ortho, function(x) filter_by_variance(x, n=1500))
#' # Previously calculated SFT powers
#' powers <- c(13, 15)
#' gcn_osa <- exp2gcn(exp_ortho$osa, net_type = "signed hybrid",
#'                    SFTpower = powers[1], cor_method = "pearson")
#' gcn_zma <- exp2gcn(exp_ortho$zma, net_type = "signed hybrid",
#'                    SFTpower = powers[2], cor_method = "pearson")
#' explist <- exp_ortho
#' ref_net <- gcn_osa
#' test_net <- gcn_zma
#' # 10 permutations for demonstration purposes
#' pres_netrep <- modPres_netrep(explist, ref_net, test_net,
#'                               nPerm=10, nThreads = 2)
#' }
#'
modPres_netrep <- function(explist, ref_net = NULL, test_net = NULL,
                           nPerm = 1000, nThreads = 1) {
    explist <- handleSElist(explist)
    explist <- lapply(explist, function(x) return(t(x)))

    # Set data set names
    data_names <- names(explist)
    if(is.null(names(explist))) {
        data_names <- c("cohort1", "cohort2")
        names(explist) <- data_names
    }


    # Create correlation list
    correlation_list <- list(
        as.matrix(ref_net$correlation_matrix),
        as.matrix(test_net$correlation_matrix)
    )
    names(correlation_list) <- data_names

    # Create network list
    network_list <- list(
        as.matrix(ref_net$adjacency_matrix),
        as.matrix(test_net$adjacency_matrix)
    )
    names(network_list) <- data_names

    # Set module assignments
    modAssignments <- ref_net$genes_and_modules$Modules
    names(modAssignments) <- ref_net$genes_and_modules$Genes

    # Set background label
    backgroundLabel <- "grey"
    if(!"grey" %in% modAssignments) { backgroundLabel <- 0 }

    # Calculate preservation statistics
    pres <- NetRep::modulePreservation(
        network = network_list, data = explist, correlation = correlation_list,
        moduleAssignments = modAssignments,
        discovery = data_names[1], test = data_names[2],
        nPerm = nPerm, nThreads = nThreads, verbose = FALSE
    )

    # Get preserved modules (p < 0.05 for all statistics)
    max_pval <- apply(pres$p.value, 1, max, na.rm = TRUE)
    preservedmodules <- names(max_pval[max_pval < 0.05])

    if(length(preservedmodules) > 0) {
        message(length(preservedmodules), " modules in ", data_names[1],
                " were preserved in ", data_names[2], ":", "\n",
                toString(preservedmodules))
    } else {
        message("None of the modules in ", data_names[1],
                " were preserved in ", data_names[2], ".")
    }
    return(pres)
}



#' Calculate network preservation between two expression data sets
#'
#' @param explist List of SummarizedExperiment objects or expression data frames
#' with genes (or orthogroups) in row names and samples in column names.
#' @param ref_net Reference network object returned by
#' the function \code{exp2gcn}.
#' @param test_net Test network object returned by the function \code{exp2gcn}.
#' @param algorithm Module preservation algorithm to be used. One of 'netrep'
#' (default, permutation-based) or WGCNA.
#' @param nPerm Number of permutations. Default: 1000
#' @param nThreads Number of threads to be used for parallel computing.
#' Default: 1
#'
#' @return A list containing the preservation statistics (netrep) or a ggplot
#' object with preservation statistics.
#' See \code{WGCNA::modulePreservation} or \code{NetRep::modulePreservation}
#' for more info.
#' @rdname module_preservation
#' @export
#' @examples
#' \donttest{
#' set.seed(1)
#' data(og.zma.osa)
#' data(zma.se)
#' data(osa.se)
#' og <- og.zma.osa
#' exp_ortho <- exp_genes2orthogroups(explist, og, summarize = "mean")
#' exp_ortho <- lapply(exp_ortho, function(x) filter_by_variance(x, n=1500))
#' # Previously calculated SFT powers
#' powers <- c(13, 15)
#' gcn_osa <- exp2gcn(exp_ortho$osa, net_type = "signed hybrid",
#'                    SFTpower = powers[1], cor_method = "pearson")
#' gcn_zma <- exp2gcn(exp_ortho$zma, net_type = "signed hybrid",
#'                    SFTpower = powers[2], cor_method = "pearson")
#' explist <- exp_ortho
#' ref_net <- gcn_osa
#' test_net <- gcn_zma
#' # 10 permutations for demonstration purposes
#' pres <- module_preservation(explist, ref_net, test_net, nPerm=10)
#' }
#'
module_preservation <- function(explist, ref_net = NULL, test_net = NULL,
                                algorithm = "netrep",
                                nPerm = 1000, nThreads = 1) {

    if(algorithm == "netrep") {
        pres <- modPres_netrep(
            explist = explist,
            ref_net = ref_net,
            test_net = test_net,
            nPerm = nPerm, nThreads = nThreads
        )

    } else if(algorithm == "WGCNA") {
        pres <- modPres_WGCNA(
            explist = explist,
            ref_net = ref_net,
            nPerm = nPerm
        )
    } else {
        stop("Please, specify a valid algorithm. One of 'netrep' or 'WGCNA'.")
    }

    return(pres)
}

#----Load data------------------------------------------------------------------
set.seed(1)
data(og.zma.osa)
data(zma.se)
data(osa.se)

og <- og.zma.osa
explist <- list(osa = osa.se, zma = zma.se)
explist <- lapply(
    explist, exp_preprocess, Zk_filtering = FALSE,
    variance_filter = TRUE, n = 200
)
exp_ortho <- exp_genes2orthogroups(explist, og, summarize = "mean")

powers <- c(12, 7)
gcn_osa <- exp2gcn(
    exp_ortho$osa, net_type = "signed hybrid", SFTpower = powers[1],
    cor_method = "pearson"
)
gcn_zma <- exp2gcn(
    exp_ortho$zma, net_type = "signed hybrid", SFTpower = powers[2],
    cor_method = "pearson"
)


#----Start tests----
test_that("parse_orthofinder() parses OrthoFinder's orthogroups", {
    path <- system.file("extdata", "Orthogroups.tsv", package = "BioNERO",
                        mustWork = TRUE)
    og <- parse_orthofinder(path)
    expect_equal(ncol(og), 3)
})


test_that("exp_genes2orthogroups() replaces genes with orthogroups", {
    exp_ortho <- exp_genes2orthogroups(explist, og, summarize = "mean")
    expect_equal(length(exp_ortho), 2)
    expect_equal(class(exp_ortho), "list")
    expect_true(startsWith(rownames(exp_ortho[[1]])[1], "ORTH"))
})

test_that("modPres_WGCNA() calculates module preservation", {
    explist <- exp_ortho
    ref_net <- gcn_osa
    pres_wgcna <- modPres_WGCNA(explist, ref_net, nPerm = 2)
    expect_true("patchwork" %in% class(pres_wgcna))
    expect_true("ggplot" %in% class(pres_wgcna))
})

# PASSED on Feb 26, 2021.
# Commented because R CMD check was taking more than 10m.
# test_that("modPres_netrep() calculates module preservation", {
#     explist <- exp_ortho
#     ref_net <- gcn_osa
#     test_net <- gcn_zma
#     expect_message(pres_netrep <- modPres_netrep(explist, ref_net, test_net,
#                                                  nPerm=10, nThreads = 2),
#                    "None of the modules in osa were preserved in zma.")
# })

test_that("module_preservation() calculates module preservation", {

    explist <- exp_ortho
    names(explist) <- NULL
    ref_net <- gcn_osa
    test_net <- gcn_zma
    pres <- suppressWarnings(
        module_preservation(explist, ref_net, test_net, nPerm = 2)
    )
    pres_wgcna <- module_preservation(
        explist, ref_net, nPerm = 2, algorithm = "WGCNA"
    )

    ref_net$params$cor_method <- "biweight"
    pres_wgcna2 <- module_preservation(
        explist, ref_net, nPerm = 2, algorithm = "WGCNA"
    )

    ref_net$params$cor_method <- "spearman"
    pres_wgcna2 <- module_preservation(
        explist, ref_net, nPerm = 2, algorithm = "WGCNA"
    )

    expect_equal(class(pres), "list")
    expect_true("ggplot" %in% class(pres_wgcna))
    expect_error(module_preservation(
        explist, ref_net, nPerm = 2, algorithm = "wrong"
    ))
})

#' Combine multiple expression tables (.tsv) into a single data frame
#'
#' This function reads multiple expression tables (.tsv files) in a directory
#' and combines them into a single gene expression data frame.
#'
#' @param mypath Path to directory containing .tsv files.
#' Files must have the first column in common, e.g. "Gene_ID".
#' Rows are gene IDs and columns are sample names.
#' @param pattern Pattern contained in each expression file.
#' Default is '.tsv$', which means that all files ending in '.tsv' in the
#' specified directory will be considered expression files.
#' @return Data frame with gene IDs as row names and their expression values in
#' each sample (columns).
#' @author Fabricio Almeida-Silva
#' @rdname dfs2one
#' @export
#' @importFrom utils head read.csv write.table
#' @examples
#' # Simulate two expression data frames of 100 genes and 30 samples
#' genes <- paste0(rep("Gene", 100), 1:100)
#' samples1 <- paste0(rep("Sample", 30), 1:30)
#' samples2 <- paste0(rep("Sample", 30), 31:60)
#' exp1 <- cbind(genes, as.data.frame(matrix(rnorm(100*30),nrow=100,ncol=30)))
#' exp2 <- cbind(genes, as.data.frame(matrix(rnorm(100*30),nrow=100,ncol=30)))
#' colnames(exp1) <- c("Gene", samples1)
#' colnames(exp2) <- c("Gene", samples2)
#'
#' # Write data frames to temporary files
#' tmpdir <- tempdir()
#' tmp1 <- tempfile(tmpdir = tmpdir, fileext = ".exp.tsv")
#' tmp2 <- tempfile(tmpdir = tmpdir, fileext = ".exp.tsv")
#' write.table(exp1, file=tmp1, quote=FALSE, sep="\t")
#' write.table(exp2, file=tmp2, quote=FALSE, sep="\t")
#'
#' # Load the files into one
#' exp <- dfs2one(mypath = tmpdir, pattern=".exp.tsv")
dfs2one <- function(mypath, pattern = ".tsv$"){
    filenames <- list.files(path=mypath, full.names=TRUE, pattern = pattern)
    datalist <- lapply(filenames, function(x) {
        read.csv(file=x, header=TRUE, sep="\t", stringsAsFactors=FALSE,
                 check.names=FALSE)
    })
    merged.list <- Reduce(function(x,y) merge(x,y, all.x=TRUE), datalist)
    rownames(merged.list) <- merged.list[,1]
    merged.list[,1] <- NULL
    return(merged.list)
}

#' Remove missing values in a gene expression data frame
#'
#' @param exp A gene expression data frame with genes in row names and
#' samples in column names or a `SummarizedExperiment` object.
#' @param replaceby What to use instead of NAs. One of 0 or 'mean'. Default is 0.
#'
#' @return Gene expression data frame or `SummarizedExperiment` object
#' with all NAs replaced according to the argument 'replaceby'.
#' @author Fabricio Almeida-Silva
#' @export
#' @rdname replace_na
#' @examples
#' data(zma.se)
#' exp <- replace_na(zma.se)
#' sum(is.na(exp))
replace_na <- function(exp, replaceby = 0) {
    fexp <- handleSE(exp)

    if(is(exp, "SummarizedExperiment")) {
        fexp <- SummarizedExperiment::assay(exp)
    } else {
        fexp <- exp
    }

    if(replaceby == 0) {
        fexp[is.na(fexp)] <- 0
    } else {
        indices <- which(is.na(fexp), arr.ind = TRUE)
        fexp[indices] <- rowMeans(fexp, na.rm = TRUE)[indices[,1]]
    }

    if(is(exp, "SummarizedExperiment")) {
        fexp <- exp2SE(fexp, exp)
    }

    return(fexp)
}

#' Remove genes that are not expressed based on a user-defined threshold
#'
#' @param exp A gene expression data frame with genes in row names
#' and samples in column names or a `SummarizedExperiment` object.
#' @param method Criterion to filter non-expressed genes out.
#' One of "mean", "median", "percentage", or "allsamples". Default is "median".
#' @param min_exp If method is 'mean', 'median', or 'allsamples',
#' the minimum value for a gene to be considered expressed.
#' If method is 'percentage', the minimum value each gene must have in
#' at least n percent of samples to be considered expressed.
#' @param min_percentage_samples In case the user chooses 'percentage' as method,
#' expressed genes must have expression >= min_exp in at least this percentage.
#' Values must range from 0 to 1.
#'
#' @return Filtered gene expression data frame or `SummarizedExperiment` object.
#' @author Fabricio Almeida-Silva
#' @export
#' @importFrom matrixStats rowMedians
#' @seealso
#'  \code{\link[matrixStats]{rowMedians}}
#'  \code{\link[WGCNA]{goodSamplesGenes}}
#' @rdname remove_nonexp
#' @examples
#' data(zma.se)
#' filt_exp <- remove_nonexp(zma.se, min_exp = 5)
remove_nonexp <- function(exp, method="median", min_exp=1, min_percentage_samples=0.25) {
    fexp <- handleSE(exp)

    if(method == "median") {
        final_exp <- fexp[matrixStats::rowMedians(as.matrix(fexp)) >= min_exp,]
    } else if (method == "mean") {
        final_exp <- fexp[rowMeans(fexp) >= min_exp,]
    } else if (method == "percentage") {
        min_n <- ncol(fexp) * min_percentage_samples
        final_exp <- fexp[rowSums(fexp >= min_exp) >= min_n, ]
    } else if (method == "allsamples") {
        final_exp <- fexp[rowSums(fexp >= min_exp) == ncol(fexp), ]
    } else {
        stop("No method specified. Please, choose a filtering method - mean, median or percentage")
    }

    if(is(exp, "SummarizedExperiment")) {
        final_exp <- exp2SE(final_exp, exp)
    }

    return(final_exp)
}

#' Keep only genes with the highest variances
#'
#' @param exp A gene expression data frame with genes in row names
#' and samples in column names or a `SummarizedExperiment` object.
#' @param n Number of most variable genes (e.g., n=5000 will
#' keep the top 5000 most variable genes).
#' @param percentile Percentile of most highly variable genes
#' (e.g., percentile=0.1 will keep the top 10 percent most variable genes).
#' Values must range from 0 to 1.
#'
#' @return Expression data frame or `SummarizedExperiment` object with
#' the most variable genes in row names and samples in column names.
#' @author Fabricio Almeida-Silva
#' @export
#' @rdname filter_by_variance
#' @examples
#' data(zma.se)
#' filt_exp <- filter_by_variance(zma.se, p=0.1)
filter_by_variance <- function(exp, n=NULL, percentile=NULL) {
    fexp <- handleSE(exp)

    gene_var <- data.frame(Genes = rownames(fexp), Var = apply(fexp, 1, var))
    gene_var_ordered <- gene_var[order(gene_var$Var, decreasing = TRUE), ]
    if(!is.null(n) & is.null(percentile)) {
        top_var <- gene_var_ordered$Genes[seq_len(n)]
    } else if(is.null(n) & !is.null(percentile)) {
        p <- nrow(gene_var_ordered) * percentile
        top_var <- gene_var_ordered$Genes[seq_len(p)]
    } else {
        stop("Please, choose either 'n' or 'percentile'.")
    }
    top_variant_exp <- fexp[rownames(fexp) %in% top_var, ]

    if(is(exp, "SummarizedExperiment")) {
        top_variant_exp <- exp2SE(top_variant_exp, exp)
    }

    return(top_variant_exp)
}

#' Filter outlying samples based on the standardized connectivity (Zk) method
#'
#' @param exp A gene expression data frame with genes in row names
#' and samples in column names or a `SummarizedExperiment` object.
#' @param zk Standardized connectivity threshold. Default is -2.
#' @param cor_method Correlation method. One of "pearson", "biweight"
#' or "spearman". Default is "spearman".
#'
#' @return Filtered gene expression data frame or `SummarizedExperiment` object.
#' @author Fabricio Almeida-Silva
#' @importFrom WGCNA adjacency bicor
#' @seealso
#'  \code{\link[WGCNA]{adjacency}}
#' @rdname ZKfiltering
#' @export
#' @examples
#' data(zma.se)
#' filt_exp <- ZKfiltering(zma.se)
#' @references
#' Oldham, M. C., Langfelder, P., & Horvath, S. (2012). Network methods for
#' describing sample relationships in genomic datasets: application to
#' Huntington’s disease. BMC systems biology, 6(1), 1-18.
ZKfiltering <- function(exp, zk = -2, cor_method = "spearman") {
    fexp <- handleSE(exp)

    if(cor_method == "pearson") {
        A <- WGCNA::adjacency(fexp, type = "distance")
    } else if(cor_method == "biweight") {
        A <- WGCNA::adjacency(
            fexp, type = "distance", corFnc = bicor,
            corOptions = list(use = 'p', maxPOutliers = 0.05)
        )
    } else if(cor_method == "spearman"){
        A <- WGCNA::adjacency(
            fexp, type = "distance", corOptions = list(use = 'p', method = "spearman")
        )
    } else {
        stop("Please, specify a correlation method (one of 'spearman', 'pearson' or 'biweight').")
    }

    k <- colSums(A) - 1
    Z.k <- scale(k)

    # Remove outliers
    remove.samples <- Z.k < zk | is.na(Z.k)
    fexp <- fexp[, !remove.samples]
    message("Number of removed samples: ", sum(remove.samples))

    if(is(exp, "SummarizedExperiment")) {
        fexp <- exp2SE(fexp, exp)
    }
    return(fexp)
}


#' Quantile normalize the expression data
#'
#' @param exp A gene expression data frame with genes in row names
#' and samples in column names.
#'
#' @return Expression matrix with normalized values
#' @rdname q_normalize
#' @export
#' @examples
#' data(zma.se)
#' exp <- SummarizedExperiment::assay(zma.se)
#' norm_exp <- q_normalize(exp)
q_normalize <- function(exp) {
    n <- nrow(exp)
    rank.exp <- apply(exp, 2, rank)
    U <- rank.exp / (n+1)
    qnorm(U)
}

#' Apply Principal Component (PC)-based correction for confounding artifacts
#'
#' @param exp A gene expression data frame with genes in row names
#' and samples in column names or a `SummarizedExperiment` object.
#' @param verbose Logical indicating whether to display progress
#' messages or not. Default: FALSE.
#'
#' @return Corrected expression data frame or `SummarizedExperiment` object.
#' @author Fabricio Almeida-Silva
#' @export
#' @seealso
#'  \code{\link[sva]{num.sv}},\code{\link[sva]{sva_network}}
#' @rdname PC_correction
#' @importFrom sva num.sv sva_network
#' @examples
#' data(zma.se)
#' exp <- filter_by_variance(zma.se, n=500)
#' exp <- PC_correction(exp)
#' @references
#' Parsana, P., Ruberman, C., Jaffe, A. E., Schatz, M. C., Battle, A., &
#' Leek, J. T. (2019). Addressing confounding artifacts in reconstruction of
#' gene co-expression networks. Genome biology, 20(1), 1-6.
PC_correction <- function(exp, verbose = FALSE) {
    fexp <- handleSE(exp)

    texp <- t(fexp) # transpose data frame
    texp <- as.matrix(texp)
    mod <- matrix(1, nrow = nrow(texp), ncol = 1)
    colnames(mod) <- "Intercept"

    if(verbose) { message("Calculating number of PCs to be removed...") }
    nsv <- sva::num.sv(t(texp), mod, method = "be")

    if(verbose) { message("Number of PCs estimated to be removed: ", nsv) }

    # PC residualization of gene expression data using sva_network
    if(verbose){ message("Removing PCs that contribute to noise...") }
    exprs_corrected <- sva::sva_network(texp, nsv)
    exprs_corrected_norm <- q_normalize(exprs_corrected)
    final.exp.corrected <- as.data.frame(exprs_corrected_norm)
    final.exp.corrected <- t(final.exp.corrected)

    if(is(exp, "SummarizedExperiment")) {
        final.exp.corrected <- exp2SE(final.exp.corrected, exp)
    }
    return(final.exp.corrected)
}


#' Preprocess expression data for network reconstruction
#'
#' @param exp A gene expression data frame with genes in row names
#' and samples in column names or a `SummarizedExperiment` object.
#' @param NA_rm Logical. It specifies whether to remove missing values
#' from the expression data frame or not. Default = TRUE.
#' @param replaceby If NA_rm is TRUE, what to use instead of NAs.
#' One of 0 or 'mean'. Default is 0.
#' @param Zk_filtering Logical. It specifies whether to filter outlying samples
#' by Zk or not. Default: TRUE.
#' @param zk If Zk_filtering is TRUE, the standardized connectivity threshold.
#' Samples below this threshold will be considered outliers. Default is -2.
#' @param cor_method If Zk_filtering is TRUE, the correlation method to use.
#' One of 'spearman', 'bicor', or 'pearson'. Default is 'spearman'.
#' @param remove_nonexpressed Logical. It specifies whether non-expressed genes
#' should be removed or not. Default is TRUE.
#' @param method If remove_nonexpressed is TRUE, the criterion to filter
#' non-expressed genes out. One of "mean", "median", "percentage",
#' or "allsamples". Default is 'median'.
#' @param min_exp If method is 'mean', 'median', or 'allsamples',
#' the minimum value for a gene to be considered expressed.
#' If method is 'percentage', the minimum value each gene must have in at least
#' n percent of samples to be considered expressed.
#' @param min_percentage_samples If method is 'percentage', expressed genes
#' must have expression >= min_exp in at least this percentage.
#' Values must range from 0 to 1. Default = 0.25.
#' @param remove_confounders Logical. If TRUE, it removes principal components
#' that add noise to the data.
#' @param variance_filter Logical. If TRUE, it will filter genes by variance.
#' Default is FALSE.
#' @param n If variance_filter is TRUE, the number of
#' most variable genes to keep.
#' @param percentile If variance_filter is TRUE, the percentage of
#' most variable genes to keep.
#' @param vstransform Logical indicating if data should be
#' variance stabilizing transformed. This parameter can only be set to TRUE
#' if data is a matrix of raw read counts.
#'
#' @return Processed gene expression data frame with gene IDs in row names
#' and sample names in column names or `SummarizedExperiment` object.
#' @author Fabricio Almeida-Silva
#' @seealso
#'  \code{\link[DESeq2]{varianceStabilizingTransformation}}
#' @rdname exp_preprocess
#' @export
#' @examples
#' data(zma.se)
#' exp <- exp_preprocess(zma.se, variance_filter=TRUE, n=1000)
#' @references
#' Love, M. I., Huber, W., & Anders, S. (2014). Moderated estimation of
#' fold change and dispersion for RNA-seq data with
#' DESeq2. Genome biology, 15(12), 1-21.
exp_preprocess <- function(exp, NA_rm = TRUE, replaceby = 0,
                           Zk_filtering = TRUE, zk = -2,
                           cor_method = "spearman",
                           remove_nonexpressed = TRUE, method = "median",
                           min_exp = 1, min_percentage_samples = 0.25,
                           remove_confounders = TRUE,
                           variance_filter = FALSE, n = NULL, percentile = NULL,
                           vstransform = FALSE) {
    # Remove missing values
    if(NA_rm) {
        exp <- replace_na(exp, replaceby = replaceby)
    }
    # Remove non-expressed genes
    if(remove_nonexpressed) {
        exp <- remove_nonexp(
            exp, method = method,
            min_exp = min_exp,
            min_percentage_samples = min_percentage_samples
        )
    }
    # Apply VST for count data
    if(vstransform) {
        if(!requireNamespace("DESeq2", quietly = TRUE)) {
            stop("vstransform = TRUE requires the Bioconductor package 'DESeq2'.")
        }

        fexp <- as.matrix(handleSE(exp))
        fexp <- as.data.frame(DESeq2::varianceStabilizingTransformation(fexp))
        if(is(exp, "SummarizedExperiment")) {
            fexp <- exp2SE(fexp, exp)
        }
        exp <- fexp
    }
    # Filter by variance
    if(variance_filter) {
        exp <- filter_by_variance(exp, n = n, percentile = percentile)
    }
    # Zk filtering
    if(Zk_filtering) {
        exp <- ZKfiltering(exp, zk = zk, cor_method = cor_method)
    }
    # Remove confounders
    if(remove_confounders) {
        exp <- PC_correction(exp)
    }
    return(exp)
}

#----Set up data sets----
data(zma.se)
exp <- SummarizedExperiment::assay(zma.se)
metadata <- SummarizedExperiment::colData(zma.se)
metadata <- data.frame(
    Samples = rownames(metadata),
    Tissue = metadata$Tissue
)

simexp <- matrix(
    rnorm(100 * 50, mean = 10, sd = 2),
    nrow = 100,
    dimnames = list(
        paste0("gene", seq_len(100)),
        paste0("sample", seq_len(50))
    )
)
simexp[c(1, 5, 7), c(10, 20, 30)] <- NA

#----Start tests----
test_that("dfs2one reads in multiple tables and binds them into a data frame", {
    genes <- paste0(rep("Gene", 100), 1:100)
    samples1 <- paste0(rep("Sample", 30), 1:30)
    samples2 <- paste0(rep("Sample", 30), 31:60)

    # Simulate two expression data frames of 100 genes and 30 samples
    exp1 <- cbind(genes, as.data.frame(matrix(rnorm(100*30),nrow=100,ncol=30)))
    exp2 <- cbind(genes, as.data.frame(matrix(rnorm(100*30),nrow=100,ncol=30)))
    colnames(exp1) <- c("Gene", samples1)
    colnames(exp2) <- c("Gene", samples2)

    # Write data frames to temporary files
    tmpdir <- tempdir()
    tmp1 <- tempfile(tmpdir = tmpdir, fileext = ".exp.tsv")
    tmp2 <- tempfile(tmpdir = tmpdir, fileext = ".exp.tsv")

    write.table(exp1, file=tmp1, quote=FALSE, sep="\t")
    write.table(exp2, file=tmp2, quote=FALSE, sep="\t")

    expect_equal(nrow(dfs2one(mypath = tmpdir, pattern=".exp.tsv")), 100)
    expect_equal(ncol(dfs2one(mypath = tmpdir, pattern=".exp.tsv")), 60)
})


test_that("replace_na() removes NAs for both SE and expression data frame", {

    expinput <- replace_na(simexp, "mean")
    seinput <- replace_na(zma.se)

    expect_equal(sum(is.na(expinput)), 0)
    expect_equal(sum(is.na(seinput)), 0)
    expect_equal(dim(seinput), dim(zma.se))
})


test_that("remove_nonexp() removes non-expressed genes", {
    filt_exp1 <- remove_nonexp(exp, min_exp=10)
    filt_exp2 <- remove_nonexp(zma.se, min_exp=10)
    filt_exp3 <- remove_nonexp(zma.se, method = "percentage", min_exp=10)
    filt_exp4 <- remove_nonexp(zma.se, method="mean", min_exp=10)
    filt_exp5 <- remove_nonexp(zma.se, method="allsamples", min_exp=10)

    expect_error(remove_nonexp(exp, min_exp = 10, method = "error"))

    expect_true(all.equal(dim(filt_exp1), dim(filt_exp2)))
    expect_equal(class(filt_exp1), "data.frame")
    expect_true(class(filt_exp2) == "SummarizedExperiment")
    expect_true(nrow(filt_exp2) < nrow(zma.se))
    expect_true(nrow(filt_exp3) < nrow(zma.se))
    expect_true(nrow(filt_exp4) < nrow(zma.se))
    expect_true(nrow(filt_exp5) < nrow(zma.se))
})


test_that("filter_by_variance() filters gene expression data by variance", {

    filt_exp1 <- filter_by_variance(exp, n=3000)
    filt_exp2 <- filter_by_variance(zma.se, percentile = 0.1)

    expect_error(filter_by_variance(exp))

    expect_true(class(filt_exp2) == "SummarizedExperiment")
    expect_equal(class(filt_exp1), "data.frame")
    expect_equal(nrow(filt_exp1), 3000)
    expect_true(nrow(filt_exp2) <= nrow(zma.se) * 0.1)
})


test_that("ZKfiltering() filters outliers", {
    filt_exp1 <- ZKfiltering(exp)
    filt_exp2 <- ZKfiltering(zma.se)
    filt_exp3 <- ZKfiltering(zma.se, cor_method = "pearson")
    filt_exp4 <- ZKfiltering(zma.se, cor_method = "biweight", zk = -2.5)

    expect_error(ZKfiltering(exp, cor_method = "error"))

    expect_true(class(filt_exp2) == "SummarizedExperiment")
    expect_true(all.equal(dim(filt_exp1), dim(filt_exp2)))
    expect_message(filt_exp3 <- ZKfiltering(zma.se, cor_method = "pearson"),
                   "Number of removed samples: 1"
    )
})


test_that("q_normalize() performs quantile normalization in an expression data frame", {
    norm_exp <- q_normalize(exp)
    cols <- sample(seq_len(ncol(norm_exp)), size=2, replace=FALSE)

    expect_equal(
        IQR(norm_exp[, cols[1]]), IQR(norm_exp[, cols[2]])
    )
})


test_that("PC_correction() removes confounders based on principal components", {
    filt_exp <- filter_by_variance(exp, n = 500)
    filt_exp2 <- filter_by_variance(zma.se, n=500)
    pc_cor1 <- PC_correction(filt_exp, verbose = TRUE)
    pc_cor2 <- PC_correction(filt_exp2)

    expect_equal(dim(pc_cor1), dim(pc_cor2))
})


test_that("exp_preprocess() preprocesses the expression data at once", {

    filt_exp1 <- exp_preprocess(exp, variance_filter = TRUE, n=500)
    filt_exp2 <- exp_preprocess(zma.se, variance_filter = TRUE, n=500)

    if(requireNamespace("DESeq2", quietly = TRUE)) {

        counts <- matrix(
            sample.int(100, size = 50 * 100, replace = TRUE), nrow = 50
        )

        filt_exp3 <- exp_preprocess(
            counts, variance_filter = TRUE, n = 500, vstransform = TRUE,
            Zk_filtering = FALSE
        )
    }

    expect_equal(dim(filt_exp1), dim(filt_exp2))
})

#' Plot heatmap of hierarchically clustered sample correlations or gene expression
#'
#' @param exp A gene expression data frame with genes in row names
#' and samples in column names or a `SummarizedExperiment` object.
#' @param col_metadata A data frame containing sample names in row names and
#' sample annotation in the subsequent columns. The maximum number of columns
#' is 3 to ensure legends can be visualized. Ignored if `exp` is
#' a `SummarizedExperiment` object, since the function will extract colData.
#' Default: NA.
#' @param row_metadata A data frame containing gene IDs in row names and
#' gene functional classification in the first column. The maximum number
#' of columns is 3 to ensure legends can be visualized. Default: NA.
#' @param coldata_cols A vector (either numeric or character) indicating
#' which columns should be extracted from column metadata if \strong{exp}
#' is a `SummarizedExperiment` object. The vector can contain column
#' indices (numeric) or column names (character). By default, all columns are
#' used.
#' @param rowdata_cols A vector (either numeric or character) indicating
#' which columns should be extracted from row metadata if \strong{exp}
#' is a `SummarizedExperiment` object. The vector can contain column
#' indices (numeric) or column names (character). By default, all columns are
#' used.
#' @param type Type of heatmap to plot. One of 'samplecor' (sample correlations)
#' or 'expr'. Default: 'samplecor'.
#' @param cor_method Correlation method to use in
#' case \strong{type} is "samplecor". One of 'spearman' or 'pearson'.
#' Default is 'spearman'.
#' @param palette RColorBrewer palette to use. Default is "Blues" for sample
#' correlation heatmaps and "YlOrRd" for gene expression heatmaps.
#' @param log_trans Logical indicating whether to log transform the expression
#' data or not. Default: FALSE.
#' @param ... Additional arguments to be passed
#' to \code{ComplexHeatmap::pheatmap()}. These arguments can be used to control
#' heatmap aesthetics, such as show/hide row and column names,
#' change font size, activate/deactivate hierarchical clustering, etc. For a
#' complete list of the options, see \code{?ComplexHeatmap::pheatmap()}.
#'
#' @return A heatmap of sample correlations or gene expression.
#' @author Fabricio Almeida-Silva
#' @seealso
#'  \code{\link[RColorBrewer]{RColorBrewer}}
#' @rdname plot_heatmap
#' @export
#' @importFrom RColorBrewer brewer.pal
#' @importFrom ComplexHeatmap pheatmap
#' @importFrom methods is
#' @examples
#' \donttest{
#' data(filt.se)
#' plot_heatmap(filt.se)
#' }
plot_heatmap <- function(exp, col_metadata = NA, row_metadata = NA,
                         coldata_cols = NULL, rowdata_cols = NULL,
                         type = "samplecor", cor_method = 'spearman',
                         palette = NULL, log_trans = FALSE, ...) {

    fexp <- handleSE(exp)
    if(methods::is(exp, "SummarizedExperiment")) {
        col_metadata <- se2metadata(exp, coldata_cols = coldata_cols)$coldata
        row_metadata <- se2metadata(exp, rowdata_cols = rowdata_cols)$rowdata
    }

    # Get column metadata with sorted rows + list of named vectors with colors
    cdata <- metadata2colors(col_metadata)
    col_metadata <- cdata$metadata
    col_colors <- cdata$colors

    # Get row metadata with sorted rows + list of named vectors with colors
    rdata <- metadata2colors(row_metadata)
    row_metadata <- rdata$metadata
    row_colors <- rdata$colors

    annotation_colors <- c(col_colors, row_colors)

    # Reorder rows and columns of the expression matrix based on metadata
    if(is.data.frame(col_metadata)) { fexp <- fexp[, rownames(col_metadata)] }
    if(is.data.frame(row_metadata)) { fexp <- fexp[rownames(row_metadata), ] }

    # Should data be log-transformed?
    if(log_trans) { fexp <- log2(fexp + 1) }

    # Get attributes (color palette, matrix, title, and legend title)
    hm_attr <- heatmap_attributes(fexp, palette, type, cor_method)

    # Plot heatmap
    hm <- ComplexHeatmap::pheatmap(
        as.matrix(hm_attr$mat),
        name = hm_attr$name,
        color = hm_attr$pal,
        border_color = NA,
        annotation_row = row_metadata,
        annotation_col = col_metadata,
        main = hm_attr$title,
        annotation_colors = annotation_colors,
        ...
    )

    return(hm)
}

#' Plot Principal Component Analysis (PCA) of samples
#'
#' @param exp A gene expression data frame with genes in row names
#' and samples in column names or a `SummarizedExperiment` object.
#' @param metadata A data frame of sample metadata containing sample names
#' in row names and sample annotation in subsequent columns.
#' Ignored if `exp` is a `SummarizedExperiment` object, since colData will be
#' automatically extracted.
#' @param metadata_cols A vector (either numeric or character) indicating
#' which columns should be extracted from column metadata if \strong{exp}
#' is a `SummarizedExperiment` object. The vector can contain column
#' indices (numeric) or column names (character). By default, all columns are
#' used.
#' @param log_trans Logical indicating whether the gene expression matrix
#' should be log transformed using \code{log(exp + 1)}. Default: FALSE.
#' @param PCs Numeric vector of length 2 indicating the principal components
#' to be plotted on the x-axis and y-axis, respectively.
#' Default: \code{c(1, 2)}.
#' @param size Numeric indicating the point size. Default is 2.
#' @return A ggplot object with the PCA plot.
#'
#' @author Fabricio Almeida-Silva
#' @seealso
#'  \code{\link[ggplot2]{ggplot}}
#' @rdname plot_PCA
#' @export
#' @importFrom ggplot2 ggplot aes geom_point scale_color_manual labs
#' theme_classic ggtitle theme element_text
#' @importFrom SummarizedExperiment colData
#' @importFrom stats prcomp
#' @importFrom rlang .data
#' @examples
#' data(zma.se)
#' plot_PCA(zma.se, log_trans = TRUE)
plot_PCA <- function(exp, metadata, metadata_cols = NULL,
                     log_trans = FALSE, PCs = c(1, 2), size = 2) {

    # Get expression matrix and colData if exp is a `SummarizedExperiment`
    fexp <- handleSE(exp)
    if(methods::is(exp, "SummarizedExperiment")) {
        metadata <- se2metadata(exp, coldata_cols = metadata_cols)$coldata
    }

    if(ncol(metadata) > 2) { stop("Sample metadata must contain up to 2 columns.") }
    if(log_trans) { fexp <- log2(fexp + 1) }

    # Perform PCA
    pca <- prcomp(t(fexp))

    # Get 2 data frames: PCs with metadata, and variance explained
    pcs <- merge(as.data.frame(pca$x), metadata, by = "row.names")
    varexp <- data.frame(
        row.names = colnames(pca$x),
        Var = round(100 * pca$sdev^2 / sum(pca$sdev^2), 1)
    )

    # Get plot parameters
    pc_x <- paste0("PC", PCs[1])
    pc_y <- paste0("PC", PCs[2])
    colors <- custom_palette(1)[seq_along(unique(metadata[, 1]))]
    color_by <- names(metadata)[1]
    shape_by <- names(metadata)[2]

    point <- geom_point(aes(color = .data[[color_by]]), size = size)
    if(ncol(metadata) > 1) {
        point <- geom_point(
            aes(color = .data[[color_by]], shape = .data[[shape_by]]),
            size = size
        )
    }

    # Create plot
    p <- ggplot(pcs, aes(x = .data[[pc_x]], y = .data[[pc_y]])) +
        point +
        labs(
            title = "PCA of samples",
            x = paste0(pc_x, " (", varexp[pc_x, ], "%)"),
            y = paste0(pc_y, " (", varexp[pc_y, ], "%)")
        ) +
        scale_color_manual(values = colors) +
        theme_classic()

    return(p)
}


#' Get housekeeping genes from global expression profile
#'
#' @param exp A gene expression data frame with genes in row names and
#' samples in column names or a `SummarizedExperiment` object.
#'
#' @details This function identifies housekeeping genes, which are
#' broadly expressed genes with low variation in a global scale across samples.
#' For some cases, users would want to remove these genes as they are not
#' interesting for coexpression network analyses.
#' See references for more details.
#'
#' @return Character vector of housekeeping gene IDs.
#' @author Fabricio Almeida-Silva
#' @rdname get_HK
#' @export
#' @references
#' Machado, F.B., Moharana, K.C., Almeida‐Silva, F., Gazara, R.K.,
#' Pedrosa‐Silva, F., Coelho, F.S., Grativol, C. and Venancio, T.M. (2020),
#' Systematic analysis of 1298 RNA‐Seq samples and construction of a
#' comprehensive soybean (Glycine max) expression atlas. Plant J, 103: 1894-1909.
#' @examples
#' data(zma.se)
#' hk <- get_HK(zma.se)
get_HK <- function(exp) {
    exp <- handleSE(exp)
    exp <- exp
    exp[exp < 1] <- 0 # expression values < 1 are considered as not expressed
    ncols <- ncol(exp)
    final.exp <- exp[rowSums(exp > 0) == ncols,]
    final.exp$mean <- rowMeans(final.exp[, seq_len(ncols)])
    final.exp$sd <-  apply(final.exp[, seq_len(ncols)], 1, sd)
    final.exp$covar <- final.exp$sd / final.exp$mean
    final.exp$max <- apply(final.exp[, seq_len(ncols)], 1, max)
    final.exp$min <- apply(final.exp[, seq_len(ncols)], 1, min)
    final.exp$MFC <- final.exp$max / final.exp$min
    final.exp$MFC.CoV <- final.exp$MFC * final.exp$covar
    hk <- head(
        final.exp[order(final.exp$MFC.CoV, decreasing = FALSE),],
        n = nrow(final.exp) * 0.25
    )
    hk.genes <- rownames(hk)
    return(hk.genes)
}

#' Plot expression profile of given genes across samples
#'
#' @param genes Character vector containing a subset of genes from which
#' edges will be extracted. It can be ignored if \code{plot_module} is TRUE.
#' @param exp A gene expression data frame with genes in row names and
#' samples in column names or a `SummarizedExperiment` object.
#' @param metadata A data frame of sample metadata containing sample names
#' in row names and sample annotation in subsequent columns.
#' Ignored if `exp` is a `SummarizedExperiment` object, since colData will be
#' automatically extracted.
#' @param metadata_cols A character or numeric scalar indicating
#' which column should be extracted from column metadata if \strong{exp}
#' is a `SummarizedExperiment` object. The column to be extracted can be
#' represented by indices (numeric) or column names (character).
#' By default, the first column is used.
#' @param plot_module Logical indicating whether to plot a whole module or not.
#' If set to FALSE, \code{genes} must be specified.
#' @param net List object returned by \code{exp2gcn}.
#' @param modulename Name of the module to plot.
#' @param bg_line Character indicating what to show in the background (black)
#' line. One of "mean" or "median". Default: "mean".
#'
#' @return A ggplot object showing the expression profile of some genes
#' across all samples.
#' @author Fabricio Almeida-Silva
#'
#' @rdname plot_expression_profile
#' @export
#' @importFrom reshape2 melt
#' @importFrom ggplot2 ggplot geom_tile aes geom_line stat_summary labs
#' theme theme_classic element_blank
#' @examples
#' data(zma.se)
#' data(filt.se)
#' genes <- rownames(filt.se)
#' plot_expression_profile(genes = genes, exp = zma.se, plot_module = FALSE)
plot_expression_profile <- function(genes, exp, metadata, metadata_cols = 1,
                                    plot_module = TRUE, net, modulename,
                                    bg_line = "mean") {

    # Extract expression matrix and colData if exp is a `SummarizedExperiment`
    if(is(exp, "SummarizedExperiment")) {
        metadata <- se2metadata(exp, coldata_cols = metadata_cols)$coldata
    }
    exp <- as.matrix(handleSE(exp))

    # Reorder rows of the metadata based on levels of the chosen variable
    cols <- metadata2colors(metadata)
    metadata <- cols$metadata
    colors <- cols$colors[[1]]

    # Get genes to plot and plotting parameters
    title <- "Expression profile"
    if(plot_module) {
        genes_modules <- net$genes_and_modules
        genes <- genes_modules[genes_modules$Modules == modulename, "Genes"]
        title <- paste("Expression profile for module", modulename)
    }

    # Reshape expression matrix to long format
    exp_long <- reshape2::melt(exp[rownames(exp) %in% genes, ])
    names(exp_long) <- c("Gene", "Sample", "Expression")

    # Convert `Sample` column to factor and reorder levels based on metadata
    metadata$Sample <- factor(rownames(metadata), levels = rownames(metadata))
    exp_long$Sample <- factor(exp_long$Sample, levels = rownames(metadata))

    # Y-axis limit for the tile
    metadata$background <- mean(exp_long$Expression)

    # Plot expression profiles
    var <- names(metadata)[1]
    p <- ggplot(exp_long, aes(x = .data$Sample, y = .data$Expression)) +
        geom_tile(data = metadata, aes(
            x = .data$Sample, y = .data$background, fill = .data[[var]]
        ), alpha = 0.3, height = Inf) +
        scale_fill_manual(values = colors) +
        geom_line(aes(group = .data$Gene), alpha = 0.3, color = "firebrick") +
        stat_summary(aes(group = 1), linewidth = 1, fun = bg_line, geom = "line") +
        labs(x = "", title = title) +
        theme_classic() +
        theme(
            axis.text.x = element_blank(),
            legend.position = "bottom"
        )

    return(p)
}


#' Plot number of genes per module
#'
#' @param net List object returned by \code{exp2gcn}.
#' @return A ggplot object with a bar plot of gene number in each module.
#'
#' @rdname plot_ngenes_per_module
#' @export
#' @importFrom ggplot2 ggplot geom_col scale_fill_manual theme_bw labs
#' theme element_text geom_text xlim
#' @examples
#' data(filt.se)
#' gcn <- exp2gcn(filt.se, SFTpower = 18, cor_method = "pearson")
#' plot_ngenes_per_module(gcn)
plot_ngenes_per_module <- function(net = NULL) {

    # Create a data frame of gene counts per module
    genes_and_modules <- net$genes_and_modules
    frequency_df <- as.data.frame(table(genes_and_modules$Modules))
    names(frequency_df) <- c("Module", "Frequency")

    # Reorder `Frequency` column in descending order
    frequency_df <- frequency_df[order(frequency_df$Frequency, decreasing = TRUE), ]
    frequency_df$Module <- factor(
        frequency_df$Module,
        levels = frequency_df$Module
    )

    # Get colors and axis limits
    cols <- levels(frequency_df$Module)
    xmax <- ceiling(max(frequency_df$Frequency) / 100) * 100

    # Create plot
    p <- ggplot(frequency_df, aes(x = .data$Frequency, y = .data$Module)) +
        geom_col(aes(fill = .data$Module), color = "black") +
        geom_text(aes(label = .data$Frequency), hjust = -0.3) +
        scale_fill_manual(values = cols) +
        xlim(0, xmax) +
        theme_bw() +
        labs(title = "Number of genes per module") +
        theme(legend.position = "none")

    return(p)
}

#----Load data----------------------------------------------------------------
set.seed(12)
data(filt.se)

# Simulate data
## Expression data
exp <- matrix(
    rnorm(1000, mean = 10, sd = 2), nrow = 100, ncol = 100, byrow = TRUE
)
rownames(exp) <- paste0("Gene", seq_len(nrow(exp)))
colnames(exp) <- paste0("Sample", seq_len(ncol(exp)))

## Sample metadata
col_metadata <- data.frame(
    row.names = colnames(exp),
    Class = paste0("Class", sample(1:5, size = ncol(exp), replace = TRUE))
)

## Gene metadata
row_metadata <- data.frame(
    row.names = rownames(exp),
    Pathway = paste0("Pathway", sample(1:5, size = nrow(exp), replace = TRUE))
)

## SummarizedExperiment object
se <- SummarizedExperiment::SummarizedExperiment(
    exp, colData = col_metadata
)

# Get GCN
gcn <- suppressWarnings(
    exp2gcn(filt.se, SFTpower = 16, cor_method = "pearson")
)

#----Start tests----------------------------------------------------------------
test_that("plot_heatmap() correctly handles row and col annotation", {

    p <- plot_heatmap(
        exp, col_metadata, row_metadata, cluster_cols = FALSE,
        cluster_rows = FALSE, type = "expr", log_trans = TRUE
    )

    p2 <- plot_heatmap(
        se, col_metadata, row_metadata, cluster_cols = FALSE,
        cluster_rows = FALSE, type = "samplecor", log_trans = FALSE,
        palette = "Greens"
    )

    expect_error(
        plot_heatmap(
            exp, col_metadata, row_metadata, cluster_cols = FALSE,
            cluster_rows = FALSE, type = "error", log_trans = TRUE
        )
    )


    expect_equal(attr(class(p), "package"), "ComplexHeatmap")
    expect_equal(attr(class(p2), "package"), "ComplexHeatmap")

})


test_that("plot_PCA() performs PCA and plots it", {

    p1 <- plot_PCA(exp, col_metadata, log_trans = TRUE)
    p2 <- plot_PCA(filt.se, log_trans = FALSE, PCs = c(1,3))

    expect_error(
        plot_PCA(exp, cbind(col_metadata, col_metadata, col_metadata))
    )

    expect_true("ggplot" %in% class(p1))
    expect_true("ggplot" %in% class(p2))
})


test_that("get_HK() returns housekeeping genes in a character vector", {
    hk <- get_HK(zma.se)

    expect_equal(class(hk), "character")
    expect_true(length(hk) <= floor(nrow(zma.se) * 0.25))
})


test_that("plot_expression_profile() plots expression in a line plot", {
    genes <- rownames(filt.se)

    p1 <- plot_expression_profile(
        genes = genes, exp = zma.se, plot_module = FALSE
    )

    p2 <- plot_expression_profile(
        exp = filt.se, plot_module = TRUE, net = gcn, modulename = "black"
    )

    expect_true("ggplot" %in% class(p1))
    expect_true("ggplot" %in% class(p2))
})


test_that("plot_ngenes_per_module() returns a barplot of genes per module", {
    p <- plot_ngenes_per_module(gcn)

    expect_true("ggplot" %in% class(p))
})

---
title: "Network comparison: consensus modules and module preservation"
author:
- name: Fabricio Almeida-Silva
  affiliation: Universidade Estadual do Norte Fluminense Darcy Ribeiro, RJ, Brazil
- name: Thiago Motta Venancio
  affiliation: Universidade Estadual do Norte Fluminense Darcy Ribeiro, RJ, Brazil
output:
  BiocStyle::html_document:
    toc: true
    number_sections: yes
bibliography: vignette3.bib
vignette: |
  %\VignetteIndexEntry{Network comparison: consensus modules and module preservation}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  message = TRUE,
  warning = FALSE,
  cache = FALSE,
  fig.align = 'center',
  fig.width = 5,
  fig.height = 4,
  crop = NULL
)
```

# Installation

```{r installation, eval=FALSE}
if(!requireNamespace('BiocManager', quietly = TRUE))
  install.packages('BiocManager')

BiocManager::install("BioNERO")
```

```{r load_package}
# Load package after installation
library(BioNERO)
set.seed(12) # for reproducibility
```

# Introduction

Comparing different coexpression networks can reveal relevant biological 
patterns. For instance, seeking for **consensus modules** can identify 
coexpression modules that occur in all data sets regardless of natural 
variation and, hence, are core players of the studied phenotype. 
Additionally, **module preservation** within and across species can reveal 
patterns of conservation and divergence between transcriptomes. 
In this vignette, we will explore consensus modules and module preservation 
analyses with `BioNERO`. Although they seem similar, their goals are opposite:
while consensus modules identification focuses on the commonalities, module 
preservation focuses on the divergences. 


# Data loading and description

We will use RNA-seq data of maize (*Zea mays*) and rice (*Oryza sativa*) 
obtained from @Shin2020. 

```{r inspect_data}
data(zma.se)
zma.se

data(osa.se)
osa.se
```

All `BioNERO`'s functions for consensus modules and module preservation 
analyses require the expression data to be in a **list**. Each element of the 
list can be a SummarizedExperiment object (recommended) or an expression data 
frame with genes in row names and samples in column names. 


# Consensus modules

The most common objective in consensus modules identification is to find core 
modules across different tissues or treatments for the same species. 
For instance, one can infer GCNs for different types of cancer in human 
tissues (say prostate and liver) and identify modules that occur in all sets, 
which are likely core components of cancer biology. Likewise, one can also 
identify consensus modules across samples from different geographical origins 
to find modules that are not affected by population structure or kinship.

## Data preprocessing

Here, we will subset 22 random samples from the maize data twice and find 
consensus modules between the two sets.

```{r create_list_consensus}
# Preprocess data and keep top 2000 genes with highest variances
filt_zma <- exp_preprocess(zma.se, variance_filter = TRUE, n = 2000)

# Create different subsets by resampling data
zma_set1 <- filt_zma[, sample(colnames(filt_zma), size=22, replace=FALSE)]
zma_set2 <- filt_zma[, sample(colnames(filt_zma), size=22, replace=FALSE)]
colnames(zma_set1)
colnames(zma_set2)

# Create list
zma_list <- list(set1 = zma_set1, set2 = zma_set2)
length(zma_list)
```

## Identification of consensus modules

As described in the first vignette, before inferring the GCNs, we need to 
identify the optimal $\beta$ power that makes the network closer to a scale-free 
topology. We can do that with `consensus_SFT_fit()`.

```{r consensus_sft, message=FALSE}
cons_sft <- consensus_SFT_fit(zma_list, setLabels = c("Maize 1", "Maize 2"),
                              cor_method = "pearson")
```

This function returns a list with the optimal \beta powers and a summary plot, 
exactly as `SFT_fit()` does.

```{r sft_results, fig.width=8}
powers <- cons_sft$power
powers
cons_sft$plot
```

Now, we can infer GCNs and identify consensus modules across data sets.

```{r consensus_modules_identification}
consensus <- consensus_modules(zma_list, power = powers, cor_method = "pearson")
names(consensus)
head(consensus$genes_cmodules)
```

Finally, we can correlate consensus module eigengenes to sample metadata 
(here, plant tissues).[^1]

[^1]: **NOTE:** Blank grey cells in the heatmap represent correlation 
values that have opposite sign in the expression sets. For each correlation 
pair, consensus correlations are calculated by selecting the minimum value 
across matrices of consensus module-trait correlations.

```{r consensus_trait_cor, fig.width=5, fig.height=5}
consensus_trait <- consensus_trait_cor(consensus, cor_method = "pearson")
head(consensus_trait)
```

As with the output of `module_trait_cor()`, users can visualize the output
of `consensus_trait_cor()` with the function `plot_module_trait_cor()`.

```{r plot-consensus-trait-cor, fig.width = 5, fig.height = 6}
plot_module_trait_cor(consensus_trait)
```

# Module preservation

Module preservation is often used to study patterns of evolutionary 
conservation and divergence across transcriptomes, an approach 
named *phylotranscriptomics*. This way, one can investigate how evolution 
shaped the expression profiles for particular gene families across taxa.

## Data preprocessing

To calculate module preservation statistics, gene IDs must be shared by 
the expression sets. For intraspecies comparisons, this is an easy task, 
as gene IDs are the same. However, for interspecies comparisons, users need 
to identify orthogroups between the different species and collapse the 
gene-level expression values to orthogroup-level expression values. 
This way, all expression sets will have common row names. We recommend 
identifying orthogroups with **OrthoFinder** [@Emms2015], as it is simple 
to use and widely used.[^2] Here, we will compare maize and rice expression 
profiles. The orthogroups between these species were downloaded from the 
PLAZA 4.0 Monocots database [@VanBel2018].

[^2]: **PRO TIP:** If you identify orthogroups with **OrthoFinder**, 
`BioNERO` has a helper function named `parse_orthofinder()` that parses 
the *Orthogroups.tsv* file generated by OrthoFinder into a suitable data frame 
for module preservation analysis. See `?parse_orthofinder` for more details.

```{r load_orthogroups}
data(og.zma.osa)
head(og.zma.osa)
```

As you can see, the orthogroup object for `BioNERO` must be a data frame with 
orthogroups, species IDs and gene IDs, respectively. Let's collapse gene-level 
expression to orthogroup-level with `exp_genes2orthogroups()`. By default, if 
there is more than one gene in the same orthogroup for a given species, their 
expression levels are summarized to the median. Users can also summarize 
to the mean.


```{r genes2orthogorups}
# Store SummarizedExperiment objects in a list
zma_osa_list <- list(osa = osa.se, zma = zma.se)

# Collapse gene-level expression to orthogroup-level
ortho_exp <- exp_genes2orthogroups(zma_osa_list, og.zma.osa, summarize = "mean")

# Inspect new expression data
ortho_exp$osa[1:5, 1:5]
ortho_exp$zma[1:5, 1:5]
```

Now, we will preprocess both expression sets and keep only the top 1000 
orthogroups with the highest variances for demonstration purposes. 

```{r create_list_preservation}
# Preprocess data and keep top 1000 genes with highest variances
ortho_exp <- lapply(ortho_exp, exp_preprocess, variance_filter=TRUE, n=1000)

# Check orthogroup number
sapply(ortho_exp, nrow)
```

## Calculating module preservation statistics

Now that row names are comparable, we can infer GCNs for each set. We will 
do that iteratively with lapply.

```{r gcn_inference}
# Calculate SFT power
power_ortho <- lapply(ortho_exp, SFT_fit, cor_method="pearson")

# Infer GCNs
gcns <- lapply(seq_along(power_ortho), function(n) 
  exp2gcn(ortho_exp[[n]], SFTpower = power_ortho[[n]]$power, 
          cor_method = "pearson")
  )

length(gcns)
```

Initially, module preservation analyses were performed with WGCNA's 
algorithm [@Langfelder2008]. However, the summary preservation statistics 
used by WGCNA rely on parametric assumptions that are often not met. 
For this purpose, the NetRep algorithm [@Ritchie2016] is more accurate 
than WGCNA, as it uses non-parametric permutation analyses. 
Both algorithms are implemented in `BioNERO` for comparison purposes, 
but we **strongly recommend** using the NetRep algorithm. 
Module preservation analysis can be performed with a 
single function: `module_preservation()`.

```{r preservation}
# Using rice as reference and maize as test
pres <- module_preservation(ortho_exp, 
                            ref_net = gcns[[1]], 
                            test_net = gcns[[2]], 
                            algorithm = "netrep")
```

None of the modules in rice were preserved in maize. This can be 
either due to the small number of orthogroups we have chosen or to the 
natural biological variation between species and sampled tissues. You can 
(and should) include more orthogroups in your analyses for a better view of 
transcriptional conservation between species.

# Identifying singletons and duplicated genes

Finally, `BioNERO` can identify singletons and duplicated genes 
with `is_singleton()`. This function returns logical vectors indicating if 
each of the input genes is a singleton or not.

```{r singleton}
# Sample 50 random genes
genes <- sample(rownames(zma.se), size = 50)
is_singleton(genes, og.zma.osa)
```


# Session info {.unnumbered}

This vignette was created under the following conditions:

```{r sessionInfo, echo=FALSE}
sessionInfo()
```

# References

---
title: "Gene coexpression network inference"
author: 
- name: Fabricio Almeida-Silva
  affiliation: Universidade Estadual do Norte Fluminense Darcy Ribeiro, RJ, Brazil
- name: Thiago Motta Venancio
  affiliation: Universidade Estadual do Norte Fluminense Darcy Ribeiro, RJ, Brazil
output: 
  BiocStyle::html_document:
    toc: true
    number_sections: yes
bibliography: vignette1.bib
vignette: >
  %\VignetteIndexEntry{Gene coexpression network inference}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r setup, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  message = TRUE,
  warning = FALSE,
  cache = FALSE,
  fig.align = 'center',
  fig.width = 5,
  fig.height = 4,
  crop = NULL
)
```

# Introduction

To date, several packages have been developed to infer gene coexpression 
networks from expression data, such as WGCNA [@Langfelder2008], 
CEMiTool [@Russo2018] and petal [@Petereit2016]. However, network inference 
and analysis is a non-trivial task that requires solid statistical background, 
especially for data preprocessing and proper interpretation of results. 
Because of that, inexperienced researchers often struggle to choose the most 
suitable algorithms for their projects. Besides, different packages are 
required for each step of a standard network analysis, and their distinct 
syntaxes can hinder interoperability between packages, particularly for 
non-advanced R users. Here, we have developed an all-in-one R package that 
uses state-of-the-art algorithms to facilitate the workflow of biological 
network analysis, from data acquisition to analysis and interpretation. 
This will likely accelerate network analysis pipelines and advance 
systems biology research.

# Installation

```{r installation, eval=FALSE}
if(!requireNamespace('BiocManager', quietly = TRUE))
  install.packages('BiocManager')

BiocManager::install("BioNERO")
```

```{r load_package}
# Load package after installation
library(BioNERO)
set.seed(123) # for reproducibility
```

# Data loading and preprocessing

For this tutorial, we will use maize (*Zea mays*) gene expression data 
normalized in TPM. The data were obtained from @Shin2020 and were filtered 
for package size issues. For more information on the data set, see `?zma.se`. 
The data set is stored as a SummarizedExperiment object.[^1]

[^1]: **NOTE:** In case you have many tab-separated expression tables in a 
directory, `BioNERO` has a helper function named `dfs2one()` to load all these 
files and combine them into a single data frame.

The input expression data in `BioNERO` can be both a SummarizedExperiment 
object or a gene expression matrix or data frame with genes in rows and samples 
in columns. However, we strongly recommend using SummarizedExperiment objects 
for easier interoperability with other Bioconductor packages.

```{r data_loading_soybean, message=FALSE}
data(zma.se)

# Take a quick look at the data
zma.se
SummarizedExperiment::colData(zma.se)
```

## Step-by-step data preprocessing

This section is suitable for users who want to have more control of their data 
analysis, as they can inspect the data set after each preprocessing step and 
analyze how different options to the arguments would affect the expression data. 
If you want a quick start, you can skip to the next 
section (*Automatic, one-step data preprocessing*).

**Step 1:** Replacing missing values. By default, `replace_na()` will replace 
NAs with 0. Users can also replace NAs with the mean of each row 
(generally not advisable, but it can be useful in very specific cases).

```{r remove_na}
exp_filt <- replace_na(zma.se)
sum(is.na(zma.se))
```

**Step 2:** Removing non-expressed genes. Here, for faster network 
reconstruction, we will remove every gene whose median value is below 10. 
The function's default for `min_exp` is 1. 
For other options, see `?remove_nonexp`.

```{r remove_nonexp}
exp_filt <- remove_nonexp(exp_filt, method = "median", min_exp = 10)
dim(exp_filt)
```

**Step 3 (optional):** Filtering genes by variance. It is reasonable to remove 
genes whose expression values do not vary much across samples, since we often 
want to find genes that are more or less expressed in particular conditions. 
Here, we will keep only the top 2000 most variable genes. Users can also filter 
by percentile (e.g., the top 10% most variable genes).

```{r filter_by_variance}
exp_filt <- filter_by_variance(exp_filt, n = 2000)
dim(exp_filt)
```

**Step 4:** Removing outlying samples. There are several methods to remove 
outliers. We have implemented the Z.K (standardized connectivity) 
method [@Oldham2012] in `ZKfiltering()`, which is a network-based approach to 
remove outliers. This method has proven to be more suitable for network 
analysis, since it can remove outliers that other methods 
(such as hierarchical clustering) cannot identify. By default, BioNERO 
considers all samples with ZK \< 2 as outliers, but this parameter 
is flexible if users want to change it.

```{r ZKfiltering}
exp_filt <- ZKfiltering(exp_filt, cor_method = "pearson")
dim(exp_filt)
```

**Step 5:** Adjusting for confounding artifacts. This is an important step 
to avoid spurious correlations resulting from confounders. The method was 
described by @Parsana2019, who developed a principal component (PC)-based 
correction for confounders. After correction, the expression data are quantile 
normalized, so every gene follows an approximate normal distribution.

```{r PC_correction}
exp_filt <- PC_correction(exp_filt)
```

## Automatic, one-step data preprocessing

Alternatively, users can preprocess their data with a single function. 
The function `exp_preprocess()` is a wrapper for the functions `replace_na()`, 
`remove_nonexp()`, `filter_by_variance()`, `ZKfiltering()` and `PC_correction()`. 
The arguments passed to `exp_preprocess()` will be used by each of these 
functions to generate a filtered expression data frame in a single step.[^2]

[^2]: **NOTE:** Here, we are using TPM-normalized data. If you have expression 
data as raw read counts, set the argument `vstransform = TRUE` 
in `exp_preprocess()`. This will apply DESeq2's variance stabilizing 
transformation [@Love2014] to your count data.

```{r exp_preprocess}
final_exp <- exp_preprocess(
    zma.se, min_exp = 10, variance_filter = TRUE, n = 2000
)
identical(dim(exp_filt), dim(final_exp))

# Take a look at the final data
final_exp
```

```{r include=FALSE}
# This object is no longer necessary
rm(exp_filt)
```

# Exploratory data analysis

`BioNERO` includes some functions for easy data exploration. These functions 
were created to avoid having to type code chunks that, although small, will be 
used many times. The idea here is to make the user experience with biological 
network analysis as easy and simple as possible.

**Plotting heatmaps:** the function `plot_heatmap()` plots heatmaps of 
correlations between samples or gene expression in a single line. 
Besides the arguments users can pass to parameters in `plot_heatmap()`, 
they can also pass additional arguments to parameters 
in `ComplexHeatmap::pheatmap()` to have additional control additional on
plot aesthetics (e.g., hide/show gene and sample names, activate/deactivate 
clustering for rows and/or columns, etc).

```{r plot_heatmap, fig.width=6, fig.height=4, message=FALSE}
# Heatmap of sample correlations
p <- plot_heatmap(final_exp, type = "samplecor", show_rownames = FALSE)
p

# Heatmap of gene expression (here, only the first 50 genes)
p <- plot_heatmap(
    final_exp[1:50, ], type = "expr", show_rownames = FALSE, show_colnames = FALSE
)
p
```

**Principal component analysis (PCA):** the function `plot_PCA()` performs a 
PCA and plots whatever pair of PCs users choose (PC1 and PC2 by default), as 
well the percentage of variance explained by each PC.

```{r pcaplot, fig.small=TRUE}
plot_PCA(final_exp)
```

# Gene coexpression network inference

Now that we have our filtered and normalized expression data, we can 
reconstruct a gene coexpression network (GCN) with the 
WGCNA algorithm [@Langfelder2008]. First of all, we need to identify the 
most suitable $\beta$ power that makes the network satisfy the scale-free 
topology. We do that with the function `SFT_fit()`. Correlation values are 
raised to a power $\beta$ to amplify their distances and, hence, to make the 
module detection algorithm more powerful. The higher the value of $\beta$, the 
closer to the scale-free topology the network is. However, a very high $\beta$ 
power reduces mean connectivity, which is not desired. To solve this trade-off, 
we pick the lowest $\beta$ power above a certain threshold (by default 
in `SFT_fit()`, 0.8). This makes the network close to the scale-free topology 
without dramatically reducing the mean connectivity.

```{r sft_fit}
sft <- SFT_fit(final_exp, net_type = "signed hybrid", cor_method = "pearson")
sft$power
power <- sft$power
```

As we can see, the optimal power is `r power`. However, 
we **strongly recommend** a visual inspection of the simulation of 
different $\beta$ powers, as WGCNA can fail to return the most 
appropriate $\beta$ power in some cases.[^3] The function `SFT_fit()` 
automatically saves a ggplot object in the second element of the resulting 
list. To visualize it, you simply have to access the plot.

[^3]: **PRO TIP:** If your $\beta$ power is too low (say below 6), look at the plot as a sanity check.

```{r plot_sft, fig.width=8}
sft$plot
```

Now, we can use the power calculated by `SFT_fit()` to infer the GCN. 
The function `exp2gcn()` infers a GCN and outputs a list of 7 elements, each of 
which will be used by other functions in the analysis pipeline.

```{r exp2net}
net <- exp2gcn(
    final_exp, net_type = "signed hybrid", SFTpower = power, 
    cor_method = "pearson"
)
names(net)
```

The function `exp2gcn()` saves objects in the last element of the resulting 
list that can be subsequently used to plot common figures in GCN papers. 
The figures are publication-ready and display i. a dendrogram of genes and 
modules; ii. heatmap of pairwise correlations between module eigengenes.

```{r dendro}
# Dendro and colors
plot_dendro_and_colors(net)
```

```{r eigengene_net, fig.height=4, fig.width=6}
# Eigengene networks
plot_eigengene_network(net)
```

Let's see the number of genes per module.

```{r genes_per_module, fig.wide=TRUE}
plot_ngenes_per_module(net)
```

# Gene coexpression network analysis

Now that we have our coexpression network, we can start exploring some 
of its properties.

## Assessing module stability

The function `module_stability()` allows users to check if the identified 
coexpression modules are stable (i.e., if they can resist removal of a 
particular sample). This function will resample the data set and rerun the 
module detection algorithm *n* times (default: 30) and return a PDF figure 
displaying a gene dendrogram and colors representing modules identified in 
each run. By looking at the figure, you can detect if a particular module is 
only found in a very small fraction of the runs, which suggests instability. 
Here, we will perform only 5 resampling runs for demonstration purposes.[^4]

[^4]: **NOTE:** The calculations performed by this function may take a 
long time depending on the your network size. Use it only if you have 
some reason to suspect that the modules are highly dependent on a particular 
set of samples.

```{r module_stability}
module_stability(final_exp, net, nRuns = 5)
```

## Module-trait associations

The function `module_trait_cor()` can be used to calculate module-trait 
correlations. This analysis is useful to identify modules that are positively 
or negatively correlated with particular traits, which means that their gene 
expression levels go up or down in these conditions. Here, tissues will be 
considered traits, so we want to identify groups of genes whose expression 
levels are inhibited or induced in particular tissues. Alternatively, one can 
use continuous variables (e.g., metabolite content, protein concentration, 
height) or discrete variables (e.g., disease index) as traits.[^5]

[^5]: **NOTE:** The function `gene_significance()` works just 
like `module_trait_cor()`, but it correlates individual genes (not the 
whole module) to traits. This function is very useful if you have a set of 
candidate genes and you want to find which of them are more associated with 
the trait of interest. See `?gene_significance()` for more details.

```{r moddtraitcor}
MEtrait <- module_trait_cor(exp = final_exp, MEs = net$MEs)
head(MEtrait)
```

Next, you can use the function `plot_module_trait_cor()` to visualize the output
of `module_trait_cor()` as follows:

```{r plot-module-trait-cor, fig.width = 5, fig.height = 5}
plot_module_trait_cor(MEtrait)
```


## Visualizing module expression profile

The heatmap above shows that genes in the *yellow* module are negatively 
correlated with endosperm samples. We can visually explore it 
with `plot_expression_profile()`.

```{r exp_profile, fig.width=5, fig.height=3}
plot_expression_profile(
    exp = final_exp, 
    net = net, 
    plot_module = TRUE, 
    modulename = "yellow"
)
```

## Enrichment analysis

After identifying modules that are inhibited or enhanced in particular 
tissues, users would likely want to find to which biological processes 
(e.g., GO biological process) or pathways (e.g., Reactome, KEGG, MapMan) 
these genes are related. This can be done with enrichment analyses, which 
can uncover terms that are found more than expected by chance in a module.

The easiest way to accomplish this is to use the 
function `module_enrichment()`, which performs enrichment analysis for 
all modules at once. To illustrate it, we will scan coexpression modules 
for enriched protein domains using all genes in the network as background. 
The Interpro annotation was downloaded from 
the PLAZA 4.0 Monocots database [@VanBel2018].

```{r module_enrichment}
# Enrichment analysis for conserved protein domains (Interpro)
data(zma.interpro)
interpro_enrichment <- module_enrichment(
    net = net, 
    background_genes = rownames(final_exp),
    annotation = zma.interpro
)

# Print results without geneIDs for better visualization
interpro_enrichment[, -6]
```

As we can see, two modules are enriched in genes with particular protein 
domains. We could get the same result with the 
function `enrichment_analysis()`, which performs enrichment analysis for 
a user-defined gene set instead of all modules. [^6]

[^6]: **NOTE:** The functions `module_enrichment()` 
and `enrichment_analysis()` can be parallelized with `BiocParallel` to 
increase speed. The default parallel back-end is *SerialParam()*, but this can 
be modified in the argument `bp_param`.

## Hub gene identification

Hub genes are often identified using two different 
metrics: **module membership (MM)** (i.e., correlation of a gene to its 
module eigengene) and **degree** (i.e., sum of connection weights of a 
gene to all other genes in the module). Some researchers consider the 
top 10% genes with the highest degree as hubs, while others consider those 
with MM \> 0.8. To avoid false positives, `BioNERO`'s algorithm combines 
both metrics and defines hub genes as the top 10% genes with highest degree 
that have MM \> 0.8. Hubs can be identified with the function `get_hubs_gcn()`.

```{r}
hubs <- get_hubs_gcn(final_exp, net)
head(hubs)
```

## Extracting subgraphs

Subgraph extraction can be particularly useful to visualize specific 
modules, and it can be done with the function `get_edge_list()`. The 
function returns the subgraph as an edge list. Users can also extract an 
edge list for a particular gene set instead of a module.

```{r}
edges <- get_edge_list(net, module="midnightblue")
head(edges)
```

The function `get_edge_list()` returns a fully connected subgraph for 
the specified module or gene set. However, filtering weak correlations is 
desirable and can be accomplished by setting the argument `filter = TRUE`, 
which will remove edges based on one of optimal scale-free topology 
fit (default), p-value, Z-score, or an arbitrary minimum correlation 
coefficient. [^7] For more details details, check `?get_edge_list()`.

[^7]: **PRO TIP:** Generally, we advise you to filter by optimal scale-free 
topology fit (default). However, if you want to specify your own correlation 
filter for some reason (e.g., visualization), we **strongly recommend** using 
the function `check_SFT()` to check if your resulting graph satisfies the
scale-free topology. If it does not, then your graph does not resemble *real* 
biological networks and, hence, one cannot trust it for 
biological interpretations.

```{r filter_edges, fig.small=TRUE}
# Remove edges based on optimal scale-free topology fit
edges_filtered <- get_edge_list(net, module = "midnightblue", filter = TRUE)
dim(edges_filtered)

# Remove edges based on p-value
edges_filtered <- get_edge_list(
    net, module = "midnightblue",
    filter = TRUE, method = "pvalue", 
    nSamples = ncol(final_exp)
)
dim(edges_filtered)

# Remove edges based on minimum correlation
edges_filtered <- get_edge_list(
    net, module = "midnightblue", 
    filter = TRUE, method = "min_cor", rcutoff = 0.7
)
dim(edges_filtered)
```

## Network visualization

As we now have an edge list for a module, let's visualize it with the 
function `plot_gcn()`. By default, this function only labels the top 5 hubs 
(or less if there are less than 5 hubs). However, this can be customized 
according to the user's preference (see `?plot_gcn` for more information).

```{r plot_gcn, fig.width=5, fig.height=5}
plot_gcn(
    edgelist_gcn = edges_filtered, 
    net = net, 
    color_by = "module", 
    hubs = hubs
)
```

Networks can also be visualized interactively by 
setting `interactive = TRUE` in `plot_gcn`.

```{r interactive_gcn}
plot_gcn(
    edgelist_gcn = edges_filtered, 
    net = net,
    color_by = "module",
    hubs = hubs,
    interactive = TRUE,
    dim_interactive = c(500, 500)
)
```

## Network statistics

Finally, the function `net_stats()` can be used to calculate the main 
network statistics (or properties, or indices), namely: *connectivity*,
*scaled connectivity*, *clustering coefficient*, *maximum adjacency ratio*, 
*density*, *centralization*, *heterogeneity*, *diameter*, 
*betweenness* (optional), and *closeness* (optional).

Depending on your system capacities and network size, this may take 
a very long time. Hence, if you are willing to calculate network statistics 
for your data set, grab a cup of coffee, because the waiting may be long.

# Session information {.unnumbered}

This vignette was created under the following conditions:

```{r}
sessionInfo()
```

# References

---
title: "Gene regulatory network inference"
author: 
- name: Fabricio Almeida-Silva
  affiliation: Universidade Estadual do Norte Fluminense Darcy Ribeiro, RJ, Brazil
- name: Thiago Motta Venancio
  affiliation: Universidade Estadual do Norte Fluminense Darcy Ribeiro, RJ, Brazil
output: 
  BiocStyle::html_document:
    toc: true
    number_sections: yes
bibliography: vignette2.bib
vignette: >
  %\VignetteIndexEntry{Gene regulatory network inference with BioNERO}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  message = TRUE,
  warning = FALSE,
  cache = FALSE,
  fig.align = 'center',
  fig.width = 5,
  fig.height = 4,
  crop = NULL
)
```

# Installation

```{r installation, eval=FALSE}
if(!requireNamespace('BiocManager', quietly = TRUE))
  install.packages('BiocManager')

BiocManager::install("BioNERO")
```

```{r load_package}
# Load package after installation
library(BioNERO)
set.seed(123) # for reproducibility
```


# Introduction and algorithm description

In the previous vignette, we explored all aspects of gene coexpression 
networks (GCNs), which are represented as **undirected weighted graphs**. It 
is **undirected** because, for a given link between *gene A* and *gene B*, we 
can only say that these genes are coexpressed, but we cannot know 
whether *gene A* controls *gene B* or otherwise. Further, **weighted** means 
that some coexpression relationships between gene pairs are stronger than 
others. In this vignette, we will demonstrate how to infer gene regulatory 
networks (GRNs) from expression data with BioNERO. GRNs display interactions 
between regulators (e.g., transcription factors or miRNAs) and their targets 
(e.g., genes). Hence, they are represented as **directed unweighted graphs**.

Numerous algorithms have been developed to infer GRNs from expression data. 
However, the algorithm performances are highly dependent on the benchmark 
data set. To solve this uncertainty, @Marbach2012 proposed the application 
of the *"wisdom of the crowds"* principle to GRN inference. This approach 
consists in inferring GRNs with different algorithms, ranking the interactions 
identified by each method, and calculating the average rank for each 
interaction across all algorithms used. This way, we can have consensus, 
high-confidence edges to be used in biological interpretations. 
For that, `BioNERO` implements three popular algorithms: 
GENIE3 [@Huynh-Thu2010], ARACNE [@Margolin2006] and CLR [@Faith2007].


# Data preprocessing

Before inferring the GRN, we will preprocess the expression data the 
same way we did in the previous vignette.

```{r}
# Load example data set
data(zma.se)

# Preprocess the expression data
final_exp <- exp_preprocess(
    zma.se, 
    min_exp = 10, 
    variance_filter = TRUE, 
    n = 2000
)
```


# Gene regulatory network inference

`BioNERO` requires only 2 objects for GRN inference: the **expression data** 
(SummarizedExperiment, matrix or data frame) and a character vector 
of **regulators** (transcription factors or miRNAs). The transcription factors 
used in this vignette were downloaded from PlantTFDB 4.0 [@Jin2017].

```{r load_tfs}
data(zma.tfs)
head(zma.tfs)
```

## Consensus GRN inference

Inferring GRNs based on the *wisdom of the crowds* principle can be done with 
a single function: `exp2grn()`. This function will infer GRNs with GENIE3, 
ARACNE and CLR, calculate average ranks for each interaction and filter the 
resulting network based on the optimal scale-free topology (SFT) fit. In the 
filtering step, *n* different networks are created by subsetting the top *n* 
quantiles. For instance, if a network of 10,000 edges is given as input 
with `nsplit = 10`, 10 different networks will be created: the first 
with 1,000 edges, the second with 2,000 edges, and so on, with the last 
network being the original input network. Then, for each network, the function 
will calculate the SFT fit and select the best fit.

```{r exp2grn, fig.small=TRUE}
# Using 10 trees for demonstration purposes. Use the default: 1000
grn <- exp2grn(
    exp = final_exp, 
    regulators = zma.tfs$Gene, 
    nTrees = 10
)
head(grn)
```


## Algorithm-specific GRN inference

This section is directed to users who, for some reason 
(e.g., comparison, exploration), want to infer GRNs with particular algorithms. 
The available algorithms are:

**GENIE3:** a regression-tree based algorithm that decomposes the 
prediction of GRNs for *n* genes into *n* regression problems. For each 
regression problem, the expression profile of a target gene is predicted 
from the expression profiles of all other genes using random forests (default) 
or extra-trees.

```{r genie3}
# Using 10 trees for demonstration purposes. Use the default: 1000
genie3 <- grn_infer(
    final_exp, 
    method = "genie3", 
    regulators = zma.tfs$Gene, 
    nTrees = 10)
head(genie3)
dim(genie3)
```


**ARACNE:** information-theoretic algorithm that aims to remove indirect 
interactions inferred by coexpression.

```{r aracne}
aracne <- grn_infer(final_exp, method = "aracne", regulators = zma.tfs$Gene)
head(aracne)
dim(aracne)
```


**CLR:** extension of the relevance networks algorithm that uses mutual 
information to identify regulatory interactions.

```{r clr}
clr <- grn_infer(final_exp, method = "clr", regulators = zma.tfs$Gene)
head(clr)
dim(clr)
```


Users can also infer GRNs with the 3 algorithms at once using the 
function `exp_combined()`. The resulting edge lists are stored in a list 
of 3 elements. [^1]

[^1]: **NOTE:** Under the hood, `exp2grn()` uses `exp_combined()` followed 
by averaging ranks with `grn_average_rank()` and filtering with `grn_filter()`.

```{r grn_combined}
grn_list <- grn_combined(final_exp, regulators = zma.tfs$Gene, nTrees = 10)
head(grn_list$genie3)
head(grn_list$aracne)
head(grn_list$clr)
```

# Gene regulatory network analysis

After inferring the GRN, `BioNERO` allows users to perform some common 
downstream analyses.

## Hub gene identification

GRN hubs are defined as the top 10% most highly connected regulators, but 
this percentile is flexible in `BioNERO`.[^2] They can be identified 
with `get_hubs_grn()`. 

[^2]: **NOTE:** Remember: GRNs are represented as **directed** graphs. 
This implies that only regulators are taken into account when identifying 
hubs. The goal here is to identify regulators (e.g., transcription factors) 
that control the expression of several genes.

```{r get_hubs}
hubs <- get_hubs_grn(grn)
hubs
```

## Network visualization


```{r plot_static, fig.height=4, fig.width=4}
plot_grn(grn)
```

GRNs can also be visualized interactively for exploratory purposes.

```{r plot_interactive}
plot_grn(grn, interactive = TRUE, dim_interactive = c(500,500))
```

Finally, `BioNERO` can also be used for visualization and hub identification 
in protein-protein (PPI) interaction networks. The functions `get_hubs_ppi()` 
and `plot_ppi()` work the same way as their equivalents for 
GRNs (`get_hubs_grn()` and `plot_grn()`).


# Session information {.unnumbered}

This vignette was created under the following conditions:

```{r}
sessionInfo()
```

# References

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_preprocess.R
\name{exp_preprocess}
\alias{exp_preprocess}
\title{Preprocess expression data for network reconstruction}
\usage{
exp_preprocess(
  exp,
  NA_rm = TRUE,
  replaceby = 0,
  Zk_filtering = TRUE,
  zk = -2,
  cor_method = "spearman",
  remove_nonexpressed = TRUE,
  method = "median",
  min_exp = 1,
  min_percentage_samples = 0.25,
  remove_confounders = TRUE,
  variance_filter = FALSE,
  n = NULL,
  percentile = NULL,
  vstransform = FALSE
)
}
\arguments{
\item{exp}{A gene expression data frame with genes in row names
and samples in column names or a `SummarizedExperiment` object.}

\item{NA_rm}{Logical. It specifies whether to remove missing values
from the expression data frame or not. Default = TRUE.}

\item{replaceby}{If NA_rm is TRUE, what to use instead of NAs.
One of 0 or 'mean'. Default is 0.}

\item{Zk_filtering}{Logical. It specifies whether to filter outlying samples
by Zk or not. Default: TRUE.}

\item{zk}{If Zk_filtering is TRUE, the standardized connectivity threshold.
Samples below this threshold will be considered outliers. Default is -2.}

\item{cor_method}{If Zk_filtering is TRUE, the correlation method to use.
One of 'spearman', 'bicor', or 'pearson'. Default is 'spearman'.}

\item{remove_nonexpressed}{Logical. It specifies whether non-expressed genes
should be removed or not. Default is TRUE.}

\item{method}{If remove_nonexpressed is TRUE, the criterion to filter
non-expressed genes out. One of "mean", "median", "percentage",
or "allsamples". Default is 'median'.}

\item{min_exp}{If method is 'mean', 'median', or 'allsamples',
the minimum value for a gene to be considered expressed.
If method is 'percentage', the minimum value each gene must have in at least
n percent of samples to be considered expressed.}

\item{min_percentage_samples}{If method is 'percentage', expressed genes
must have expression >= min_exp in at least this percentage.
Values must range from 0 to 1. Default = 0.25.}

\item{remove_confounders}{Logical. If TRUE, it removes principal components
that add noise to the data.}

\item{variance_filter}{Logical. If TRUE, it will filter genes by variance.
Default is FALSE.}

\item{n}{If variance_filter is TRUE, the number of
most variable genes to keep.}

\item{percentile}{If variance_filter is TRUE, the percentage of
most variable genes to keep.}

\item{vstransform}{Logical indicating if data should be
variance stabilizing transformed. This parameter can only be set to TRUE
if data is a matrix of raw read counts.}
}
\value{
Processed gene expression data frame with gene IDs in row names
and sample names in column names or `SummarizedExperiment` object.
}
\description{
Preprocess expression data for network reconstruction
}
\examples{
data(zma.se)
exp <- exp_preprocess(zma.se, variance_filter=TRUE, n=1000)
}
\references{
Love, M. I., Huber, W., & Anders, S. (2014). Moderated estimation of
fold change and dispersion for RNA-seq data with
DESeq2. Genome biology, 15(12), 1-21.
}
\seealso{
\code{\link[DESeq2]{varianceStabilizingTransformation}}
}
\author{
Fabricio Almeida-Silva
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_preprocess.R
\name{dfs2one}
\alias{dfs2one}
\title{Combine multiple expression tables (.tsv) into a single data frame}
\usage{
dfs2one(mypath, pattern = ".tsv$")
}
\arguments{
\item{mypath}{Path to directory containing .tsv files.
Files must have the first column in common, e.g. "Gene_ID".
Rows are gene IDs and columns are sample names.}

\item{pattern}{Pattern contained in each expression file.
Default is '.tsv$', which means that all files ending in '.tsv' in the
specified directory will be considered expression files.}
}
\value{
Data frame with gene IDs as row names and their expression values in
each sample (columns).
}
\description{
This function reads multiple expression tables (.tsv files) in a directory
and combines them into a single gene expression data frame.
}
\examples{
# Simulate two expression data frames of 100 genes and 30 samples
genes <- paste0(rep("Gene", 100), 1:100)
samples1 <- paste0(rep("Sample", 30), 1:30)
samples2 <- paste0(rep("Sample", 30), 31:60)
exp1 <- cbind(genes, as.data.frame(matrix(rnorm(100*30),nrow=100,ncol=30)))
exp2 <- cbind(genes, as.data.frame(matrix(rnorm(100*30),nrow=100,ncol=30)))
colnames(exp1) <- c("Gene", samples1)
colnames(exp2) <- c("Gene", samples2)

# Write data frames to temporary files
tmpdir <- tempdir()
tmp1 <- tempfile(tmpdir = tmpdir, fileext = ".exp.tsv")
tmp2 <- tempfile(tmpdir = tmpdir, fileext = ".exp.tsv")
write.table(exp1, file=tmp1, quote=FALSE, sep="\t")
write.table(exp2, file=tmp2, quote=FALSE, sep="\t")

# Load the files into one
exp <- dfs2one(mypath = tmpdir, pattern=".exp.tsv")
}
\author{
Fabricio Almeida-Silva
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gcn_inference.R
\name{plot_eigengene_network}
\alias{plot_eigengene_network}
\title{Plot eigengene network}
\usage{
plot_eigengene_network(gcn, palette = "PRGn")
}
\arguments{
\item{gcn}{List object returned by \code{exp2gcn}.}

\item{palette}{Character indicating the name of the RColorBrewer palette
to use. Default: "PRGn".}
}
\value{
A base plot with the eigengene network
}
\description{
Plot eigengene network
}
\examples{
data(filt.se)
gcn <- exp2gcn(filt.se, SFTpower = 18, cor_method = "pearson")
plot_eigengene_network(gcn)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{og.zma.osa}
\alias{og.zma.osa}
\title{Orthogroups between maize and rice}
\format{
A 3-column data frame with orthogroups, species IDs and gene IDs.
}
\usage{
data(og.zma.osa)
}
\description{
The orthogroups were downloaded from the PLAZA 4.0 Monocots database.
}
\examples{
data(og.zma.osa)
}
\references{
Van Bel, M., Diels, T., Vancaester, E., Kreft, L., Botzki, A.,
Van de Peer, Y., ... & Vandepoele, K. (2018). PLAZA 4.0: an integrative
resource for functional, evolutionary and comparative plant
genomics. Nucleic acids research, 46(D1), D1190-D1196.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/network_visualization.R
\name{plot_gcn}
\alias{plot_gcn}
\title{Plot gene coexpression network from edge list}
\usage{
plot_gcn(
  edgelist_gcn,
  net,
  color_by = "module",
  hubs = NULL,
  show_labels = "tophubs",
  top_n_hubs = 5,
  curvature = 0,
  interactive = FALSE,
  dim_interactive = c(600, 600)
)
}
\arguments{
\item{edgelist_gcn}{Data frame containing the edge list for the GCN.
The edge list can be generated with \code{get_edge_list()}.}

\item{net}{List object returned by \code{exp2net}.}

\item{color_by}{How should nodes be colored? It must be either "module"
(nodes will have the colors of their modules) or a 2-column data frame
containing genes in the first column and a custom gene annotation
in the second column. Default: "module".}

\item{hubs}{Data frame containing hub genes in the first column,
their modules in the second column, and intramodular connectivity in
the third column.}

\item{show_labels}{Character indicating which nodes will be labeled.
One of "all", "allhubs", "tophubs", or "none". Default: tophubs.}

\item{top_n_hubs}{Number of top hubs to be labeled. It is only valid
if \code{show_labels} equals "tophubs". Default is 5.}

\item{curvature}{Numeric indicating the amount of curvature in edges.
Negative values produce left-hand curves, positive values produce right-hand
curves, and zero produces a straight line. Default: 0.1.}

\item{interactive}{Logical indicating whether the network should be
interactive or not. Default is FALSE.}

\item{dim_interactive}{Numeric vector with width and height of window
for interactive plotting. Default: c(600,600).}
}
\value{
A ggplot object.
}
\description{
Plot gene coexpression network from edge list
}
\examples{
data(filt.se)
gcn <- exp2gcn(filt.se, SFTpower = 18, cor_method = "pearson")
gcn_edges <- get_edge_list(gcn, module="brown", filter=TRUE,
                           method="min_cor")
hubs <- get_hubs_gcn(filt.se, gcn)
p <- plot_gcn(gcn_edges, gcn, hubs = hubs)
}
\author{
Fabricio Almeida-Silva
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gcn_inference.R
\name{module_stability}
\alias{module_stability}
\title{Perform module stability analysis}
\usage{
module_stability(exp, net, nRuns = 20)
}
\arguments{
\item{exp}{A gene expression data frame with genes in row names and
samples in column names or a `SummarizedExperiment` object.}

\item{net}{List object returned by \code{exp2gcn}.}

\item{nRuns}{Number of times to resample. Default is 20.}
}
\value{
A base plot with the module stability results.
}
\description{
Perform module stability analysis
}
\examples{
data(filt.se)
filt <- filt.se[1:100, ] # reducing even further for testing purposes
# The SFT fit was previously calculated and the optimal power was 16
gcn <- exp2gcn(filt, SFTpower = 16, cor_method = "pearson")
# For simplicity, only 2 runs
module_stability(exp = filt, net = gcn, nRuns = 2)
}
\seealso{
\code{\link[WGCNA]{sampledBlockwiseModules}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gcn_inference.R
\name{SFT_fit}
\alias{SFT_fit}
\title{Pick power to fit network to a scale-free topology}
\usage{
SFT_fit(exp, net_type = "signed", rsquared = 0.8, cor_method = "spearman")
}
\arguments{
\item{exp}{A gene expression data frame with genes in row names and
samples in column names or a `SummarizedExperiment` object.}

\item{net_type}{Network type. One of 'signed', 'signed hybrid' or
'unsigned'. Default is signed.}

\item{rsquared}{R squared cutoff. Default is 0.8.}

\item{cor_method}{Correlation method. One of "pearson", "biweight"
or "spearman". Default is "spearman".}
}
\value{
A list containing:
\itemize{
  \item{\strong{power} Numeric, optimal power based on scale-free topology fit}
  \item{\strong{plot} A ggplot object displaying main statistics of the SFT fit test}
}
}
\description{
Pick power to fit network to a scale-free topology
}
\examples{
data(filt.se)
sft <- SFT_fit(filt.se, cor_method = "pearson")
}
\seealso{
\code{\link[WGCNA]{pickSoftThreshold}}
}
\author{
Fabricio Almeida-Silva
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BioNERO-package.R
\docType{package}
\name{BioNERO-package}
\alias{BioNERO}
\alias{BioNERO-package}
\title{BioNERO: Biological Network Reconstruction Omnibus}
\description{
\if{html}{\figure{logo.png}{options: style='float: right' alt='logo' width='120'}}

BioNERO aims to integrate all aspects of biological network inference in a single package, including data preprocessing, exploratory analyses, network inference, and analyses for biological interpretations. BioNERO can be used to infer gene coexpression networks (GCNs) and gene regulatory networks (GRNs) from gene expression data. Additionally, it can be used to explore topological properties of protein-protein interaction (PPI) networks. GCN inference relies on the popular WGCNA algorithm. GRN inference is based on the "wisdom of the crowds" principle, which consists in inferring GRNs with multiple algorithms (here, CLR, GENIE3 and ARACNE) and calculating the average rank for each interaction pair. As all steps of network analyses are included in this package, BioNERO makes users avoid having to learn the syntaxes of several packages and how to communicate between them. Finally, users can also identify consensus modules across independent expression sets and calculate intra and interspecies module preservation statistics between different networks.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/almeidasilvaf/BioNERO}
  \item Report bugs at \url{https://github.com/almeidasilvaf/BioNERO/issues}
}

}
\author{
\strong{Maintainer}: Fabricio Almeida-Silva \email{fabricio_almeidasilva@hotmail.com} (\href{https://orcid.org/0000-0002-5314-2964}{ORCID})

Authors:
\itemize{
  \item Thiago Venancio (\href{https://orcid.org/0000-0002-2215-8082}{ORCID})
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/network_comparison.R
\name{modPres_WGCNA}
\alias{modPres_WGCNA}
\title{Calculate module preservation between two expression data sets using WGCNA's algorithm}
\usage{
modPres_WGCNA(explist, ref_net, nPerm = 200)
}
\arguments{
\item{explist}{List of expression data frames or SummarizedExperiment objects.}

\item{ref_net}{Reference network object returned by the function \code{exp2net}.}

\item{nPerm}{Number of permutations for the module preservation statistics.
It must be greater than 1. Default: 200.}
}
\value{
A ggplot object with module preservation statistics.
}
\description{
Calculate module preservation between two expression data sets using WGCNA's algorithm
}
\examples{
\donttest{
set.seed(1)
data(og.zma.osa)
data(zma.se)
data(osa.se)
explist <- list(Zma = zma.se, Osa = osa.se)
og <- og.zma.osa
exp_ortho <- exp_genes2orthogroups(explist, og, summarize = "mean")
exp_ortho <- lapply(exp_ortho, function(x) filter_by_variance(x, n=1500))
# Previously calculated power
powers <- c(13, 15)
gcn_osa <- exp2gcn(exp_ortho$Osa, net_type = "signed hybrid",
                   SFTpower = powers[1], cor_method = "pearson")
explist <- exp_ortho
ref_net <- gcn_osa
# 5 permutations for demonstration purposes
pres_wgcna <- modPres_WGCNA(explist, ref_net, nPerm=5)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/exploratory_analysis.R
\name{plot_PCA}
\alias{plot_PCA}
\title{Plot Principal Component Analysis (PCA) of samples}
\usage{
plot_PCA(
  exp,
  metadata,
  metadata_cols = NULL,
  log_trans = FALSE,
  PCs = c(1, 2),
  size = 2
)
}
\arguments{
\item{exp}{A gene expression data frame with genes in row names
and samples in column names or a `SummarizedExperiment` object.}

\item{metadata}{A data frame of sample metadata containing sample names
in row names and sample annotation in subsequent columns.
Ignored if `exp` is a `SummarizedExperiment` object, since colData will be
automatically extracted.}

\item{metadata_cols}{A vector (either numeric or character) indicating
which columns should be extracted from column metadata if \strong{exp}
is a `SummarizedExperiment` object. The vector can contain column
indices (numeric) or column names (character). By default, all columns are
used.}

\item{log_trans}{Logical indicating whether the gene expression matrix
should be log transformed using \code{log(exp + 1)}. Default: FALSE.}

\item{PCs}{Numeric vector of length 2 indicating the principal components
to be plotted on the x-axis and y-axis, respectively.
Default: \code{c(1, 2)}.}

\item{size}{Numeric indicating the point size. Default is 2.}
}
\value{
A ggplot object with the PCA plot.
}
\description{
Plot Principal Component Analysis (PCA) of samples
}
\examples{
data(zma.se)
plot_PCA(zma.se, log_trans = TRUE)
}
\seealso{
\code{\link[ggplot2]{ggplot}}
}
\author{
Fabricio Almeida-Silva
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/exploratory_analysis.R
\name{plot_expression_profile}
\alias{plot_expression_profile}
\title{Plot expression profile of given genes across samples}
\usage{
plot_expression_profile(
  genes,
  exp,
  metadata,
  metadata_cols = 1,
  plot_module = TRUE,
  net,
  modulename,
  bg_line = "mean"
)
}
\arguments{
\item{genes}{Character vector containing a subset of genes from which
edges will be extracted. It can be ignored if \code{plot_module} is TRUE.}

\item{exp}{A gene expression data frame with genes in row names and
samples in column names or a `SummarizedExperiment` object.}

\item{metadata}{A data frame of sample metadata containing sample names
in row names and sample annotation in subsequent columns.
Ignored if `exp` is a `SummarizedExperiment` object, since colData will be
automatically extracted.}

\item{metadata_cols}{A character or numeric scalar indicating
which column should be extracted from column metadata if \strong{exp}
is a `SummarizedExperiment` object. The column to be extracted can be
represented by indices (numeric) or column names (character).
By default, the first column is used.}

\item{plot_module}{Logical indicating whether to plot a whole module or not.
If set to FALSE, \code{genes} must be specified.}

\item{net}{List object returned by \code{exp2gcn}.}

\item{modulename}{Name of the module to plot.}

\item{bg_line}{Character indicating what to show in the background (black)
line. One of "mean" or "median". Default: "mean".}
}
\value{
A ggplot object showing the expression profile of some genes
across all samples.
}
\description{
Plot expression profile of given genes across samples
}
\examples{
data(zma.se)
data(filt.se)
genes <- rownames(filt.se)
plot_expression_profile(genes = genes, exp = zma.se, plot_module = FALSE)
}
\author{
Fabricio Almeida-Silva
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/consensus_modules.R
\name{consensus_modules}
\alias{consensus_modules}
\title{Identify consensus modules across independent data sets}
\usage{
consensus_modules(
  exp_list,
  metadata,
  power,
  cor_method = "spearman",
  net_type = "signed hybrid",
  module_merging_threshold = 0.8,
  TOM_type = NULL,
  verbose = FALSE
)
}
\arguments{
\item{exp_list}{A list containing the expression data frames with genes in
row names and samples in column names or `SummarizedExperiment` objects.
The list can be created by using \code{list(exp1, exp2, ..., expn)}.}

\item{metadata}{A data frame containing sample names in row names and
sample annotation in the first column.
Ignored if `exp_list` is a list of `SummarizedExperiment` objects, since
the function will extract colData.}

\item{power}{Numeric vector of beta power for each expression set
as calculated by \code{consensus_SFT_fit}.}

\item{cor_method}{Correlation method used for network reconstruction.
One of "spearman" (default), "biweight", or "pearson".}

\item{net_type}{Network type. One of "signed hybrid" (default),
"signed" or "unsigned".}

\item{module_merging_threshold}{Correlation threshold to merge
similar modules into a single one. Default: 0.8.}

\item{TOM_type}{Character indicating the type of Topological
Overlap Matrix to (TOM) create. One of
'unsigned', 'signed', 'signed Nowick', 'unsigned 2', 'signed 2',
and 'signed Nowick 2'. By default, TOM type is automatically
selected based on network type.}

\item{verbose}{Logical indicating whether to display progress
messages or not. Default: FALSE.}
}
\value{
A list containing 4 elements: \describe{
  \item{consMEs}{Consensus module eigengenes}
  \item{exprSize}{Description of the multi-set object returned by the function \code{WGCNA::checkSets}}
  \item{sampleInfo}{Metadata for each expression set}
  \item{genes_cmodules}{Data frame of genes and consensus modules}
  \item{dendro_plot_objects}{Objects to be used in dendrogram plotting}
}
}
\description{
Identify consensus modules across independent data sets
}
\examples{
set.seed(12)
data(zma.se)
filt.zma <- filter_by_variance(zma.se, n=500)
zma.set1 <- filt.zma[, sample(colnames(filt.zma), size=20, replace=FALSE)]
zma.set2 <- filt.zma[, sample(colnames(filt.zma), size=20, replace=FALSE)]
list.sets <- list(zma.set1, zma.set2)
# SFT power previously identified with consensus_SFT_fit()
cons_mod <- consensus_modules(list.sets, power = c(11, 13),
                              cor_method = "pearson")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/exploratory_analysis.R
\name{plot_ngenes_per_module}
\alias{plot_ngenes_per_module}
\title{Plot number of genes per module}
\usage{
plot_ngenes_per_module(net = NULL)
}
\arguments{
\item{net}{List object returned by \code{exp2gcn}.}
}
\value{
A ggplot object with a bar plot of gene number in each module.
}
\description{
Plot number of genes per module
}
\examples{
data(filt.se)
gcn <- exp2gcn(filt.se, SFTpower = 18, cor_method = "pearson")
plot_ngenes_per_module(gcn)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/consensus_modules.R
\name{consensus_trait_cor}
\alias{consensus_trait_cor}
\title{Correlate set-specific modules and consensus modules to sample information}
\usage{
consensus_trait_cor(consensus, cor_method = "pearson", metadata_cols = NULL)
}
\arguments{
\item{consensus}{Consensus network returned by \code{consensus_modules}.}

\item{cor_method}{Correlation method to be used. One of 'spearman' or
'pearson'. Default: 'pearson'.}

\item{metadata_cols}{A vector (either numeric or character) indicating
which columns should be extracted from column metadata if \strong{exp}
is a `SummarizedExperiment` object. The vector can contain column
indices (numeric) or column names (character). By default, all columns are
used.}
}
\value{
Data frame of consensus module-trait correlations and p-values,
with the following variables:
\describe{
  \item{trait}{Factor, trait name. Each trait corresponds to a variable
               of the sample metadata (if numeric) or levels of a variable
               (if categorical).}
  \item{ME}{Factor, module eigengene.}
  \item{cor}{Numeric, correlation.}
  \item{pvalue}{Numeric, correlation P-values.}
  \item{group}{Character, name of the metadata variable.}
}
}
\description{
Correlate set-specific modules and consensus modules to sample information
}
\examples{
set.seed(12)
data(zma.se)
filt.zma <- filter_by_variance(zma.se, n=500)
zma.set1 <- filt.zma[, sample(colnames(filt.zma), size=20, replace=FALSE)]
zma.set2 <- filt.zma[, sample(colnames(filt.zma), size=20, replace=FALSE)]
list.sets <- list(zma.set1, zma.set2)
# SFT power previously identified with consensus_SFT_fit()
consensus <- consensus_modules(list.sets, power = c(11, 13),
                               cor_method = "pearson")
consensus_trait <- consensus_trait_cor(consensus, cor_method = "pearson")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gcn_inference.R
\name{plot_module_trait_cor}
\alias{plot_module_trait_cor}
\title{Plot a heatmap of module-trait correlations}
\usage{
plot_module_trait_cor(corandp, palette = "RdYlBu", transpose = FALSE)
}
\arguments{
\item{corandp}{A data frame of module-trait correlations as returned
by \code{module_trait_cor()}.}

\item{palette}{Character indicating which RColorBrewer palette to use.
Default: 'RdYlBu'.}

\item{transpose}{Logical indicating whether to transpose the heatmap
or not.}
}
\value{
A `Heatmap` object created by \code{ComplexHeatmap::pheatmap()}.
}
\description{
Plot a heatmap of module-trait correlations
}
\details{
Significance levels:
1 asterisk: significant at alpha = 0.05.
2 asterisks: significant at alpha = 0.01.
3 asterisks: significant at alpha = 0.001.
no asterisk: not significant.
}
\examples{
data(filt.se)
gcn <- exp2gcn(filt.se, SFTpower = 18, cor_method = "pearson")
corandp <- module_trait_cor(filt.se, MEs = gcn$MEs)
plot_module_trait_cor(corandp)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gcn_inference.R
\name{plot_dendro_and_colors}
\alias{plot_dendro_and_colors}
\title{Plot dendrogram of genes and modules}
\usage{
plot_dendro_and_colors(gcn)
}
\arguments{
\item{gcn}{List object returned by \code{exp2gcn}.}
}
\value{
A base plot with the gene dendrogram and modules.
}
\description{
Plot dendrogram of genes and modules
}
\examples{
data(filt.se)
gcn <- exp2gcn(filt.se, SFTpower = 18, cor_method = "pearson")
plot_dendro_and_colors(gcn)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_preprocess.R
\name{replace_na}
\alias{replace_na}
\title{Remove missing values in a gene expression data frame}
\usage{
replace_na(exp, replaceby = 0)
}
\arguments{
\item{exp}{A gene expression data frame with genes in row names and
samples in column names or a `SummarizedExperiment` object.}

\item{replaceby}{What to use instead of NAs. One of 0 or 'mean'. Default is 0.}
}
\value{
Gene expression data frame or `SummarizedExperiment` object
with all NAs replaced according to the argument 'replaceby'.
}
\description{
Remove missing values in a gene expression data frame
}
\examples{
data(zma.se)
exp <- replace_na(zma.se)
sum(is.na(exp))
}
\author{
Fabricio Almeida-Silva
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gcn_inference.R
\name{module_trait_cor}
\alias{module_trait_cor}
\title{Correlate module eigengenes to trait}
\usage{
module_trait_cor(
  exp,
  metadata,
  MEs,
  metadata_cols = NULL,
  cor_method = "pearson"
)
}
\arguments{
\item{exp}{A gene expression data frame with genes in row names and
samples in column names or a `SummarizedExperiment` object.}

\item{metadata}{A data frame containing sample names in row names and
sample annotation in the first column. Ignored if `exp` is
a `SummarizedExperiment` object, since the function will extract colData.}

\item{MEs}{Module eigengenes. It is the 2nd element of the result list
generated by the function \code{exp2gcn}.}

\item{metadata_cols}{A vector (either numeric or character) indicating
which columns should be extracted from column metadata if \strong{exp}
is a `SummarizedExperiment` object. The vector can contain column
indices (numeric) or column names (character). By default, all columns are
used.}

\item{cor_method}{Method to calculate correlation. One of 'pearson',
'spearman' or 'kendall'. Default is 'spearman'.}
}
\value{
A data frame with correlation and correlation p-values for each pair
of ME and trait, with the following variables:
\describe{
  \item{ME}{Factor, module eigengene.}
  \item{trait}{Factor, trait name. Each trait corresponds to a variable
               of the sample metadata (if numeric) or levels of a variable
               (if categorical).}
  \item{cor}{Numeric, correlation.}
  \item{pvalue}{Numeric, correlation P-values.}
  \item{group}{Character, name of the metadata variable.}
}
}
\description{
Correlate module eigengenes to trait
}
\examples{
data(filt.se)
gcn <- exp2gcn(filt.se, SFTpower = 18, cor_method = "pearson")
module_trait_cor(filt.se, MEs = gcn$MEs)
}
\author{
Fabricio Almeida-Silva
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/consensus_modules.R
\name{consensus_SFT_fit}
\alias{consensus_SFT_fit}
\title{Pick power to fit networks to scale-free topology}
\usage{
consensus_SFT_fit(
  exp_list,
  setLabels = NULL,
  metadata = NULL,
  cor_method = "spearman",
  net_type = "signed hybrid",
  rsquared = 0.8
)
}
\arguments{
\item{exp_list}{A list of expression data frames or
SummarizedExperiment objects.
If input is a list of data frames, row names must correspond to gene IDs
and column names to samples.
The list can be created with \code{list(exp1, exp2, ..., expn)}.}

\item{setLabels}{Character vector containing labels for each expression set.}

\item{metadata}{A data frame containing sample names in row names and
sample annotation in the first column.
Ignored if `exp_list` is a list of `SummarizedExperiment` objects, since
the function will extract colData.}

\item{cor_method}{Correlation method used for network reconstruction.
One of "spearman" (default), "biweight", or "pearson".}

\item{net_type}{Network type. One of "signed hybrid" (default),
"signed" or "unsigned".}

\item{rsquared}{Minimum R squared to consider the network similar to
a scale-free topology. Default is 0.8.}
}
\value{
A list of 2 elements: \describe{
  \item{power}{Numeric vector of optimal beta powers to fit networks to SFT}
  \item{plot}{A ggplot object displaying main statistics of the SFT fit test}
}
}
\description{
Pick power to fit networks to scale-free topology
}
\examples{
set.seed(12)
data(zma.se)
filt.zma <- filter_by_variance(zma.se, n=500)
zma.set1 <- filt.zma[, sample(colnames(filt.zma), size=20, replace=FALSE)]
zma.set2 <- filt.zma[, sample(colnames(filt.zma), size=20, replace=FALSE)]
list.sets <- list(zma.set1, zma.set2)
cons_sft <- consensus_SFT_fit(list.sets, setLabels = c("Maize1", "Maize2"),
                              cor_method = "pearson")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/network_visualization.R
\name{detect_communities}
\alias{detect_communities}
\title{Detect communities in a network}
\usage{
detect_communities(edgelist, method = igraph::cluster_infomap, directed = TRUE)
}
\arguments{
\item{edgelist}{Data frame containing the network as an edge list.
First column must be node 1 and second column must be node 2.
Additional columns will be interpreted as edge attributes and will
be modified by this function.}

\item{method}{igraph function to be used for community detection.
Available functions are cluster_infomap, cluster_edge_betweenness,
cluster_fast_greedy, cluster_walktrap, cluster_spinglass,
cluster_leading_eigen, cluster_louvain, and cluster_label_prop.
Default is cluster_infomap.}

\item{directed}{Logical indicating whether the network is directed (GRN only)
or not (GCN and PPI networks). Default: TRUE.}
}
\value{
A data frame containing node names in the first column, and
communities to which nodes belong in the second column.
}
\description{
Detect communities in a network
}
\examples{
data(filt.se)
tfs <- sample(rownames(filt.se), size=50, replace=FALSE)
grn_edges <- grn_infer(filt.se, method = "clr", regulators = tfs)
com <- detect_communities(grn_edges, directed=TRUE)
}
\seealso{
\code{\link[igraph]{cluster_infomap}},
 \code{\link[igraph]{cluster_edge_betweenness}},
 \code{\link[igraph]{cluster_fast_greedy}},
 \code{\link[igraph]{cluster_walktrap}},
 \code{\link[igraph]{cluster_spinglass}},
 \code{\link[igraph]{cluster_leading_eigen}},
 \code{\link[igraph]{cluster_louvain}},
 \code{\link[igraph]{cluster_label_prop}}
}
\author{
Fabricio Almeida-Silva
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gcn_inference.R
\name{gene_significance}
\alias{gene_significance}
\title{Calculate gene significance for a given group of genes}
\usage{
gene_significance(
  exp,
  metadata,
  metadata_cols = NULL,
  genes = NULL,
  alpha = 0.05,
  cor_method = "pearson",
  min_cor = 0.2,
  use_abs = TRUE
)
}
\arguments{
\item{exp}{A gene expression data frame with genes in row names and
samples in column names or a `SummarizedExperiment` object.}

\item{metadata}{A data frame containing sample names in row names and
sample annotation in the first column. Ignored if `exp` is
a `SummarizedExperiment` object, since the function will extract colData.}

\item{metadata_cols}{A vector (either numeric or character) indicating
which columns should be extracted from column metadata if \strong{exp}
is a `SummarizedExperiment` object. The vector can contain column
indices (numeric) or column names (character). By default, all columns are
used.}

\item{genes}{Character vector of genes to be correlated with traits.
If not given, all genes in `exp` will be considered.}

\item{alpha}{Significance level. Default is 0.05.}

\item{cor_method}{Method to calculate correlation. One of 'pearson',
'spearman' or 'kendall'. Default is 'spearman'.}

\item{min_cor}{Minimum correlation coefficient. Default is 0.2.}

\item{use_abs}{Logical indicating whether to filter by correlation using
absolute value or not. If TRUE, a \code{min_cor} of say 0.2 would keep all
correlations above 0.2 and below -0.2. Default is TRUE.}
}
\value{
A data frame with correlation and correlation p-values for each pair
of gene and trait, with the following variables:
\describe{
  \item{gene}{Factor, gene ID.}
  \item{trait}{Factor, trait name. Each trait corresponds to a variable
               of the sample metadata (if numeric) or levels of a variable
               (if categorical).}
  \item{cor}{Numeric, correlation.}
  \item{pvalue}{Numeric, correlation P-values.}
  \item{group}{Character, name of the metadata variable.}
}
}
\description{
Calculate gene significance for a given group of genes
}
\examples{
data(filt.se)
gs <- gene_significance(filt.se)
}
\author{
Fabricio Almeida-Silva
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{exp2cor}
\alias{exp2cor}
\title{Calculate pairwise correlations between genes in a matrix}
\usage{
exp2cor(exp, cor_method = "pearson")
}
\arguments{
\item{exp}{A numeric matrix containing a gene expression matrix,
with genes in rows and samples in columns.}

\item{cor_method}{Character indicating the correlation method to use.
One of "pearson", "spearman", or "biweight". Default: "pearson".}
}
\value{
A numeric, symmetric matrix with pairwise correlations between
genes.
}
\description{
Calculate pairwise correlations between genes in a matrix
}
\examples{
# Simulate an expression matrix with 100 genes and 50 samples
exp <- matrix(
    rnorm(100 * 50, mean = 10, sd = 2),
    nrow = 100,
    dimnames = list(
        paste0("gene", seq_len(100)),
        paste0("sample", seq_len(50))
    )
)

# Calculate correlation matrix
cor_mat <- exp2cor(exp)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{zma.interpro}
\alias{zma.interpro}
\title{Maize Interpro annotation}
\format{
A 2-column data frame containing gene IDs and their
associated Interpro annotations.
}
\usage{
data(zma.interpro)
}
\description{
Interpro protein domain annotation retrieved from the
PLAZA Monocots 4.0 database.
Only genes included in \code{zma.se} are present in this subset.
}
\examples{
data(zma.interpro)
}
\references{
Van Bel, M., Diels, T., Vancaester, E., Kreft, L., Botzki, A.,
Van de Peer, Y., ... & Vandepoele, K. (2018). PLAZA 4.0: an integrative
resource for functional, evolutionary and comparative plant
genomics. Nucleic acids research, 46(D1), D1190-D1196.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/network_stats.R
\name{net_stats}
\alias{net_stats}
\title{Calculate network statistics}
\usage{
net_stats(
  adj_matrix = NULL,
  net_type = c("gcn", "ppi", "grn"),
  calculate_additional = FALSE
)
}
\arguments{
\item{adj_matrix}{Adjacency matrix that represents the network.}

\item{net_type}{One of "gcn" (gene coexpression network),
"ppi" (protein-protein interaction), or "grn" (gene regulatory network).}

\item{calculate_additional}{Logical indicating whether to calculate
additional network statistics (betweenness and closeness).
Default is FALSE.}
}
\value{
A list containing the following elements: \itemize{
  \item Connectivity
  \item ScaledConnectivity
  \item ClusterCoef
  \item MAR (for gcn only)
  \item Density
  \item Centralization
  \item Heterogeneity (gcn only)
  \item Diameter
  \item Betweenness
  \item Closeness
}
}
\description{
Calculate network statistics
}
\examples{
\donttest{
data(filt.se)
set.seed(12)
filt.se <- exp_preprocess(
    filt.se, Zk_filtering = FALSE, variance_filter = TRUE, n = 200
)
gcn <- exp2gcn(
    filt.se, SFTpower = 7, cor_method = "pearson", net_type = "signed hybrid"
)
stats <- net_stats(gcn$adjacency_matrix, net_type = "gcn")
}
}
\seealso{
\code{\link[igraph]{graph_from_adjacency_matrix}},
 \code{\link[igraph]{cliques}},\code{\link[igraph]{diameter}},
 \code{\link[igraph]{estimate_betweenness}},\code{\link[igraph]{V}},
 \code{\link[igraph]{closeness}},\code{\link[igraph]{degree}},
 \code{\link[igraph]{transitivity}},\code{\link[igraph]{edge_density}},
 \code{\link[igraph]{centr_degree}}
 \code{\link[WGCNA]{fundamentalNetworkConcepts}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gcn_inference.R
\name{plot_gene_significance}
\alias{plot_gene_significance}
\title{Plot a heatmap of gene significance}
\usage{
plot_gene_significance(corandp, palette = "RdYlBu", transpose = FALSE, ...)
}
\arguments{
\item{corandp}{A data frame of gene-trait correlations as returned
by \code{gene_significance()}.}

\item{palette}{Character indicating which RColorBrewer palette to use.
Default: 'RdYlBu'.}

\item{transpose}{Logical indicating whether to transpose the heatmap
or not.}

\item{...}{Additional arguments to \code{ComplexHeatmap::pheatmap()}.}
}
\value{
A `Heatmap` object created by \code{ComplexHeatmap::pheatmap()}.
}
\description{
Plot a heatmap of gene significance
}
\details{
Significance levels:
1 asterisk: significant at alpha = 0.05.
2 asterisks: significant at alpha = 0.01.
3 asterisks: significant at alpha = 0.001.
no asterisk: not significant.
}
\examples{
data(filt.se)
gcn <- exp2gcn(filt.se, SFTpower = 18, cor_method = "pearson")
corandp <- gene_significance(filt.se)
plot_gene_significance(corandp, show_rownames = FALSE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gcn_inference.R
\name{enrichment_analysis}
\alias{enrichment_analysis}
\title{Perform overrepresentation analysis for a set of genes}
\usage{
enrichment_analysis(
  genes,
  background_genes,
  annotation,
  column = NULL,
  correction = "BH",
  p = 0.05,
  min_setsize = 10,
  max_setsize = 500,
  bp_param = BiocParallel::SerialParam()
)
}
\arguments{
\item{genes}{Character vector containing genes for overrepresentation
analysis.}

\item{background_genes}{Character vector of genes to be used as background
for the overrepresentation analysis.}

\item{annotation}{Annotation data frame with genes in the first column and
functional annotation in the other columns. This data frame can be exported
from Biomart or similar databases.}

\item{column}{Column or columns of \strong{annotation} to be used for
enrichment.
Both character or numeric values with column indices can be used.
If users want to supply more than one column, input a character or
numeric vector. Default: all columns from \strong{annotation}.}

\item{correction}{Multiple testing correction method. One of "holm",
"hochberg", "hommel", "bonferroni", "BH", "BY", "fdr" or "none".
Default is "BH".}

\item{p}{P-value threshold. P-values below this threshold will be
considered significant. Default: 0.05.}

\item{min_setsize}{Numeric indicating the minimum gene set size to be
considered. Gene sets correspond to levels of each variable
in \strong{annotation}). Default: 10.}

\item{max_setsize}{Numeric indicating the maximum gene set size to be
considered. Gene sets correspond to levels of each variable
in \strong{annotation}). Default: 500.}

\item{bp_param}{BiocParallel back-end to be used.
Default: BiocParallel::SerialParam()}
}
\value{
A data frame of overrepresentation results with the following
variables:
\describe{
  \item{term}{character, functional term ID/name.}
  \item{genes}{numeric, intersection length between input genes and genes
               in a particular functional term.}
  \item{all}{numeric, number of all genes in a particular functional term.}
  \item{pval}{numeric, P-value for the hypergeometric test.}
  \item{padj}{numeric, P-value adjusted for multiple comparisons using
              the method specified in parameter \strong{adj}.}
  \item{category}{character, name of the grouping variable (i.e., column
                  name of \strong{annotation}).}
}
}
\description{
Perform overrepresentation analysis for a set of genes
}
\examples{
\donttest{
data(filt.se)
data(zma.interpro)
genes <- rownames(filt.se)[1:50]
background_genes <- rownames(filt.se)
annotation <- zma.interpro
# Using p = 1 to show all results
enrich <- enrichment_analysis(genes, background_genes, annotation, p = 1)
}
}
\author{
Fabricio Almeida-Silva
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_preprocess.R
\name{ZKfiltering}
\alias{ZKfiltering}
\title{Filter outlying samples based on the standardized connectivity (Zk) method}
\usage{
ZKfiltering(exp, zk = -2, cor_method = "spearman")
}
\arguments{
\item{exp}{A gene expression data frame with genes in row names
and samples in column names or a `SummarizedExperiment` object.}

\item{zk}{Standardized connectivity threshold. Default is -2.}

\item{cor_method}{Correlation method. One of "pearson", "biweight"
or "spearman". Default is "spearman".}
}
\value{
Filtered gene expression data frame or `SummarizedExperiment` object.
}
\description{
Filter outlying samples based on the standardized connectivity (Zk) method
}
\examples{
data(zma.se)
filt_exp <- ZKfiltering(zma.se)
}
\references{
Oldham, M. C., Langfelder, P., & Horvath, S. (2012). Network methods for
describing sample relationships in genomic datasets: application to
Huntington’s disease. BMC systems biology, 6(1), 1-18.
}
\seealso{
\code{\link[WGCNA]{adjacency}}
}
\author{
Fabricio Almeida-Silva
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_preprocess.R
\name{q_normalize}
\alias{q_normalize}
\title{Quantile normalize the expression data}
\usage{
q_normalize(exp)
}
\arguments{
\item{exp}{A gene expression data frame with genes in row names
and samples in column names.}
}
\value{
Expression matrix with normalized values
}
\description{
Quantile normalize the expression data
}
\examples{
data(zma.se)
exp <- SummarizedExperiment::assay(zma.se)
norm_exp <- q_normalize(exp)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/network_visualization.R
\name{plot_grn}
\alias{plot_grn}
\title{Plot gene regulatory network from edge list}
\usage{
plot_grn(
  edgelist_grn,
  show_labels = "tophubs",
  top_n_hubs = 5,
  layout = igraph::with_kk,
  arrow.gap = 0.01,
  ranked = TRUE,
  curvature = 0.1,
  interactive = FALSE,
  dim_interactive = c(600, 600)
)
}
\arguments{
\item{edgelist_grn}{Data frame containing the edge list for the GRN network.
First column is the TF and second column is the target gene.
All other columns are interpreted as edge attributes.}

\item{show_labels}{Character indicating which nodes will be labeled.
One of "all", "allhubs", "tophubs", or "none".}

\item{top_n_hubs}{Number of top hubs to be labeled. It is only valid
if \code{show_labels} equals "tophubs". Default is 5.}

\item{layout}{igraph function for the network layout. One of
with_dh, with_drl, with_gem, with_lgl, with_fr, with_graphopt,
with_kk and with_mds. Default is with_kk.}

\item{arrow.gap}{Numeric indicating the distance between nodes and arrows.
Default is 0.2.}

\item{ranked}{Logical indicating whether to treat third column of
the edge list (edge weights) as ranked values. Default: TRUE.}

\item{curvature}{Numeric indicating the amount of curvature in edges.
Negative values produce left-hand curves, positive values produce right-hand
curves, and zero produces a straight line. Default: 0.1.}

\item{interactive}{Logical indicating whether the network should be
interactive or not. Default is FALSE.}

\item{dim_interactive}{Numeric vector with width and height of window
for interactive plotting. Default: c(600,600).}
}
\value{
A ggplot object containing the network.
}
\description{
Plot gene regulatory network from edge list
}
\examples{
data(filt.se)
tfs <- sample(rownames(filt.se), size = 50, replace = FALSE)
grn_edges <- grn_infer(filt.se, method = "clr", regulators = tfs)
p <- plot_grn(grn_edges, ranked = FALSE)
}
\author{
Fabricio Almeida-Silva
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/grn_inference.R
\name{grn_combined}
\alias{grn_combined}
\title{Infer gene regulatory network with multiple algorithms and combine results in a list}
\usage{
grn_combined(
  exp,
  regulators = NULL,
  eps = 0.1,
  estimator_aracne = "spearman",
  estimator_clr = "pearson",
  remove_zero = TRUE,
  ...
)
}
\arguments{
\item{exp}{A gene expression data frame with genes in row names and
samples in column names or a `SummarizedExperiment` object.}

\item{regulators}{A character vector of regulators
(e.g., transcription factors or miRNAs). All regulators must be
included in `exp`.}

\item{eps}{Numeric value indicating the threshold used when removing
an edge: for each triplet of nodes (i,j,k), the weakest edge, say (ij),
is removed if its weight is below \code{min\{(ik),(jk)\}} - eps. Default: 0.1.}

\item{estimator_aracne}{Entropy estimator to be used in ARACNE inference.
One of "mi.empirical", "mi.mm", "mi.shrink", "mi.sg", "pearson", "spearman",
or "kendall". Default: "spearman".}

\item{estimator_clr}{Entropy estimator to be used in CLR inference.
One of "mi.empirical", "mi.mm", "mi.shrink", "mi.sg", "pearson", "spearman",
or "kendall". Default: "pearson".}

\item{remove_zero}{Logical indicating whether to remove edges
whose weight is exactly zero. Zero values indicate edges that were
removed by ARACNE. Default: TRUE.}

\item{...}{Additional arguments passed to `GENIE3::GENIE3()`.}
}
\value{
A list of data frames representing edge lists. Each list element
is an edge list for a specific method.
}
\description{
Infer gene regulatory network with multiple algorithms and combine results in a list
}
\examples{
data(filt.se)
tfs <- sample(rownames(filt.se), size=50, replace=FALSE)
grn_list <- grn_combined(filt.se, regulators=tfs, nTrees=2)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{zma.tfs}
\alias{zma.tfs}
\title{Maize transcription factors}
\format{
A data frame with gene IDs of TFs and their associated families.
}
\usage{
data(zma.tfs)
}
\description{
Transcription factors and their families were downloaded from PlantTFDB 4.0.
}
\examples{
data(zma.tfs)
}
\references{
Jin, J., Tian, F., Yang, D. C., Meng, Y. Q., Kong, L., Luo, J., &
Gao, G. (2016). PlantTFDB 4.0: toward a central hub for transcription
factors and regulatory interactions in plants. Nucleic acids research, gkw982.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/network_comparison.R
\name{module_preservation}
\alias{module_preservation}
\title{Calculate network preservation between two expression data sets}
\usage{
module_preservation(
  explist,
  ref_net = NULL,
  test_net = NULL,
  algorithm = "netrep",
  nPerm = 1000,
  nThreads = 1
)
}
\arguments{
\item{explist}{List of SummarizedExperiment objects or expression data frames
with genes (or orthogroups) in row names and samples in column names.}

\item{ref_net}{Reference network object returned by
the function \code{exp2gcn}.}

\item{test_net}{Test network object returned by the function \code{exp2gcn}.}

\item{algorithm}{Module preservation algorithm to be used. One of 'netrep'
(default, permutation-based) or WGCNA.}

\item{nPerm}{Number of permutations. Default: 1000}

\item{nThreads}{Number of threads to be used for parallel computing.
Default: 1}
}
\value{
A list containing the preservation statistics (netrep) or a ggplot
object with preservation statistics.
See \code{WGCNA::modulePreservation} or \code{NetRep::modulePreservation}
for more info.
}
\description{
Calculate network preservation between two expression data sets
}
\examples{
\donttest{
set.seed(1)
data(og.zma.osa)
data(zma.se)
data(osa.se)
og <- og.zma.osa
exp_ortho <- exp_genes2orthogroups(explist, og, summarize = "mean")
exp_ortho <- lapply(exp_ortho, function(x) filter_by_variance(x, n=1500))
# Previously calculated SFT powers
powers <- c(13, 15)
gcn_osa <- exp2gcn(exp_ortho$osa, net_type = "signed hybrid",
                   SFTpower = powers[1], cor_method = "pearson")
gcn_zma <- exp2gcn(exp_ortho$zma, net_type = "signed hybrid",
                   SFTpower = powers[2], cor_method = "pearson")
explist <- exp_ortho
ref_net <- gcn_osa
test_net <- gcn_zma
# 10 permutations for demonstration purposes
pres <- module_preservation(explist, ref_net, test_net, nPerm=10)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/grn_inference.R
\name{exp2grn}
\alias{exp2grn}
\title{Infer gene regulatory network from expression data}
\usage{
exp2grn(
  exp,
  regulators = NULL,
  eps = 0,
  estimator_aracne = "spearman",
  estimator_clr = "pearson",
  remove_zero = TRUE,
  nsplit = 10,
  ...
)
}
\arguments{
\item{exp}{A gene expression data frame with genes in row names and
samples in column names or a `SummarizedExperiment` object.}

\item{regulators}{A character vector of regulators
(e.g., transcription factors or miRNAs). All regulators must be
included in `exp`.}

\item{eps}{Numeric value indicating the threshold used when
removing an edge: for each triplet of nodes (i,j,k), the weakest edge,
say (ij), is removed if its weight is below \code{min\{(ik),(jk)\}} - eps. Default: 0.}

\item{estimator_aracne}{Entropy estimator to be used in ARACNE inference.
One of "mi.empirical", "mi.mm", "mi.shrink", "mi.sg", "pearson", "spearman",
or "kendall". Default: "spearman".}

\item{estimator_clr}{Entropy estimator to be used in CLR inference.
One of "mi.empirical", "mi.mm", "mi.shrink", "mi.sg", "pearson",
"spearman", or "kendall". Default: "pearson".}

\item{remove_zero}{Logical indicating whether to remove edges whose
weight is exactly zero. Zero values indicate edges that were
removed by ARACNE. Default: TRUE.}

\item{nsplit}{Number of groups in which the edge list will be split.
Default: 10.}

\item{...}{Additional arguments passed to `GENIE3::GENIE3()`.}
}
\value{
A filtered edge list with regulators in the first column and
targets in the second column.
}
\description{
Infer gene regulatory network from expression data
}
\details{
This function infers GRNs with ARACNE, GENIE3 and CLR, ranks correlation
weights for each GRN and calculates the average rank for each edge.
Then, the resulting GRN is filtered to keep the top n edges that lead
to the optimal scale-free topology fit.
}
\examples{
data(filt.se)
tfs <- sample(rownames(filt.se), size=50, replace=FALSE)
# Test with small number of trees for demonstration purpose
grn <- exp2grn(filt.se, regulators = tfs, nTrees=2, nsplit=2)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{filt.se}
\alias{filt.se}
\title{Filtered maize gene expression data from Shin et al., 2021.}
\format{
An object of class \code{SummarizedExperiment}
}
\usage{
data(filt.se)
}
\description{
Filtered expression data in transcripts per million (TPM) from
Shin et al., 2021. This is the same data set described in \code{zma.se},
but it only contains the top 500 genes with the highest variances.
This data set was created to be used in unit tests and examples.
}
\examples{
data(filt.se)
}
\references{
Shin, J., Marx, H., Richards, A., Vaneechoutte, D., Jayaraman, D., Maeda,
J., ... & Roy, S. (2021). A network-based comparative framework to study
conservation and divergence of proteomes in plant
phylogenies. Nucleic Acids Research, 49(1), e3-e3.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/network_comparison.R
\name{parse_orthofinder}
\alias{parse_orthofinder}
\title{Parse orthogroups identified by OrthoFinder}
\usage{
parse_orthofinder(file_path = NULL)
}
\arguments{
\item{file_path}{Path to Orthogroups/Orthogroups.tsv file generated
by OrthoFinder.}
}
\value{
A 3-column data frame with orthogroups, species IDs and
gene IDs, respectively.
}
\description{
This function converts the orthogroups file named \strong{Orthogroups.tsv} to
a 3-column data frame that can be interpreted by BioNERO.
}
\examples{
path <- system.file("extdata", "Orthogroups.tsv", package = "BioNERO")
og <- parse_orthofinder(path)
}
\author{
Fabricio Almeida-Silva
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/network_visualization.R
\name{plot_ppi}
\alias{plot_ppi}
\title{Plot protein-protein interaction network from edge list}
\usage{
plot_ppi(
  edgelist_int,
  color_by = "community",
  clustering_method = igraph::cluster_infomap,
  show_labels = "tophubs",
  top_n_hubs = 5,
  add_color_legend = TRUE,
  curvature = 0,
  interactive = FALSE,
  dim_interactive = c(600, 600)
)
}
\arguments{
\item{edgelist_int}{Data frame containing the edge list for the PPI network.
First column is the protein 1 and second column is the protein 2.
All other columns are interpreted as edge attributes.}

\item{color_by}{How should nodes be colored? It must be either "community" or
a 2-column data frame containing proteins in the first column and a
custom annotation in the second column. If "community", a clustering
algorithm will be applied. Default: "community".}

\item{clustering_method}{igraph function to be used for community detection.
Available functions are cluster_infomap, cluster_edge_betweenness,
cluster_fast_greedy, cluster_walktrap, cluster_spinglass,
cluster_leading_eigen, cluster_louvain, and cluster_label_prop.
Default is cluster_infomap.}

\item{show_labels}{Character indicating which nodes will be labeled.
One of "all", "allhubs", "tophubs", or "none".}

\item{top_n_hubs}{Number of top hubs to be labeled. It is only valid
if \code{show_labels} equals "tophubs". Default is 5.}

\item{add_color_legend}{Logical indicating whether to add a color legend
for nodes. Default: TRUE.}

\item{curvature}{Numeric indicating the amount of curvature in edges.
Negative values produce left-hand curves, positive values produce right-hand
curves, and zero produces a straight line. Default: 0.}

\item{interactive}{Logical indicating whether the network should be
interactive or not. Default is FALSE.}

\item{dim_interactive}{Numeric vector with width and height of window
for interactive plotting. Default: c(600,600).}
}
\value{
A ggplot object.
}
\description{
Plot protein-protein interaction network from edge list
}
\examples{
ppi_edges <- igraph::sample_pa(n = 500)
ppi_edges <- igraph::as_edgelist(ppi_edges)
p <- plot_ppi(ppi_edges, add_color_legend = FALSE)
}
\author{
Fabricio Almeida-Silva
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gcn_inference.R
\name{get_neighbors}
\alias{get_neighbors}
\title{Get 1st-order neighbors of a given gene or group of genes}
\usage{
get_neighbors(genes, net, cor_threshold = 0.7)
}
\arguments{
\item{genes}{Character vector containing genes from which
direct neighbors will be extracted.}

\item{net}{List object returned by \code{exp2gcn}.}

\item{cor_threshold}{Correlation threshold to filter connections.
As a weighted network is a fully connected graph, a cutoff must be selected.
Default is 0.7.}
}
\value{
List containing 1st-order neighbors for each input gene.
}
\description{
Get 1st-order neighbors of a given gene or group of genes
}
\examples{
data(filt.se)
genes <- rownames(filt.se)[1:10]
gcn <- exp2gcn(filt.se, SFTpower = 18, cor_method = "pearson")
neighbors <- get_neighbors(genes, gcn)
}
\seealso{
\code{exp2gcn} \code{SFT_fit}
}
\author{
Fabricio Almeida-Silva
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/grn_inference.R
\name{grn_infer}
\alias{grn_infer}
\title{Infer gene regulatory network with one of three algorithms}
\usage{
grn_infer(
  exp,
  regulators = NULL,
  method = c("clr", "aracne", "genie3"),
  estimator_clr = "pearson",
  estimator_aracne = "spearman",
  eps = 0.1,
  remove_zero = TRUE,
  ...
)
}
\arguments{
\item{exp}{A gene expression data frame with genes in row names and
samples in column names or a `SummarizedExperiment` object.}

\item{regulators}{A character vector of regulators
(e.g., transcription factors or miRNAs). All regulators must be
included in `exp`.}

\item{method}{GRN inference algorithm to be used. One of "clr", "aracne",
or "genie3".}

\item{estimator_clr}{Entropy estimator to be used. One of "mi.empirical",
"mi.mm", "mi.shrink", "mi.sg", "pearson", "spearman", or "kendall".
Default: "pearson".}

\item{estimator_aracne}{Entropy estimator to be used. One of "mi.empirical",
"mi.mm", "mi.shrink", "mi.sg", "pearson", "spearman", or "kendall".
Default: "spearman".}

\item{eps}{Numeric value indicating the threshold used when removing
an edge: for each triplet of nodes (i,j,k), the weakest edge, say (ij),
is removed if its weight is below \code{min\{(ik),(jk)\}} - eps. Default: 0.1.}

\item{remove_zero}{Logical indicating whether to remove edges whose weight
is exactly zero. Default: TRUE}

\item{...}{Additional arguments passed to `GENIE3::GENIE3()`.}
}
\value{
A gene regulatory network represented as an edge list.
}
\description{
The available algorithms are Context Likelihood of Relatedness (CLR),
ARACNE, or GENIE3.
}
\examples{
data(filt.se)
tfs <- sample(rownames(filt.se), size=20, replace=FALSE)
clr <- grn_infer(filt.se, method = "clr", regulators=tfs)
aracne <- grn_infer(filt.se, method = "aracne", regulators=tfs)
# only 2 trees for demonstration purposes
genie3 <- grn_infer(filt.se, method = "genie3", regulators=tfs, nTrees=2)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/network_comparison.R
\name{modPres_netrep}
\alias{modPres_netrep}
\title{Calculate module preservation between two expression data sets using NetRep's algorithm}
\usage{
modPres_netrep(
  explist,
  ref_net = NULL,
  test_net = NULL,
  nPerm = 1000,
  nThreads = 1
)
}
\arguments{
\item{explist}{List of expression data frames or SummarizedExperiment objects.}

\item{ref_net}{Reference network object returned by the function \code{exp2net}.}

\item{test_net}{Test network object returned by the function \code{exp2net}.}

\item{nPerm}{Number of permutations. Default: 1000}

\item{nThreads}{Number of threads to be used for parallel computing.
Default: 1}
}
\value{
Output list from \code{NetRep::modulePreservation} and a message in
user's standard output stating which modules are preserved.
}
\description{
Calculate module preservation between two expression data sets using NetRep's algorithm
}
\examples{
\donttest{
set.seed(1)
data(og.zma.osa)
data(zma.se)
data(osa.se)
og <- og.zma.osa
exp_ortho <- exp_genes2orthogroups(explist, og, summarize = "mean")
exp_ortho <- lapply(exp_ortho, function(x) filter_by_variance(x, n=1500))
# Previously calculated SFT powers
powers <- c(13, 15)
gcn_osa <- exp2gcn(exp_ortho$osa, net_type = "signed hybrid",
                   SFTpower = powers[1], cor_method = "pearson")
gcn_zma <- exp2gcn(exp_ortho$zma, net_type = "signed hybrid",
                   SFTpower = powers[2], cor_method = "pearson")
explist <- exp_ortho
ref_net <- gcn_osa
test_net <- gcn_zma
# 10 permutations for demonstration purposes
pres_netrep <- modPres_netrep(explist, ref_net, test_net,
                              nPerm=10, nThreads = 2)
}

}
\seealso{
\code{\link[NetRep]{modulePreservation}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gcn_inference.R
\name{exp2gcn_blockwise}
\alias{exp2gcn_blockwise}
\title{Infer gene coexpression network from gene expression in a blockwise manner}
\usage{
exp2gcn_blockwise(
  exp,
  net_type = "signed",
  module_merging_threshold = 0.8,
  SFTpower = NULL,
  cor_method = "pearson",
  TOM_type = NULL,
  max_block_size = 5000,
  min_module_size = 30,
  ...
)
}
\arguments{
\item{exp}{Either a `SummarizedExperiment` object, or a gene expression
matrix/data frame with genes in row names and samples in column names.}

\item{net_type}{Character indicating the type of network to infer.
One of 'signed', 'signed hybrid' or 'unsigned'. Default: 'signed'.}

\item{module_merging_threshold}{Numeric indicating the minimum correlation
threshold to merge similar modules into a single one. Default: 0.8.}

\item{SFTpower}{Numeric scalar indicating the value of the \eqn{\beta}
power to which correlation coefficients will be raised to ensure
scale-free topology fit. This value can be obtained with
the function \code{SFT_fit()}.}

\item{cor_method}{Character with correlation method to use.
One of "pearson" or "biweight". Default: "pearson".}

\item{TOM_type}{Character specifying the method to use to calculate a
topological overlap matrix (TOM). If NULL, TOM type will be automatically
inferred from network type specified in \strong{net_type}. Default: NULL.}

\item{max_block_size}{Numeric indicating the maximum block size for module
detection.}

\item{min_module_size}{Numeric indicating the minimum module size.
Default: 30.}

\item{...}{Additional arguments to \code{WGCNA::blockwiseModules()}.}
}
\value{
A list containing the following elements: \itemize{
  \item \emph{MEs} Data frame of module eigengenes, with samples
  in rows, and module eigengenes in columns.
  \item \emph{genes_and_modules} Data frame with columns 'Genes' (character)
  and 'Modules' (character) indicating the genes and the modules to
  which they belong.
  \item \emph{params} List with network inference parameters passed
  as input.
  \item \emph{dendro_plot_objects} List with objects to plot the dendrogram
  in \code{plot_dendro_and_colors}. Elements are named 'tree' (an hclust
  object with gene dendrogram), 'Unmerged' (character with per-gene module
  assignments before merging similar modules), and 'Merged' (character
  with per-gene module assignments after merging similar modules).
}
}
\description{
Infer gene coexpression network from gene expression in a blockwise manner
}
\examples{
data(filt.se)
# The SFT fit was previously calculated and the optimal power was 16
cor <- WGCNA::cor
gcn <- exp2gcn_blockwise(
    exp = filt.se, SFTpower = 18, cor_method = "pearson"
)
}
\author{
Fabricio Almeida-Silva
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/grn_inference.R
\name{grn_filter}
\alias{grn_filter}
\title{Filter a gene regulatory network based on optimal scale-free topology fit}
\usage{
grn_filter(edgelist, nsplit = 10, bp_param = BiocParallel::SerialParam())
}
\arguments{
\item{edgelist}{A gene regulatory network represented as an edge list.}

\item{nsplit}{Number of groups in which the edge list will be split.
Default: 10.}

\item{bp_param}{BiocParallel back-end to be used.
Default: BiocParallel::SerialParam()}
}
\value{
The edge list that best fits the scale-free topology.
}
\description{
Filter a gene regulatory network based on optimal scale-free topology fit
}
\details{
The edge list will be split in n groups and the scale-free
topology fit will be tested for each subset of the edge list.
For instance, if an edge list of 10000 rows is used as input,
the function will test SFT fit for the top 1000 edges, then top 2000 edges,
and so on up to the whole edge list.
}
\examples{
data(filt.se)
tfs <- sample(rownames(filt.se), size=50, replace=FALSE)
grn_list <- grn_combined(filt.se, regulators=tfs, nTrees=2)
ranked_grn <- grn_average_rank(grn_list)
# split in only 2 groups for demonstration purposes
filtered_edges <- grn_filter(ranked_grn, nsplit=2)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_preprocess.R
\name{filter_by_variance}
\alias{filter_by_variance}
\title{Keep only genes with the highest variances}
\usage{
filter_by_variance(exp, n = NULL, percentile = NULL)
}
\arguments{
\item{exp}{A gene expression data frame with genes in row names
and samples in column names or a `SummarizedExperiment` object.}

\item{n}{Number of most variable genes (e.g., n=5000 will
keep the top 5000 most variable genes).}

\item{percentile}{Percentile of most highly variable genes
(e.g., percentile=0.1 will keep the top 10 percent most variable genes).
Values must range from 0 to 1.}
}
\value{
Expression data frame or `SummarizedExperiment` object with
the most variable genes in row names and samples in column names.
}
\description{
Keep only genes with the highest variances
}
\examples{
data(zma.se)
filt_exp <- filter_by_variance(zma.se, p=0.1)
}
\author{
Fabricio Almeida-Silva
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/exploratory_analysis.R
\name{plot_heatmap}
\alias{plot_heatmap}
\title{Plot heatmap of hierarchically clustered sample correlations or gene expression}
\usage{
plot_heatmap(
  exp,
  col_metadata = NA,
  row_metadata = NA,
  coldata_cols = NULL,
  rowdata_cols = NULL,
  type = "samplecor",
  cor_method = "spearman",
  palette = NULL,
  log_trans = FALSE,
  ...
)
}
\arguments{
\item{exp}{A gene expression data frame with genes in row names
and samples in column names or a `SummarizedExperiment` object.}

\item{col_metadata}{A data frame containing sample names in row names and
sample annotation in the subsequent columns. The maximum number of columns
is 3 to ensure legends can be visualized. Ignored if `exp` is
a `SummarizedExperiment` object, since the function will extract colData.
Default: NA.}

\item{row_metadata}{A data frame containing gene IDs in row names and
gene functional classification in the first column. The maximum number
of columns is 3 to ensure legends can be visualized. Default: NA.}

\item{coldata_cols}{A vector (either numeric or character) indicating
which columns should be extracted from column metadata if \strong{exp}
is a `SummarizedExperiment` object. The vector can contain column
indices (numeric) or column names (character). By default, all columns are
used.}

\item{rowdata_cols}{A vector (either numeric or character) indicating
which columns should be extracted from row metadata if \strong{exp}
is a `SummarizedExperiment` object. The vector can contain column
indices (numeric) or column names (character). By default, all columns are
used.}

\item{type}{Type of heatmap to plot. One of 'samplecor' (sample correlations)
or 'expr'. Default: 'samplecor'.}

\item{cor_method}{Correlation method to use in
case \strong{type} is "samplecor". One of 'spearman' or 'pearson'.
Default is 'spearman'.}

\item{palette}{RColorBrewer palette to use. Default is "Blues" for sample
correlation heatmaps and "YlOrRd" for gene expression heatmaps.}

\item{log_trans}{Logical indicating whether to log transform the expression
data or not. Default: FALSE.}

\item{...}{Additional arguments to be passed
to \code{ComplexHeatmap::pheatmap()}. These arguments can be used to control
heatmap aesthetics, such as show/hide row and column names,
change font size, activate/deactivate hierarchical clustering, etc. For a
complete list of the options, see \code{?ComplexHeatmap::pheatmap()}.}
}
\value{
A heatmap of sample correlations or gene expression.
}
\description{
Plot heatmap of hierarchically clustered sample correlations or gene expression
}
\examples{
\donttest{
data(filt.se)
plot_heatmap(filt.se)
}
}
\seealso{
\code{\link[RColorBrewer]{RColorBrewer}}
}
\author{
Fabricio Almeida-Silva
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gcn_inference.R
\name{get_hubs_gcn}
\alias{get_hubs_gcn}
\title{Get GCN hubs}
\usage{
get_hubs_gcn(exp, net)
}
\arguments{
\item{exp}{A gene expression data frame with genes in row names and
samples in column names or a `SummarizedExperiment` object.}

\item{net}{List object returned by \code{exp2gcn}.}
}
\value{
Data frame containing gene IDs, modules and
intramodular connectivity of all hubs.
}
\description{
Get GCN hubs
}
\examples{
data(filt.se)
gcn <- exp2gcn(filt.se, SFTpower = 18, cor_method = "pearson")
hubs <- get_hubs_gcn(filt.se, gcn)
}
\seealso{
\code{\link[WGCNA]{signedKME}}
}
\author{
Fabricio Almeida-Silva
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{zma.se}
\alias{zma.se}
\title{Maize gene expression data from Shin et al., 2021.}
\format{
An object of class \code{SummarizedExperiment}
}
\usage{
data(zma.se)
}
\description{
Filtered expression data in transcripts per million (TPM) from
Shin et al., 2021. Genes with TPM values <5 in more than 60% of the samples
were removed to reduce package size. The expression data and
associated sample metadata are stored in a SummarizedExperiment object.
}
\examples{
data(zma.se)
}
\references{
Shin, J., Marx, H., Richards, A., Vaneechoutte, D., Jayaraman,
D., Maeda, J., ... & Roy, S. (2021). A network-based comparative framework
to study conservation and divergence of proteomes in plant
phylogenies. Nucleic Acids Research, 49(1), e3-e3.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{check_SFT}
\alias{check_SFT}
\title{Check scale-free topology fit for a given network}
\usage{
check_SFT(edgelist, net_type = "gcn")
}
\arguments{
\item{edgelist}{Edge list as a data frame containing node 1,
node 2 and edge weight.}

\item{net_type}{Type of biological network. One of "gcn", "grn", or "ppi".
Default: gcn.}
}
\value{
A list with SFT fit statistics and a message indicating if
the network is scale-free.
}
\description{
Check scale-free topology fit for a given network
}
\examples{
set.seed(1)
exp <- t(matrix(rnorm(10000), ncol=1000, nrow=200))
rownames(exp) <- paste0("Gene", 1:nrow(exp))
colnames(exp) <- paste0("Sample", 1:ncol(exp))
cormat <- cor(t(exp))
edges <- cormat_to_edgelist(cormat)
edges <- edges[abs(edges$Weight) > 0.10, ]
check_SFT(edges)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gcn_inference.R
\name{get_edge_list}
\alias{get_edge_list}
\title{Get edge list from an adjacency matrix for a group of genes}
\usage{
get_edge_list(
  net,
  genes = NULL,
  module = NULL,
  filter = FALSE,
  method = "optimalSFT",
  r_optimal_test = seq(0.4, 0.9, by = 0.1),
  Zcutoff = 1.96,
  pvalue_cutoff = 0.05,
  rcutoff = 0.7,
  nSamples = NULL,
  check_SFT = FALSE,
  bp_param = BiocParallel::SerialParam()
)
}
\arguments{
\item{net}{List object returned by \code{exp2gcn}.}

\item{genes}{Character vector containing a subset of genes from which
edges will be extracted. It can be ignored if the user wants to extract
an edge list for a given module instead of individual genes.}

\item{module}{Character with module name from which edges will be extracted.
To include 2 or more modules, input the names in a character vector.}

\item{filter}{Logical indicating whether to filter the edge list or not.}

\item{method}{Method to filter spurious correlations. One of "Zscore",
"optimalSFT", "pvalue" or "min_cor". See details for more information
on the methods. Default: 'optimalSFT'}

\item{r_optimal_test}{Numeric vector with the correlation thresholds
to be tested for optimal scale-free topology fit. Only valid
if \code{method} equals "optimalSFT". Default: seq(0.4, 0.9, by = 0.1)}

\item{Zcutoff}{Minimum Z-score threshold. Only valid if
\code{method} equals "Zscore". Default: 1.96}

\item{pvalue_cutoff}{Maximum P-value threshold. Only valid
if \code{method} equals "pvalue". Default: 0.05}

\item{rcutoff}{Minimum correlation threshold. Only valid
if \code{method} equals "min_cor". Default: 0.7}

\item{nSamples}{Number of samples in the data set from which
the correlation matrix was calculated. Only required
if \code{method} equals "pvalue".}

\item{check_SFT}{Logical indicating whether to test if the resulting network
is close to a scale-free topology or not. Default: FALSE.}

\item{bp_param}{BiocParallel back-end to be used.
Default: BiocParallel::SerialParam()}
}
\value{
Data frame with edge list for the input genes.
}
\description{
Get edge list from an adjacency matrix for a group of genes
}
\details{
The default method ("optimalSFT") will create several different
edge lists by filtering the original correlation matrix by the thresholds
specified in \code{r_optimal_test}. Then, it will calculate a scale-free
topology fit index for each of the possible networks and return the network
that best fits the scale-free topology.
The method "Zscore" will apply a Fisher Z-transformation for the correlation
coefficients and remove the Z-scores below the threshold specified
in \code{Zcutoff}.
The method "pvalue" will calculate Student asymptotic p-value for the
correlations and remove correlations whose p-values are above the threshold
specified in \code{pvalue_cutoff}.
The method "min_cor" will remove correlations below the minimum correlation
threshold specified in \code{rcutoff}.
}
\examples{
data(filt.se)
gcn <- exp2gcn(filt.se, SFTpower = 18, cor_method = "pearson")
genes <- rownames(filt.se)[1:50]
edges <- get_edge_list(gcn, genes=genes, filter = FALSE)
}
\seealso{
\code{\link[WGCNA]{scaleFreeFitIndex}}

\code{SFT_fit}

\code{exp2gcn}.
}
\author{
Fabricio Almeida-Silva
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/grn_inference.R
\name{get_hubs_grn}
\alias{get_hubs_grn}
\alias{get_hubs_ppi}
\title{Get hubs for gene regulatory network}
\usage{
get_hubs_grn(
  edgelist,
  top_percentile = 0.1,
  top_n = NULL,
  return_degree = FALSE,
  ranked = TRUE
)

get_hubs_ppi(
  edgelist,
  top_percentile = 0.1,
  top_n = NULL,
  return_degree = FALSE
)
}
\arguments{
\item{edgelist}{A protein-protein interaction network represented
as an edge list.}

\item{top_percentile}{Numeric from 0 to 1 indicating the percentage of
proteins with the highest degree to consider hubs. Default: 0.1.}

\item{top_n}{Numeric indicating the number of proteins with the highest
degree to consider hubs.}

\item{return_degree}{Logical indicating whether to return a data frame
of degree for all proteins. If TRUE, the function will return a list
instead of a data frame. Default: FALSE.}

\item{ranked}{Logical indicating whether to treat third column of the
edge list (edge weights) as ranked values. Ignored if the edge list only
contains 2 columns. Default: TRUE.}
}
\value{
A data frame with gene ID in the first column and out degree in
the second column or a list of two data frames with hubs and degree for
all genes, respectively.

A data frame with protein ID in the first column and degree
in the second column or a list of two data frames with hubs and degree
for all genes, respectively.
}
\description{
Get hubs for gene regulatory network

Get hubs for protein-protein interaction network
}
\examples{
data(filt.se)
tfs <- sample(rownames(filt.se), size=50, replace=FALSE)
grn_list <- grn_combined(filt.se, regulators=tfs, nTrees=2)
ranked_grn <- grn_average_rank(grn_list)
# split in only 2 groups for demonstration purposes
filtered_edges <- grn_filter(ranked_grn, nsplit=2)
hubs <- get_hubs_grn(filtered_edges)
ppi_edges <- igraph::sample_pa(n = 500)
ppi_edges <- igraph::as_edgelist(ppi_edges)
hubs <- get_hubs_ppi(ppi_edges, return_degree = TRUE)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_preprocess.R
\name{remove_nonexp}
\alias{remove_nonexp}
\title{Remove genes that are not expressed based on a user-defined threshold}
\usage{
remove_nonexp(
  exp,
  method = "median",
  min_exp = 1,
  min_percentage_samples = 0.25
)
}
\arguments{
\item{exp}{A gene expression data frame with genes in row names
and samples in column names or a `SummarizedExperiment` object.}

\item{method}{Criterion to filter non-expressed genes out.
One of "mean", "median", "percentage", or "allsamples". Default is "median".}

\item{min_exp}{If method is 'mean', 'median', or 'allsamples',
the minimum value for a gene to be considered expressed.
If method is 'percentage', the minimum value each gene must have in
at least n percent of samples to be considered expressed.}

\item{min_percentage_samples}{In case the user chooses 'percentage' as method,
expressed genes must have expression >= min_exp in at least this percentage.
Values must range from 0 to 1.}
}
\value{
Filtered gene expression data frame or `SummarizedExperiment` object.
}
\description{
Remove genes that are not expressed based on a user-defined threshold
}
\examples{
data(zma.se)
filt_exp <- remove_nonexp(zma.se, min_exp = 5)
}
\seealso{
\code{\link[matrixStats]{rowMedians}}
 \code{\link[WGCNA]{goodSamplesGenes}}
}
\author{
Fabricio Almeida-Silva
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/network_comparison.R
\name{exp_genes2orthogroups}
\alias{exp_genes2orthogroups}
\title{Collapse gene-level expression data to orthogroup level}
\usage{
exp_genes2orthogroups(explist = NULL, og = NULL, summarize = "median")
}
\arguments{
\item{explist}{List of expression data frames or SummarizedExperiment objects.}

\item{og}{Data frame of 3 columns corresponding to orthogroup,
species ID, and gene ID, respectively.
Species IDs must be the same as the names of the expression list.}

\item{summarize}{Centrality measure to summarize multiple paralogous genes
in the same orthogroup.
One of "median" or "mean". Default: "median".}
}
\value{
List of expression data frames for each species with expression
summarized at the orthogroup level.
}
\description{
For a given list of expression data, this function replaces genes with
their corresponding orthogroups to allow inter-species comparisons.
}
\examples{
\donttest{
data(og.zma.osa)
data(zma.se)
data(osa.se)
explist <- list(zma = zma.se,
                osa = osa.se)
og <- og.zma.osa
exp_ortho <- exp_genes2orthogroups(explist, og, summarize = "mean")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/exploratory_analysis.R
\name{get_HK}
\alias{get_HK}
\title{Get housekeeping genes from global expression profile}
\usage{
get_HK(exp)
}
\arguments{
\item{exp}{A gene expression data frame with genes in row names and
samples in column names or a `SummarizedExperiment` object.}
}
\value{
Character vector of housekeeping gene IDs.
}
\description{
Get housekeeping genes from global expression profile
}
\details{
This function identifies housekeeping genes, which are
broadly expressed genes with low variation in a global scale across samples.
For some cases, users would want to remove these genes as they are not
interesting for coexpression network analyses.
See references for more details.
}
\examples{
data(zma.se)
hk <- get_HK(zma.se)
}
\references{
Machado, F.B., Moharana, K.C., Almeida‐Silva, F., Gazara, R.K.,
Pedrosa‐Silva, F., Coelho, F.S., Grativol, C. and Venancio, T.M. (2020),
Systematic analysis of 1298 RNA‐Seq samples and construction of a
comprehensive soybean (Glycine max) expression atlas. Plant J, 103: 1894-1909.
}
\author{
Fabricio Almeida-Silva
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gcn_inference.R
\name{module_enrichment}
\alias{module_enrichment}
\title{Perform enrichment analysis for coexpression network modules}
\usage{
module_enrichment(
  net = NULL,
  background_genes,
  annotation,
  column = NULL,
  correction = "BH",
  p = 0.05,
  min_setsize = 10,
  max_setsize = 500,
  bp_param = BiocParallel::SerialParam()
)
}
\arguments{
\item{net}{List object returned by \code{exp2gcn}.}

\item{background_genes}{Character vector of genes to be used as background
for the Fisher's Exact Test.}

\item{annotation}{Annotation data frame with genes in the first column
and functional annotation in the other columns. This data frame can
be exported from Biomart or similar databases.}

\item{column}{Column or columns of \code{annotation} to be used for enrichment.
Both character or numeric values with column indices can be used.
If users want to supply more than one column, input a character or
numeric vector. Default: all columns from \code{annotation}.}

\item{correction}{Multiple testing correction method. One of "holm",
"hochberg", "hommel", "bonferroni", "BH", "BY", "fdr" or "none".
Default is "BH".}

\item{p}{P-value threshold. P-values below this threshold will be considered
significant. Default is 0.05.}

\item{min_setsize}{Numeric indicating the minimum gene set size to be
considered. Gene sets correspond to levels of each variable
in \strong{annotation}). Default: 10.}

\item{max_setsize}{Numeric indicating the maximum gene set size to be
considered. Gene sets correspond to levels of each variable
in \strong{annotation}). Default: 500.}

\item{bp_param}{BiocParallel back-end to be used.
Default: BiocParallel::SerialParam()}
}
\value{
A data frame of overrepresentation results with the following
variables:
\describe{
  \item{term}{character, functional term ID/name.}
  \item{genes}{numeric, intersection length between input genes and genes
               in a particular functional term.}
  \item{all}{numeric, number of all genes in a particular functional term.}
  \item{pval}{numeric, P-value for the hypergeometric test.}
  \item{padj}{numeric, P-value adjusted for multiple comparisons using
              the method specified in parameter \strong{adj}.}
  \item{category}{character, name of the grouping variable (i.e., column
                  name of \strong{annotation}).}
  \item{module}{character, module name.}
}
}
\description{
Perform enrichment analysis for coexpression network modules
}
\examples{
\donttest{
data(filt.se)
data(zma.interpro)
background <- rownames(filt.se)
gcn <- exp2gcn(filt.se, SFTpower = 18, cor_method = "pearson")
mod_enrich <- module_enrichment(gcn, background, zma.interpro, p=1)
}
}
\author{
Fabricio Almeida-Silva
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_preprocess.R
\name{PC_correction}
\alias{PC_correction}
\title{Apply Principal Component (PC)-based correction for confounding artifacts}
\usage{
PC_correction(exp, verbose = FALSE)
}
\arguments{
\item{exp}{A gene expression data frame with genes in row names
and samples in column names or a `SummarizedExperiment` object.}

\item{verbose}{Logical indicating whether to display progress
messages or not. Default: FALSE.}
}
\value{
Corrected expression data frame or `SummarizedExperiment` object.
}
\description{
Apply Principal Component (PC)-based correction for confounding artifacts
}
\examples{
data(zma.se)
exp <- filter_by_variance(zma.se, n=500)
exp <- PC_correction(exp)
}
\references{
Parsana, P., Ruberman, C., Jaffe, A. E., Schatz, M. C., Battle, A., &
Leek, J. T. (2019). Addressing confounding artifacts in reconstruction of
gene co-expression networks. Genome biology, 20(1), 1-6.
}
\seealso{
\code{\link[sva]{num.sv}},\code{\link[sva]{sva_network}}
}
\author{
Fabricio Almeida-Silva
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/grn_inference.R
\name{grn_average_rank}
\alias{grn_average_rank}
\title{Rank edge weights for GRNs and calculate average across different methods}
\usage{
grn_average_rank(list_edges)
}
\arguments{
\item{list_edges}{List containing edge lists as returned by
the function \code{grn_combined}.}
}
\value{
Edge list containing regulator, target and mean rank from
all algorithms.
}
\description{
Rank edge weights for GRNs and calculate average across different methods
}
\examples{
data(filt.se)
tfs <- sample(rownames(filt.se), size=50, replace=FALSE)
grn_list <- grn_combined(filt.se, regulators=tfs, nTrees=2)
ranked_grn <- grn_average_rank(grn_list)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{cormat_to_edgelist}
\alias{cormat_to_edgelist}
\title{Transform a correlation matrix to an edge list}
\usage{
cormat_to_edgelist(matrix)
}
\arguments{
\item{matrix}{Symmetrical correlation matrix.}
}
\value{
A 2-column data frame containing node 1, node 2 and edge weight.
}
\description{
Transform a correlation matrix to an edge list
}
\examples{
data(filt.se)
cor_mat <- cor(t(SummarizedExperiment::assay(filt.se)))
edgelist <- cormat_to_edgelist(cor_mat)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/evolutionary_analysis.R
\name{is_singleton}
\alias{is_singleton}
\title{Logical expression to check if gene or gene set is singleton or not}
\usage{
is_singleton(genes, og)
}
\arguments{
\item{genes}{Character containing gene or group of genes to be evaluated.}

\item{og}{Data frame of 3 columns corresponding to orthogroup, species ID,
and gene ID, respectively.}
}
\value{
Vector of logical values indicating if gene or group of genes
is singleton or not.
}
\description{
Logical expression to check if gene or gene set is singleton or not
}
\examples{
data(og.zma.osa)
data(filt.se)
genes <- tail(rownames(filt.se), n = 100)
is_singleton(genes, og.zma.osa)
}
\seealso{
\code{is_duplicated}
}
\author{
Fabricio Almeida-Silva
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{cor2adj}
\alias{cor2adj}
\title{Calculate an adjacency matrix from a correlation matrix}
\usage{
cor2adj(cor_matrix, beta, net_type = "signed hybrid")
}
\arguments{
\item{cor_matrix}{A numeric, symmetric matrix with pairwise correlations
between genes (i.e., a 'correlation matrix').}

\item{beta}{Numeric scalar indicating the value of the \eqn{\beta}
power to which correlation coefficients will be raised to ensure
scale-free topology fit.}

\item{net_type}{Character indicating the type of network to infer.
Default: "signed hybrid".}
}
\value{
A numeric, symmetric matrix with network adjacency values between
genes.
}
\description{
Calculate an adjacency matrix from a correlation matrix
}
\examples{
# Simulate an expression matrix with 100 genes and 50 samples
exp <- matrix(
    rnorm(100 * 50, mean = 10, sd = 2),
    nrow = 100,
    dimnames = list(
        paste0("gene", seq_len(100)),
        paste0("sample", seq_len(50))
    )
)

# Calculate correlation matrix
cor_mat <- exp2cor(exp)

# Calculate adjacency matrix (random value for beta)
adj <- cor2adj(cor_mat, beta = 9)
}
